-- structure et données minimales de la base de donnée superadmin
--
--
-- i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
--
-- @author Stéphane Sampaio
-- @copyright Adullact Projet
-- @link http://adullact.org/
-- @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
--
-- SVN Informations
-- $Date: 2014-12-17 18:59:13 +0100 (mer. 17 déc. 2014) $
-- $Revision: 561 $
-- $Author: rdubourget $
-- $HeadURL: svn+ssh://rdubourget@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/Config/sql/101_idelibre_coll_1.sql $
-- $Id: 101_idelibre_coll_1.sql 561 2014-12-17 17:59:13Z rdubourget $

--
-- PostgreSQL database dump
--

SET statement_timeout = 0;
SET client_encoding = 'UTF8';
SET standard_conforming_strings = on;
SET check_function_bodies = false;
SET client_min_messages = warning;

--
-- Name: plpgsql; Type: EXTENSION; Schema: -; Owner: 
--

CREATE EXTENSION IF NOT EXISTS plpgsql WITH SCHEMA pg_catalog;


--
-- Name: EXTENSION plpgsql; Type: COMMENT; Schema: -; Owner: 
--

COMMENT ON EXTENSION plpgsql IS 'PL/pgSQL procedural language';


--
-- Name: uuid-ossp; Type: EXTENSION; Schema: -; Owner: 
--

CREATE EXTENSION IF NOT EXISTS "uuid-ossp" WITH SCHEMA public;


--
-- Name: EXTENSION "uuid-ossp"; Type: COMMENT; Schema: -; Owner: 
--

COMMENT ON EXTENSION "uuid-ossp" IS 'generate universally unique identifiers (UUIDs)';


SET search_path = public, pg_catalog;

--
-- Name: rebuilt_sequences(); Type: FUNCTION; Schema: public; Owner: idelibre
--

CREATE FUNCTION rebuilt_sequences() RETURNS integer
    LANGUAGE plpgsql
    AS $$
  DECLARE sequencedefs RECORD; c integer ;
  BEGIN
    FOR sequencedefs IN Select
      constraint_column_usage.table_name as tablename,
      constraint_column_usage.table_name as tablename,
      constraint_column_usage.column_name as columnname,
      replace(replace(columns.column_default,'''::regclass)',''),'nextval(''','') as sequencename
      from information_schema.constraint_column_usage, information_schema.columns
      where constraint_column_usage.table_schema ='public' AND
      columns.table_schema = 'public' AND columns.table_name=constraint_column_usage.table_name
      AND constraint_column_usage.column_name = columns.column_name
      AND columns.column_default is not null
   LOOP
      EXECUTE 'select max('||sequencedefs.columnname||') from ' || sequencedefs.tablename INTO c;
      IF c is null THEN c = 0; END IF;
      IF c is not null THEN c = c+ 1; END IF;
      EXECUTE 'alter sequence ' || sequencedefs.sequencename ||' restart  with ' || c;
   END LOOP;

   RETURN 1; END;
$$;


ALTER FUNCTION public.rebuilt_sequences() OWNER TO idelibre;

SET default_tablespace = '';

SET default_with_oids = false;

--
-- Name: acos; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE acos (
    id integer NOT NULL,
    parent_id integer,
    model character varying(255) DEFAULT NULL::character varying,
    alias character varying(255) DEFAULT NULL::character varying,
    lft integer,
    rght integer,
    foreign_key uuid
);


ALTER TABLE public.acos OWNER TO idelibre;

--
-- Name: TABLE acos; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE acos IS 'Gestion des ACL (CakePHP) - ressources';


--
-- Name: acos_id_seq; Type: SEQUENCE; Schema: public; Owner: idelibre
--

CREATE SEQUENCE acos_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.acos_id_seq OWNER TO idelibre;

--
-- Name: acos_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: idelibre
--

ALTER SEQUENCE acos_id_seq OWNED BY acos.id;


--
-- Name: annexes; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE annexes (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    name character varying(500) NOT NULL,
    size integer DEFAULT 0 NOT NULL,
    type character varying(255) NOT NULL,
    content bytea,
    created timestamp without time zone,
    modified timestamp without time zone,
    projet_id uuid,
    checksum character varying(32),
    rank integer DEFAULT 0 NOT NULL
);


ALTER TABLE public.annexes OWNER TO idelibre;

--
-- Name: TABLE annexes; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE annexes IS 'Gestion des annexes de projets de délibérations (pièces jointes)';


--
-- Name: annotactions; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE annotactions (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    deltarevision_id uuid,
    created timestamp without time zone,
    modified timestamp without time zone,
    type character varying(1),
    annotation_id character varying(100),
    CONSTRAINT annotactions_type_in_chk CHECK (((type)::text = ANY (ARRAY[('C'::character varying)::text, ('U'::character varying)::text, ('D'::character varying)::text])))
);


ALTER TABLE public.annotactions OWNER TO idelibre;

--
-- Name: annotations; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE annotations (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    annot_client_id uuid,
    imgpage_id uuid NOT NULL,
    content bytea NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    user_id uuid NOT NULL,
    shared boolean DEFAULT false NOT NULL,
    rank integer DEFAULT 0 NOT NULL,
    date timestamp without time zone,
    date_modif timestamp without time zone,
    height character varying(20),
    width character varying(20),
    x character varying(20),
    y character varying(20),
    comment character varying(255)
);


ALTER TABLE public.annotations OWNER TO idelibre;

--
-- Name: annotations_users; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE annotations_users (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    user_id uuid NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    annotation_id uuid NOT NULL,
    read boolean DEFAULT false NOT NULL
);


ALTER TABLE public.annotations_users OWNER TO idelibre;

--
-- Name: aros; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE aros (
    id integer NOT NULL,
    parent_id integer,
    model character varying(255) DEFAULT NULL::character varying,
    alias character varying(255) DEFAULT NULL::character varying,
    lft integer,
    rght integer,
    foreign_key uuid
);


ALTER TABLE public.aros OWNER TO idelibre;

--
-- Name: TABLE aros; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE aros IS 'Gestion des ACL (CakePHP) - demandeurs';


--
-- Name: aros_acos; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE aros_acos (
    id integer NOT NULL,
    aro_id integer NOT NULL,
    aco_id integer NOT NULL,
    _create character varying(2) DEFAULT '0'::character varying NOT NULL,
    _read character varying(2) DEFAULT '0'::character varying NOT NULL,
    _update character varying(2) DEFAULT '0'::character varying NOT NULL,
    _delete character varying(2) DEFAULT '0'::character varying NOT NULL
);


ALTER TABLE public.aros_acos OWNER TO idelibre;

--
-- Name: TABLE aros_acos; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE aros_acos IS 'Gestion des ACL (CakePHP) - liaisons demandeurs / ressources';


--
-- Name: aros_acos_id_seq; Type: SEQUENCE; Schema: public; Owner: idelibre
--

CREATE SEQUENCE aros_acos_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.aros_acos_id_seq OWNER TO idelibre;

--
-- Name: aros_acos_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: idelibre
--

ALTER SEQUENCE aros_acos_id_seq OWNED BY aros_acos.id;


--
-- Name: aros_id_seq; Type: SEQUENCE; Schema: public; Owner: idelibre
--

CREATE SEQUENCE aros_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.aros_id_seq OWNER TO idelibre;

--
-- Name: aros_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: idelibre
--

ALTER SEQUENCE aros_id_seq OWNED BY aros.id;


--
-- Name: convocations; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE convocations (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    seance_id uuid NOT NULL,
    user_id uuid NOT NULL,
    read boolean DEFAULT false NOT NULL,
    presence boolean DEFAULT true NOT NULL,
    delegation boolean DEFAULT false NOT NULL,
    procuration boolean DEFAULT false NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    ar_horodatage timestamp without time zone,
    active boolean DEFAULT false NOT NULL,
    ar_horodatage_file bytea,
    ar_horodatage_token bytea,
    ar_received timestamp without time zone,
    ae_sent timestamp without time zone,
    ae_horodatage timestamp without time zone,
    ae_horodatage_file bytea,
    ae_horodatage_token bytea
);


ALTER TABLE public.convocations OWNER TO idelibre;

--
-- Name: TABLE convocations; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE convocations IS 'Gestion des convocations';


--
-- Name: deltarevisions; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE deltarevisions (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    num integer NOT NULL,
    delta bytea NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    user_id uuid NOT NULL
);


ALTER TABLE public.deltarevisions OWNER TO idelibre;

--
-- Name: deltarevisions_num_seq; Type: SEQUENCE; Schema: public; Owner: idelibre
--

CREATE SEQUENCE deltarevisions_num_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.deltarevisions_num_seq OWNER TO idelibre;

--
-- Name: deltarevisions_num_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: idelibre
--

ALTER SEQUENCE deltarevisions_num_seq OWNED BY deltarevisions.num;


--
-- Name: documents; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE documents (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    name character varying(255) NOT NULL,
    size integer DEFAULT 0 NOT NULL,
    type character varying(255) NOT NULL,
    content bytea,
    created timestamp without time zone,
    modified timestamp without time zone,
    checksum character varying(32)
);


ALTER TABLE public.documents OWNER TO idelibre;

--
-- Name: TABLE documents; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE documents IS 'Gestion des documents';


--
-- Name: groups; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE groups (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    name character varying(255) NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    admin boolean DEFAULT false NOT NULL,
    inseance boolean DEFAULT false,
    superuser boolean DEFAULT false,
    baseuser boolean DEFAULT true
);


ALTER TABLE public.groups OWNER TO idelibre;

--
-- Name: TABLE groups; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE groups IS 'Gestion des groupes';


--
-- Name: imgpages; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE imgpages (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    name character varying(255) NOT NULL,
    size integer DEFAULT 0 NOT NULL,
    type character varying(255) NOT NULL,
    content bytea,
    imgpagenum integer DEFAULT 0 NOT NULL,
    document_id uuid NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    checksum character varying(32)
);


ALTER TABLE public.imgpages OWNER TO idelibre;

--
-- Name: TABLE imgpages; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE imgpages IS 'Gestion des pages des documents à annoter';


--
-- Name: projets; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE projets (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    name character varying(500) NOT NULL,
    theme character varying(500),
    document_id uuid,
    vote character varying(10),
    ptheme_id integer,
    seance_id uuid NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    rank integer DEFAULT 0 NOT NULL,
    CONSTRAINT projets_vote_check CHECK (((vote)::text = ANY (ARRAY[('oui'::character varying)::text, ('non'::character varying)::text, ('nsp'::character varying)::text, ('abs'::character varying)::text])))
);


ALTER TABLE public.projets OWNER TO idelibre;

--
-- Name: TABLE projets; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE projets IS 'Gestion des projets de délibérations';


--
-- Name: pthemes; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE pthemes (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    parent_id integer,
    rght integer NOT NULL,
    lft integer NOT NULL
);


ALTER TABLE public.pthemes OWNER TO idelibre;

--
-- Name: TABLE pthemes; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE pthemes IS 'Gestion des themes de projets de délibération';


--
-- Name: pthemes_id_seq; Type: SEQUENCE; Schema: public; Owner: idelibre
--

CREATE SEQUENCE pthemes_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.pthemes_id_seq OWNER TO idelibre;

--
-- Name: pthemes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: idelibre
--

ALTER SEQUENCE pthemes_id_seq OWNED BY pthemes.id;


--
-- Name: seances; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE seances (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    name character varying(255) NOT NULL,
    date_seance timestamp without time zone NOT NULL,
    rev numeric DEFAULT 1 NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    type_id uuid,
    document_id uuid NOT NULL,
    alias character varying(255) NOT NULL
);


ALTER TABLE public.seances OWNER TO idelibre;

--
-- Name: TABLE seances; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE seances IS 'Gestion des séances';


--
-- Name: types; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE types (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    name character varying(255) NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone
);


ALTER TABLE public.types OWNER TO idelibre;

--
-- Name: TABLE types; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE types IS 'Gestion des types de séances';


--
-- Name: types_users; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE types_users (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    type_id uuid NOT NULL,
    user_id uuid NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone
);


ALTER TABLE public.types_users OWNER TO idelibre;

--
-- Name: TABLE types_users; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE types_users IS 'Liaison entre les types de séances et les utilisateurs';


--
-- Name: users; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE users (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    firstname character varying(255) NOT NULL,
    lastname character varying(255) NOT NULL,
    username character varying(255) NOT NULL,
    password character varying(255) NOT NULL,
    last_login timestamp without time zone,
    group_id uuid NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    active boolean DEFAULT true NOT NULL,
    last_logout timestamp without time zone,
    last_import timestamp without time zone,
    mail character varying(255),
    pwdmodified boolean DEFAULT false NOT NULL,
    rev integer,
    rev_date timestamp without time zone,
    seance_sync_date timestamp without time zone
);


ALTER TABLE public.users OWNER TO idelibre;

--
-- Name: TABLE users; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE users IS 'Gestion des utilisateurs';


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY acos ALTER COLUMN id SET DEFAULT nextval('acos_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY aros ALTER COLUMN id SET DEFAULT nextval('aros_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY aros_acos ALTER COLUMN id SET DEFAULT nextval('aros_acos_id_seq'::regclass);


--
-- Name: num; Type: DEFAULT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY deltarevisions ALTER COLUMN num SET DEFAULT nextval('deltarevisions_num_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY pthemes ALTER COLUMN id SET DEFAULT nextval('pthemes_id_seq'::regclass);


--
-- Data for Name: acos; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY acos (id, parent_id, model, alias, lft, rght, foreign_key) FROM stdin;
103	101	\N	setConn	203	204	\N
36	33	\N	getHorodatageFile	69	70	\N
3	2	\N	getFile	3	4	\N
80	1	\N	Imgpages	158	175	\N
4	2	\N	setConn	5	6	\N
59	56	\N	params	115	116	\N
37	33	\N	getHorodatageToken	71	72	\N
5	2	\N	processLogin	7	8	\N
6	2	\N	writeLoginConfig	9	10	\N
76	68	\N	writeUserConfig	149	150	\N
38	33	\N	setConn	73	74	\N
7	2	\N	writeUserConfig	11	12	\N
8	2	\N	writeLoginSession	13	14	\N
60	56	\N	stats	117	118	\N
39	33	\N	processLogin	75	76	\N
9	2	\N	ldebug	15	16	\N
2	1	\N	Annexes	2	19	\N
10	2	\N	ldebugHr	17	18	\N
88	80	\N	ldebugHr	173	174	\N
40	33	\N	writeLoginConfig	77	78	\N
12	11	\N	getData	21	22	\N
61	56	\N	setConn	119	120	\N
13	11	\N	setConn	23	24	\N
41	33	\N	writeUserConfig	79	80	\N
14	11	\N	processLogin	25	26	\N
77	68	\N	writeLoginSession	151	152	\N
15	11	\N	writeLoginConfig	27	28	\N
42	33	\N	writeLoginSession	81	82	\N
16	11	\N	writeUserConfig	29	30	\N
62	56	\N	processLogin	121	122	\N
17	11	\N	writeLoginSession	31	32	\N
43	33	\N	ldebug	83	84	\N
18	11	\N	ldebug	33	34	\N
11	1	\N	Clients	20	37	\N
19	11	\N	ldebugHr	35	36	\N
33	1	\N	Convocations	64	87	\N
44	33	\N	ldebugHr	85	86	\N
21	20	\N	index	39	40	\N
97	89	\N	writeUserConfig	191	192	\N
22	20	\N	add	41	42	\N
63	56	\N	writeLoginConfig	123	124	\N
23	20	\N	edit	43	44	\N
46	45	\N	save_annotations	89	90	\N
24	20	\N	delete	45	46	\N
78	68	\N	ldebug	153	154	\N
25	20	\N	info	47	48	\N
47	45	\N	getFile	91	92	\N
26	20	\N	setConn	49	50	\N
64	56	\N	writeUserConfig	125	126	\N
27	20	\N	processLogin	51	52	\N
48	45	\N	getAnnotations	93	94	\N
28	20	\N	writeLoginConfig	53	54	\N
68	1	\N	Groups	134	157	\N
29	20	\N	writeUserConfig	55	56	\N
49	45	\N	setConn	95	96	\N
30	20	\N	writeLoginSession	57	58	\N
65	56	\N	writeLoginSession	127	128	\N
31	20	\N	ldebug	59	60	\N
50	45	\N	processLogin	97	98	\N
20	1	\N	Collectivites	38	63	\N
32	20	\N	ldebugHr	61	62	\N
79	68	\N	ldebugHr	155	156	\N
51	45	\N	writeLoginConfig	99	100	\N
34	33	\N	setRead	65	66	\N
66	56	\N	ldebug	129	130	\N
35	33	\N	send	67	68	\N
56	1	\N	Environnements	110	133	\N
52	45	\N	writeUserConfig	101	102	\N
67	56	\N	ldebugHr	131	132	\N
53	45	\N	writeLoginSession	103	104	\N
111	110	\N	index	219	220	\N
54	45	\N	ldebug	105	106	\N
45	1	\N	Documents	88	109	\N
55	45	\N	ldebugHr	107	108	\N
90	89	\N	index	177	178	\N
69	68	\N	index	135	136	\N
57	56	\N	index	111	112	\N
81	80	\N	getFile	159	160	\N
58	56	\N	menu	113	114	\N
98	89	\N	writeLoginSession	193	194	\N
70	68	\N	add	137	138	\N
82	80	\N	setConn	161	162	\N
71	68	\N	edit	139	140	\N
91	89	\N	start	179	180	\N
72	68	\N	delete	141	142	\N
83	80	\N	processLogin	163	164	\N
73	68	\N	setConn	143	144	\N
104	101	\N	processLogin	205	206	\N
74	68	\N	processLogin	145	146	\N
92	89	\N	stop	181	182	\N
75	68	\N	writeLoginConfig	147	148	\N
84	80	\N	writeLoginConfig	165	166	\N
99	89	\N	ldebug	195	196	\N
85	80	\N	writeUserConfig	167	168	\N
93	89	\N	display	183	184	\N
86	80	\N	writeLoginSession	169	170	\N
108	101	\N	ldebug	213	214	\N
87	80	\N	ldebug	171	172	\N
89	1	\N	Monitor	176	199	\N
100	89	\N	ldebugHr	197	198	\N
94	89	\N	setConn	185	186	\N
95	89	\N	processLogin	187	188	\N
105	101	\N	writeLoginConfig	207	208	\N
96	89	\N	writeLoginConfig	189	190	\N
115	110	\N	getList	227	228	\N
101	1	\N	Pages	200	217	\N
106	101	\N	writeUserConfig	209	210	\N
102	101	\N	display	201	202	\N
109	101	\N	ldebugHr	215	216	\N
112	110	\N	add	221	222	\N
107	101	\N	writeLoginSession	211	212	\N
114	110	\N	delete	225	226	\N
119	110	\N	writeUserConfig	235	236	\N
113	110	\N	edit	223	224	\N
118	110	\N	writeLoginConfig	233	234	\N
116	110	\N	setConn	229	230	\N
117	110	\N	processLogin	231	232	\N
120	110	\N	writeLoginSession	237	238	\N
121	110	\N	ldebug	239	240	\N
122	110	\N	ldebugHr	241	242	\N
124	123	\N	index	245	246	\N
110	1	\N	Pthemes	218	243	\N
125	123	\N	view	247	248	\N
126	123	\N	add	249	250	\N
127	123	\N	edit	251	252	\N
128	123	\N	delete	253	254	\N
129	123	\N	ajout	255	256	\N
130	123	\N	liste	257	258	\N
131	123	\N	details	259	260	\N
132	123	\N	suppression	261	262	\N
123	1	\N	Seances	244	281	\N
133	123	\N	edition	263	264	\N
192	183	\N	processLogin	381	382	\N
134	123	\N	sendConvocations	265	266	\N
135	123	\N	setConn	267	268	\N
168	154	\N	ldebug	333	334	\N
136	123	\N	processLogin	269	270	\N
137	123	\N	writeLoginConfig	271	272	\N
207	200	\N	writeLoginSession	410	411	\N
138	123	\N	writeUserConfig	273	274	\N
154	1	\N	Srvusers	306	337	\N
139	123	\N	writeLoginSession	275	276	\N
169	154	\N	ldebugHr	335	336	\N
140	123	\N	ldebug	277	278	\N
141	123	\N	ldebugHr	279	280	\N
193	183	\N	writeLoginConfig	383	384	\N
143	142	\N	index	283	284	\N
171	170	\N	index	339	340	\N
144	142	\N	add	285	286	\N
145	142	\N	edit	287	288	\N
194	183	\N	writeUserConfig	385	386	\N
172	170	\N	add	341	342	\N
146	142	\N	delete	289	290	\N
147	142	\N	setConn	291	292	\N
173	170	\N	edit	343	344	\N
148	142	\N	processLogin	293	294	\N
208	200	\N	ldebug	412	413	\N
149	142	\N	writeLoginConfig	295	296	\N
195	183	\N	writeLoginSession	387	388	\N
174	170	\N	delete	345	346	\N
150	142	\N	writeUserConfig	297	298	\N
151	142	\N	writeLoginSession	299	300	\N
175	170	\N	users	347	348	\N
152	142	\N	ldebug	301	302	\N
142	1	\N	Srvroles	282	305	\N
153	142	\N	ldebugHr	303	304	\N
1	\N	\N	controllers	1	418	\N
196	183	\N	ldebug	389	390	\N
176	170	\N	setConn	349	350	\N
155	154	\N	index	307	308	\N
199	1	\N	DebugKit	396	417	\N
156	154	\N	add	309	310	\N
177	170	\N	processLogin	351	352	\N
157	154	\N	edit	311	312	\N
183	1	\N	Users	364	393	\N
197	183	\N	ldebugHr	391	392	\N
158	154	\N	delete	313	314	\N
178	170	\N	writeLoginConfig	353	354	\N
159	154	\N	get_proof	315	316	\N
200	199	\N	ToolbarAccess	397	416	\N
160	154	\N	login	317	318	\N
179	170	\N	writeUserConfig	355	356	\N
161	154	\N	logout	319	320	\N
198	1	\N	AclExtras	394	395	\N
162	154	\N	mdplost	321	322	\N
180	170	\N	writeLoginSession	357	358	\N
163	154	\N	setConn	323	324	\N
209	200	\N	ldebugHr	414	415	\N
164	154	\N	processLogin	325	326	\N
181	170	\N	ldebug	359	360	\N
165	154	\N	writeLoginConfig	327	328	\N
170	1	\N	Types	338	363	\N
166	154	\N	writeUserConfig	329	330	\N
167	154	\N	writeLoginSession	331	332	\N
182	170	\N	ldebugHr	361	362	\N
184	183	\N	index	365	366	\N
201	200	\N	history_state	398	399	\N
185	183	\N	add	367	368	\N
186	183	\N	edit	369	370	\N
187	183	\N	delete	371	372	\N
202	200	\N	sql_explain	400	401	\N
188	183	\N	login	373	374	\N
189	183	\N	logout	375	376	\N
203	200	\N	setConn	402	403	\N
190	183	\N	mdplost	377	378	\N
191	183	\N	setConn	379	380	\N
204	200	\N	processLogin	404	405	\N
205	200	\N	writeLoginConfig	406	407	\N
206	200	\N	writeUserConfig	408	409	\N
\.


--
-- Name: acos_id_seq; Type: SEQUENCE SET; Schema: public; Owner: idelibre
--

SELECT pg_catalog.setval('acos_id_seq', 209, true);


--
-- Data for Name: annexes; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY annexes (id, name, size, type, content, created, modified, projet_id, checksum, rank) FROM stdin;
\.


--
-- Data for Name: annotactions; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY annotactions (id, deltarevision_id, created, modified, type, annotation_id) FROM stdin;
\.


--
-- Data for Name: annotations; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY annotations (id, annot_client_id, imgpage_id, content, created, modified, user_id, shared, rank, date, date_modif, height, width, x, y, comment) FROM stdin;
\.


--
-- Data for Name: annotations_users; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY annotations_users (id, user_id, created, modified, annotation_id, read) FROM stdin;
\.


--
-- Data for Name: aros; Type: TABLE DATA; Schema: public; Owner: idelibre


--4	1	User	Administrateur	2	3	e64e47b6-9027-4d73-a49b-70fd2ee0475e
--

COPY aros (id, parent_id, model, alias, lft, rght, foreign_key) FROM stdin;
1	\N	Group	Administrateur	1	4	26b380d8-93b2-41e9-9f68-da98c55ccb30
2	\N	Group	Acteur	5	6	b2f7a7e4-4ab1-4d93-ac13-d7ca540b4c16
3	\N	Group	Utilisateur	7	8	9f803280-eb21-46b3-bcec-74ffc4d2aab8
4	1	User	Administrateur	2	3	e64e47b6-9027-4d73-a49b-70fd2ee0475e
\.


--
-- Data for Name: aros_acos; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY aros_acos (id, aro_id, aco_id, _create, _read, _update, _delete) FROM stdin;
1	4	25	1	1	1	1
2	4	56	1	1	1	1
3	4	36	1	1	1	1
4	4	37	1	1	1	1
5	4	68	1	1	1	1
6	4	110	1	1	1	1
7	4	170	1	1	1	1
8	4	183	1	1	1	1
9	3	56	1	1	1	1
10	3	129	1	1	1	1
11	3	130	1	1	1	1
12	3	131	1	1	1	1
13	3	133	1	1	1	1
14	3	132	1	1	1	1
15	3	134	1	1	1	1
16	3	33	1	1	1	1
17	3	11	1	1	1	1
18	3	185	1	1	1	1
19	3	110	1	1	1	1
20	3	170	1	1	1	1
21	3	47	1	1	1	1
22	3	81	1	1	1	1
23	3	3	1	1	1	1
24	2	56	1	1	1	1
25	2	123	1	1	1	1
26	2	33	1	1	1	1
27	2	47	1	1	1	1
28	2	48	1	1	1	1
29	2	81	1	1	1	1
30	2	3	1	1	1	1
31	2	11	1	1	1	1
\.


--
-- Name: aros_acos_id_seq; Type: SEQUENCE SET; Schema: public; Owner: idelibre
--

SELECT pg_catalog.setval('aros_acos_id_seq', 31, true);


--
-- Name: aros_id_seq; Type: SEQUENCE SET; Schema: public; Owner: idelibre
--

SELECT pg_catalog.setval('aros_id_seq', 4, true);


--
-- Data for Name: convocations; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY convocations (id, seance_id, user_id, read, presence, delegation, procuration, created, modified, ar_horodatage, active, ar_horodatage_file, ar_horodatage_token, ar_received, ae_sent, ae_horodatage, ae_horodatage_file, ae_horodatage_token) FROM stdin;
\.


--
-- Data for Name: deltarevisions; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY deltarevisions (id, num, delta, created, modified, user_id) FROM stdin;
\.


--
-- Name: deltarevisions_num_seq; Type: SEQUENCE SET; Schema: public; Owner: idelibre
--

SELECT pg_catalog.setval('deltarevisions_num_seq', 1, false);


--
-- Data for Name: documents; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY documents (id, name, size, type, content, created, modified, checksum) FROM stdin;
\.


--
-- Data for Name: groups; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY groups (id, name, created, modified, admin, inseance, superuser, baseuser) FROM stdin;
26b380d8-93b2-41e9-9f68-da98c55ccb30	Administrateur	2013-02-11 11:21:45.788881	2013-06-13 17:19:10	t	f	f	f
b2f7a7e4-4ab1-4d93-ac13-d7ca540b4c16	Acteur	2013-02-11 11:21:45.805519	2013-06-26 13:36:31	f	t	f	t
9f803280-eb21-46b3-bcec-74ffc4d2aab8	Utilisateur	2013-02-11 11:21:45.832553	2013-06-26 13:36:49	f	f	t	f
\.


--
-- Data for Name: imgpages; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY imgpages (id, name, size, type, content, imgpagenum, document_id, created, modified, checksum) FROM stdin;
\.


--
-- Data for Name: projets; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY projets (id, name, theme, document_id, vote, ptheme_id, seance_id, created, modified, rank) FROM stdin;
\.


--
-- Data for Name: pthemes; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY pthemes (id, name, parent_id, rght, lft) FROM stdin;
\.


--
-- Name: pthemes_id_seq; Type: SEQUENCE SET; Schema: public; Owner: idelibre
--

SELECT pg_catalog.setval('pthemes_id_seq', 1, true);


--
-- Data for Name: seances; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY seances (id, name, date_seance, created, modified, type_id, document_id, alias) FROM stdin;
\.


--
-- Data for Name: types; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY types (id, name, created, modified) FROM stdin;
\.


--
-- Data for Name: types_users; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY types_users (id, type_id, user_id, created, modified) FROM stdin;
\.


--
-- Data for Name: users; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY users (id, firstname, lastname, username, password, last_login, group_id, created, modified, active, last_logout, last_import, mail, pwdmodified, rev, rev_date, seance_sync_date) FROM stdin;
e64e47b6-9027-4d73-a49b-70fd2ee0475e	Administrateur	Administrateur	Administrateur	mot de pass non valide	\N	26b380d8-93b2-41e9-9f68-da98c55ccb30	2013-11-07 15:54:40.942895	2013-11-07 15:54:40.942895	t	\N	\N	\N	f	\N	\N	\N
\.


--
-- Name: acos_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY acos
    ADD CONSTRAINT acos_pkey PRIMARY KEY (id);


--
-- Name: annexes_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY annexes
    ADD CONSTRAINT annexes_pkey PRIMARY KEY (id);


--
-- Name: annotactions_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY annotactions
    ADD CONSTRAINT annotactions_pkey PRIMARY KEY (id);


--
-- Name: aros_acos_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY aros_acos
    ADD CONSTRAINT aros_acos_pkey PRIMARY KEY (id);


--
-- Name: aros_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY aros
    ADD CONSTRAINT aros_pkey PRIMARY KEY (id);


--
-- Name: convocations_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY seances
    ADD CONSTRAINT convocations_pkey PRIMARY KEY (id);


--
-- Name: convocusers_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY convocations
    ADD CONSTRAINT convocusers_pkey PRIMARY KEY (id);


--
-- Name: deltarevisions_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY deltarevisions
    ADD CONSTRAINT deltarevisions_pkey PRIMARY KEY (id);


--
-- Name: documents_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY documents
    ADD CONSTRAINT documents_pkey PRIMARY KEY (id);


--
-- Name: groups_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY groups
    ADD CONSTRAINT groups_pkey PRIMARY KEY (id);


--
-- Name: imgpages_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY imgpages
    ADD CONSTRAINT imgpages_pkey PRIMARY KEY (id);


--
-- Name: name_unique_idx; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY groups
    ADD CONSTRAINT name_unique_idx UNIQUE (name);


--
-- Name: projets_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY projets
    ADD CONSTRAINT projets_pkey PRIMARY KEY (id);


--
-- Name: pthemes_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY pthemes
    ADD CONSTRAINT pthemes_pkey PRIMARY KEY (id);


--
-- Name: seances_alias_key; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY seances
    ADD CONSTRAINT seances_alias_key UNIQUE (alias);


--
-- Name: types_name_unique_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY types
    ADD CONSTRAINT types_name_unique_pkey UNIQUE (name);


--
-- Name: types_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY types
    ADD CONSTRAINT types_pkey PRIMARY KEY (id);


--
-- Name: types_users_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY types_users
    ADD CONSTRAINT types_users_pkey PRIMARY KEY (id);


--
-- Name: user_mail_unique_key; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY users
    ADD CONSTRAINT user_mail_unique_key UNIQUE (mail);


--
-- Name: user_username_unique_key; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY users
    ADD CONSTRAINT user_username_unique_key UNIQUE (username);


--
-- Name: users_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY users
    ADD CONSTRAINT users_pkey PRIMARY KEY (id);


--
-- Name: aro_aco_key; Type: INDEX; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE UNIQUE INDEX aro_aco_key ON aros_acos USING btree (aro_id, aco_id);


--
-- Name: annexes_projet_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY annexes
    ADD CONSTRAINT annexes_projet_id_fkey FOREIGN KEY (projet_id) REFERENCES projets(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: annotactions_deltarevision_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY annotactions
    ADD CONSTRAINT annotactions_deltarevision_id_fkey FOREIGN KEY (deltarevision_id) REFERENCES deltarevisions(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: convocations_type_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY seances
    ADD CONSTRAINT convocations_type_id_fkey FOREIGN KEY (type_id) REFERENCES types(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: convocusers_convocation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY convocations
    ADD CONSTRAINT convocusers_convocation_id_fkey FOREIGN KEY (seance_id) REFERENCES seances(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: convocusers_user_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY convocations
    ADD CONSTRAINT convocusers_user_id_fkey FOREIGN KEY (user_id) REFERENCES users(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: deltarevisions_user_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY deltarevisions
    ADD CONSTRAINT deltarevisions_user_id_fkey FOREIGN KEY (user_id) REFERENCES users(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: imgpages_document_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY imgpages
    ADD CONSTRAINT imgpages_document_id_fkey FOREIGN KEY (document_id) REFERENCES documents(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: projets_document_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY projets
    ADD CONSTRAINT projets_document_id_fkey FOREIGN KEY (document_id) REFERENCES documents(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: projets_ptheme_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY projets
    ADD CONSTRAINT projets_ptheme_id_fkey FOREIGN KEY (ptheme_id) REFERENCES pthemes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: projets_seance_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY projets
    ADD CONSTRAINT projets_seance_id_fkey FOREIGN KEY (seance_id) REFERENCES seances(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: seances_document_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY seances
    ADD CONSTRAINT seances_document_id_fkey FOREIGN KEY (document_id) REFERENCES documents(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: types_users_type_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY types_users
    ADD CONSTRAINT types_users_type_id_fkey FOREIGN KEY (type_id) REFERENCES types(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: types_users_user_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY types_users
    ADD CONSTRAINT types_users_user_id_fkey FOREIGN KEY (user_id) REFERENCES users(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: users_group_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY users
    ADD CONSTRAINT users_group_id_fkey FOREIGN KEY (group_id) REFERENCES groups(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: public; Type: ACL; Schema: -; Owner: postgres
--

REVOKE ALL ON SCHEMA public FROM PUBLIC;
REVOKE ALL ON SCHEMA public FROM postgres;
GRANT ALL ON SCHEMA public TO postgres;
GRANT ALL ON SCHEMA public TO PUBLIC;


--
-- PostgreSQL database dump complete
--

