--
-- PostgreSQL database dump
--

SET statement_timeout = 0;
SET lock_timeout = 0;
SET client_encoding = 'UTF8';
SET standard_conforming_strings = on;
SET check_function_bodies = false;
SET client_min_messages = warning;

--
-- Name: plpgsql; Type: EXTENSION; Schema: -; Owner: 
--

CREATE EXTENSION IF NOT EXISTS plpgsql WITH SCHEMA pg_catalog;


--
-- Name: EXTENSION plpgsql; Type: COMMENT; Schema: -; Owner: 
--

COMMENT ON EXTENSION plpgsql IS 'PL/pgSQL procedural language';


--
-- Name: uuid-ossp; Type: EXTENSION; Schema: -; Owner: 
--

CREATE EXTENSION IF NOT EXISTS "uuid-ossp" WITH SCHEMA public;


--
-- Name: EXTENSION "uuid-ossp"; Type: COMMENT; Schema: -; Owner: 
--

COMMENT ON EXTENSION "uuid-ossp" IS 'generate universally unique identifiers (UUIDs)';


SET search_path = public, pg_catalog;

--
-- Name: rebuilt_sequences(); Type: FUNCTION; Schema: public; Owner: idelibre
--

CREATE FUNCTION rebuilt_sequences() RETURNS integer
    LANGUAGE plpgsql
    AS $$
  DECLARE sequencedefs RECORD; c integer ;
  BEGIN
    FOR sequencedefs IN Select
      constraint_column_usage.table_name as tablename,
      constraint_column_usage.table_name as tablename,
      constraint_column_usage.column_name as columnname,
      replace(replace(columns.column_default,'''::regclass)',''),'nextval(''','') as sequencename
      from information_schema.constraint_column_usage, information_schema.columns
      where constraint_column_usage.table_schema ='public' AND
      columns.table_schema = 'public' AND columns.table_name=constraint_column_usage.table_name
      AND constraint_column_usage.column_name = columns.column_name
      AND columns.column_default is not null
   LOOP
      EXECUTE 'select max('||sequencedefs.columnname||') from ' || sequencedefs.tablename INTO c;
      IF c is null THEN c = 0; END IF;
      IF c is not null THEN c = c+ 1; END IF;
      EXECUTE 'alter sequence ' || sequencedefs.sequencename ||' restart  with ' || c;
   END LOOP;

   RETURN 1; END;
$$;


ALTER FUNCTION public.rebuilt_sequences() OWNER TO idelibre;

SET default_tablespace = '';

SET default_with_oids = false;

--
-- Name: acos; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE acos (
    id integer NOT NULL,
    parent_id integer,
    model character varying(255) DEFAULT NULL::character varying,
    alias character varying(255) DEFAULT NULL::character varying,
    lft integer,
    rght integer,
    foreign_key uuid
);


ALTER TABLE public.acos OWNER TO idelibre;

--
-- Name: TABLE acos; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE acos IS 'Gestion des ACL (CakePHP) - ressources';


--
-- Name: acos_id_seq; Type: SEQUENCE; Schema: public; Owner: idelibre
--

CREATE SEQUENCE acos_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.acos_id_seq OWNER TO idelibre;

--
-- Name: acos_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: idelibre
--

ALTER SEQUENCE acos_id_seq OWNED BY acos.id;


--
-- Name: annexes; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE annexes (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    name character varying(500) NOT NULL,
    size integer DEFAULT 0 NOT NULL,
    type character varying(255) NOT NULL,
    content bytea,
    created timestamp without time zone,
    modified timestamp without time zone,
    projet_id uuid,
    checksum character varying(32),
    rank integer DEFAULT 0 NOT NULL
);


ALTER TABLE public.annexes OWNER TO idelibre;

--
-- Name: TABLE annexes; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE annexes IS 'Gestion des annexes de projets de délibérations (pièces jointes)';


--
-- Name: annotactions; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE annotactions (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    deltarevision_id uuid,
    created timestamp without time zone,
    modified timestamp without time zone,
    type character varying(1),
    annotation_id character varying(100),
    CONSTRAINT annotactions_type_in_chk CHECK (((type)::text = ANY (ARRAY[('C'::character varying)::text, ('U'::character varying)::text, ('D'::character varying)::text])))
);


ALTER TABLE public.annotactions OWNER TO idelibre;

--
-- Name: annotations; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE annotations (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    annot_client_id uuid,
    imgpage_id uuid NOT NULL,
    content bytea NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    user_id uuid NOT NULL,
    shared boolean DEFAULT false NOT NULL,
    rank integer DEFAULT 0 NOT NULL,
    date timestamp without time zone,
    date_modif timestamp without time zone,
    height character varying(20),
    width character varying(20),
    x character varying(20),
    y character varying(20),
    comment character varying(255)
);


ALTER TABLE public.annotations OWNER TO idelibre;

--
-- Name: annotations_users; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE annotations_users (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    user_id uuid NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    annotation_id uuid NOT NULL,
    read boolean DEFAULT false NOT NULL
);


ALTER TABLE public.annotations_users OWNER TO idelibre;

--
-- Name: aros; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE aros (
    id integer NOT NULL,
    parent_id integer,
    model character varying(255) DEFAULT NULL::character varying,
    alias character varying(255) DEFAULT NULL::character varying,
    lft integer,
    rght integer,
    foreign_key uuid
);


ALTER TABLE public.aros OWNER TO idelibre;

--
-- Name: TABLE aros; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE aros IS 'Gestion des ACL (CakePHP) - demandeurs';


--
-- Name: aros_acos; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE aros_acos (
    id integer NOT NULL,
    aro_id integer NOT NULL,
    aco_id integer NOT NULL,
    _create character varying(2) DEFAULT '0'::character varying NOT NULL,
    _read character varying(2) DEFAULT '0'::character varying NOT NULL,
    _update character varying(2) DEFAULT '0'::character varying NOT NULL,
    _delete character varying(2) DEFAULT '0'::character varying NOT NULL
);


ALTER TABLE public.aros_acos OWNER TO idelibre;

--
-- Name: TABLE aros_acos; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE aros_acos IS 'Gestion des ACL (CakePHP) - liaisons demandeurs / ressources';


--
-- Name: aros_acos_id_seq; Type: SEQUENCE; Schema: public; Owner: idelibre
--

CREATE SEQUENCE aros_acos_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.aros_acos_id_seq OWNER TO idelibre;

--
-- Name: aros_acos_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: idelibre
--

ALTER SEQUENCE aros_acos_id_seq OWNED BY aros_acos.id;


--
-- Name: aros_id_seq; Type: SEQUENCE; Schema: public; Owner: idelibre
--

CREATE SEQUENCE aros_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.aros_id_seq OWNER TO idelibre;

--
-- Name: aros_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: idelibre
--

ALTER SEQUENCE aros_id_seq OWNED BY aros.id;


--
-- Name: convocations; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE convocations (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    seance_id uuid NOT NULL,
    user_id uuid NOT NULL,
    read boolean DEFAULT false NOT NULL,
    presence boolean DEFAULT true NOT NULL,
    delegation boolean DEFAULT false NOT NULL,
    procuration boolean DEFAULT false NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    ar_horodatage timestamp without time zone,
    active boolean DEFAULT false NOT NULL,
    ar_horodatage_file bytea,
    ar_horodatage_token bytea,
    ar_received timestamp without time zone,
    ae_sent timestamp without time zone,
    ae_horodatage timestamp without time zone,
    ae_horodatage_file bytea,
    ae_horodatage_token bytea
);


ALTER TABLE public.convocations OWNER TO idelibre;

--
-- Name: TABLE convocations; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE convocations IS 'Gestion des convocations';


--
-- Name: deltarevisions; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE deltarevisions (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    num integer NOT NULL,
    delta bytea NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    user_id uuid NOT NULL
);


ALTER TABLE public.deltarevisions OWNER TO idelibre;

--
-- Name: deltarevisions_num_seq; Type: SEQUENCE; Schema: public; Owner: idelibre
--

CREATE SEQUENCE deltarevisions_num_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.deltarevisions_num_seq OWNER TO idelibre;

--
-- Name: deltarevisions_num_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: idelibre
--

ALTER SEQUENCE deltarevisions_num_seq OWNED BY deltarevisions.num;


--
-- Name: documents; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE documents (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    name character varying(255) NOT NULL,
    size integer DEFAULT 0 NOT NULL,
    type character varying(255) NOT NULL,
    content bytea,
    created timestamp without time zone,
    modified timestamp without time zone,
    checksum character varying(32)
);


ALTER TABLE public.documents OWNER TO idelibre;

--
-- Name: TABLE documents; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE documents IS 'Gestion des documents';


--
-- Name: groups; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE groups (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    name character varying(255) NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    admin boolean DEFAULT false NOT NULL,
    inseance boolean DEFAULT false,
    superuser boolean DEFAULT false,
    baseuser boolean DEFAULT true
);


ALTER TABLE public.groups OWNER TO idelibre;

--
-- Name: TABLE groups; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE groups IS 'Gestion des groupes';


--
-- Name: imgpages; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE imgpages (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    name character varying(255) NOT NULL,
    size integer DEFAULT 0 NOT NULL,
    type character varying(255) NOT NULL,
    content bytea,
    imgpagenum integer DEFAULT 0 NOT NULL,
    document_id uuid NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    checksum character varying(32)
);


ALTER TABLE public.imgpages OWNER TO idelibre;

--
-- Name: TABLE imgpages; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE imgpages IS 'Gestion des pages des documents à annoter';


--
-- Name: projets; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE projets (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    name character varying(500) NOT NULL,
    theme character varying(500),
    document_id uuid,
    vote character varying(10),
    ptheme_id integer,
    seance_id uuid NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    rank integer DEFAULT 0 NOT NULL,
    CONSTRAINT projets_vote_check CHECK (((vote)::text = ANY (ARRAY[('oui'::character varying)::text, ('non'::character varying)::text, ('nsp'::character varying)::text, ('abs'::character varying)::text])))
);


ALTER TABLE public.projets OWNER TO idelibre;

--
-- Name: TABLE projets; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE projets IS 'Gestion des projets de délibérations';


--
-- Name: pthemes; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE pthemes (
    id integer NOT NULL,
    name character varying(255) NOT NULL,
    parent_id integer,
    rght integer NOT NULL,
    lft integer NOT NULL
);


ALTER TABLE public.pthemes OWNER TO idelibre;

--
-- Name: TABLE pthemes; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE pthemes IS 'Gestion des themes de projets de délibération';


--
-- Name: pthemes_id_seq; Type: SEQUENCE; Schema: public; Owner: idelibre
--

CREATE SEQUENCE pthemes_id_seq
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;


ALTER TABLE public.pthemes_id_seq OWNER TO idelibre;

--
-- Name: pthemes_id_seq; Type: SEQUENCE OWNED BY; Schema: public; Owner: idelibre
--

ALTER SEQUENCE pthemes_id_seq OWNED BY pthemes.id;


--
-- Name: seances; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE seances (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    name character varying(255) NOT NULL,
    date_seance timestamp without time zone NOT NULL,
    rev numeric DEFAULT 1 NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    type_id uuid,
    document_id uuid NOT NULL,
    alias character varying(255) NOT NULL
);


ALTER TABLE public.seances OWNER TO idelibre;

--
-- Name: TABLE seances; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE seances IS 'Gestion des séances';


--
-- Name: types; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE types (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    name character varying(255) NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone
);


ALTER TABLE public.types OWNER TO idelibre;

--
-- Name: TABLE types; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE types IS 'Gestion des types de séances';


--
-- Name: types_users; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE types_users (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    type_id uuid NOT NULL,
    user_id uuid NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone
);


ALTER TABLE public.types_users OWNER TO idelibre;

--
-- Name: TABLE types_users; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE types_users IS 'Liaison entre les types de séances et les utilisateurs';


--
-- Name: users; Type: TABLE; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE TABLE users (
    id uuid DEFAULT uuid_generate_v4() NOT NULL,
    firstname character varying(255) NOT NULL,
    lastname character varying(255) NOT NULL,
    username character varying(255) NOT NULL,
    password character varying(255) NOT NULL,
    last_login timestamp without time zone,
    group_id uuid NOT NULL,
    created timestamp without time zone,
    modified timestamp without time zone,
    active boolean DEFAULT true NOT NULL,
    last_logout timestamp without time zone,
    last_import timestamp without time zone,
    mail character varying(255),
    pwdmodified boolean DEFAULT false NOT NULL,
    rev integer,
    rev_date timestamp without time zone,
    seance_sync_date timestamp without time zone
);


ALTER TABLE public.users OWNER TO idelibre;

--
-- Name: TABLE users; Type: COMMENT; Schema: public; Owner: idelibre
--

COMMENT ON TABLE users IS 'Gestion des utilisateurs';


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY acos ALTER COLUMN id SET DEFAULT nextval('acos_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY aros ALTER COLUMN id SET DEFAULT nextval('aros_id_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY aros_acos ALTER COLUMN id SET DEFAULT nextval('aros_acos_id_seq'::regclass);


--
-- Name: num; Type: DEFAULT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY deltarevisions ALTER COLUMN num SET DEFAULT nextval('deltarevisions_num_seq'::regclass);


--
-- Name: id; Type: DEFAULT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY pthemes ALTER COLUMN id SET DEFAULT nextval('pthemes_id_seq'::regclass);


--
-- Data for Name: acos; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY acos (id, parent_id, model, alias, lft, rght, foreign_key) FROM stdin;
36	33	\N	getHorodatageFile	69	70	\N
3	2	\N	getFile	3	4	\N
80	1	\N	Imgpages	158	175	\N
4	2	\N	setConn	5	6	\N
59	56	\N	params	115	116	\N
37	33	\N	getHorodatageToken	71	72	\N
5	2	\N	processLogin	7	8	\N
6	2	\N	writeLoginConfig	9	10	\N
76	68	\N	writeUserConfig	149	150	\N
38	33	\N	setConn	73	74	\N
7	2	\N	writeUserConfig	11	12	\N
8	2	\N	writeLoginSession	13	14	\N
60	56	\N	stats	117	118	\N
39	33	\N	processLogin	75	76	\N
9	2	\N	ldebug	15	16	\N
2	1	\N	Annexes	2	19	\N
10	2	\N	ldebugHr	17	18	\N
88	80	\N	ldebugHr	173	174	\N
40	33	\N	writeLoginConfig	77	78	\N
12	11	\N	getData	21	22	\N
61	56	\N	setConn	119	120	\N
13	11	\N	setConn	23	24	\N
41	33	\N	writeUserConfig	79	80	\N
14	11	\N	processLogin	25	26	\N
77	68	\N	writeLoginSession	151	152	\N
15	11	\N	writeLoginConfig	27	28	\N
42	33	\N	writeLoginSession	81	82	\N
16	11	\N	writeUserConfig	29	30	\N
62	56	\N	processLogin	121	122	\N
17	11	\N	writeLoginSession	31	32	\N
43	33	\N	ldebug	83	84	\N
18	11	\N	ldebug	33	34	\N
11	1	\N	Clients	20	37	\N
19	11	\N	ldebugHr	35	36	\N
33	1	\N	Convocations	64	87	\N
44	33	\N	ldebugHr	85	86	\N
108	102	\N	writeLoginSession	213	214	\N
21	20	\N	index	39	40	\N
97	89	\N	writeUserConfig	191	192	\N
22	20	\N	add	41	42	\N
63	56	\N	writeLoginConfig	123	124	\N
23	20	\N	edit	43	44	\N
46	45	\N	save_annotations	89	90	\N
24	20	\N	delete	45	46	\N
78	68	\N	ldebug	153	154	\N
25	20	\N	info	47	48	\N
47	45	\N	getFile	91	92	\N
26	20	\N	setConn	49	50	\N
64	56	\N	writeUserConfig	125	126	\N
27	20	\N	processLogin	51	52	\N
48	45	\N	getAnnotations	93	94	\N
28	20	\N	writeLoginConfig	53	54	\N
68	1	\N	Groups	134	157	\N
29	20	\N	writeUserConfig	55	56	\N
49	45	\N	setConn	95	96	\N
30	20	\N	writeLoginSession	57	58	\N
65	56	\N	writeLoginSession	127	128	\N
31	20	\N	ldebug	59	60	\N
50	45	\N	processLogin	97	98	\N
20	1	\N	Collectivites	38	63	\N
32	20	\N	ldebugHr	61	62	\N
79	68	\N	ldebugHr	155	156	\N
51	45	\N	writeLoginConfig	99	100	\N
34	33	\N	setRead	65	66	\N
66	56	\N	ldebug	129	130	\N
35	33	\N	send	67	68	\N
56	1	\N	Environnements	110	133	\N
52	45	\N	writeUserConfig	101	102	\N
67	56	\N	ldebugHr	131	132	\N
53	45	\N	writeLoginSession	103	104	\N
54	45	\N	ldebug	105	106	\N
45	1	\N	Documents	88	109	\N
55	45	\N	ldebugHr	107	108	\N
90	89	\N	index	177	178	\N
69	68	\N	index	135	136	\N
57	56	\N	index	111	112	\N
81	80	\N	getFile	159	160	\N
58	56	\N	menu	113	114	\N
104	102	\N	setConn	205	206	\N
98	89	\N	writeLoginSession	193	194	\N
70	68	\N	add	137	138	\N
82	80	\N	setConn	161	162	\N
71	68	\N	edit	139	140	\N
91	89	\N	start	179	180	\N
72	68	\N	delete	141	142	\N
83	80	\N	processLogin	163	164	\N
73	68	\N	setConn	143	144	\N
74	68	\N	processLogin	145	146	\N
92	89	\N	stop	181	182	\N
75	68	\N	writeLoginConfig	147	148	\N
84	80	\N	writeLoginConfig	165	166	\N
99	89	\N	ldebug	195	196	\N
85	80	\N	writeUserConfig	167	168	\N
93	89	\N	display	183	184	\N
86	80	\N	writeLoginSession	169	170	\N
115	111	\N	delete	227	228	\N
87	80	\N	ldebug	171	172	\N
89	1	\N	Monitor	176	199	\N
100	89	\N	ldebugHr	197	198	\N
94	89	\N	setConn	185	186	\N
105	102	\N	processLogin	207	208	\N
95	89	\N	processLogin	187	188	\N
109	102	\N	ldebug	215	216	\N
96	89	\N	writeLoginConfig	189	190	\N
101	1	\N	NewEmptyPHP	200	201	\N
106	102	\N	writeLoginConfig	209	210	\N
102	1	\N	Pages	202	219	\N
103	102	\N	display	203	204	\N
110	102	\N	ldebugHr	217	218	\N
107	102	\N	writeUserConfig	211	212	\N
112	111	\N	index	221	222	\N
114	111	\N	edit	225	226	\N
119	111	\N	writeLoginConfig	235	236	\N
113	111	\N	add	223	224	\N
118	111	\N	processLogin	233	234	\N
116	111	\N	getList	229	230	\N
117	111	\N	setConn	231	232	\N
120	111	\N	writeUserConfig	237	238	\N
121	111	\N	writeLoginSession	239	240	\N
122	111	\N	ldebug	241	242	\N
123	111	\N	ldebugHr	243	244	\N
125	124	\N	index	247	248	\N
111	1	\N	Pthemes	220	245	\N
126	124	\N	view	249	250	\N
127	124	\N	add	251	252	\N
128	124	\N	edit	253	254	\N
129	124	\N	delete	255	256	\N
130	124	\N	ajout	257	258	\N
131	124	\N	liste	259	260	\N
132	124	\N	details	261	262	\N
124	1	\N	Seances	246	285	\N
218	211	\N	writeLoginSession	432	433	\N
133	124	\N	suppression	263	264	\N
194	1	\N	Users	386	415	\N
134	124	\N	edition	265	266	\N
168	156	\N	get_proof	333	334	\N
135	124	\N	sendConvocations	267	268	\N
136	124	\N	refreshSeance	269	270	\N
191	181	\N	writeLoginSession	379	380	\N
137	124	\N	setConn	271	272	\N
138	124	\N	processLogin	273	274	\N
169	156	\N	login	335	336	\N
139	124	\N	writeLoginConfig	275	276	\N
140	124	\N	writeUserConfig	277	278	\N
208	194	\N	ldebugHr	413	414	\N
141	124	\N	writeLoginSession	279	280	\N
142	124	\N	ldebug	281	282	\N
170	156	\N	logout	337	338	\N
143	124	\N	ldebugHr	283	284	\N
192	181	\N	ldebug	381	382	\N
171	156	\N	WSTellGlobalAuth	339	340	\N
145	144	\N	index	287	288	\N
146	144	\N	add	289	290	\N
181	1	\N	Types	360	385	\N
172	156	\N	mdplost	341	342	\N
147	144	\N	edit	291	292	\N
193	181	\N	ldebugHr	383	384	\N
148	144	\N	delete	293	294	\N
173	156	\N	setConn	343	344	\N
149	144	\N	setConn	295	296	\N
209	1	\N	AclExtras	416	417	\N
150	144	\N	processLogin	297	298	\N
174	156	\N	processLogin	345	346	\N
151	144	\N	writeLoginConfig	299	300	\N
152	144	\N	writeUserConfig	301	302	\N
175	156	\N	writeLoginConfig	347	348	\N
153	144	\N	writeLoginSession	303	304	\N
154	144	\N	ldebug	305	306	\N
195	194	\N	index	387	388	\N
144	1	\N	Srvroles	286	309	\N
155	144	\N	ldebugHr	307	308	\N
176	156	\N	writeUserConfig	349	350	\N
157	156	\N	index	311	312	\N
177	156	\N	writeLoginSession	351	352	\N
158	156	\N	addAdminColl	313	314	\N
196	194	\N	add	389	390	\N
159	156	\N	add	315	316	\N
178	156	\N	ldebug	353	354	\N
160	156	\N	listeMandat	317	318	\N
156	1	\N	Srvusers	310	357	\N
161	156	\N	authorizedCollectivite	319	320	\N
179	156	\N	ldebugHr	355	356	\N
162	156	\N	authorizedCollectiviteId	321	322	\N
219	211	\N	ldebug	434	435	\N
180	1	\N	SrvusersController1	358	359	\N
163	156	\N	toRemoveMandat	323	324	\N
164	156	\N	edit	325	326	\N
197	194	\N	edit	391	392	\N
165	156	\N	editWalletManager	327	328	\N
166	156	\N	editPortefeuilleMandat	329	330	\N
1	\N	\N	controllers	1	440	\N
198	194	\N	delete	393	394	\N
167	156	\N	delete	331	332	\N
182	181	\N	index	361	362	\N
210	1	\N	DebugKit	418	439	\N
183	181	\N	add	363	364	\N
211	210	\N	ToolbarAccess	419	438	\N
199	194	\N	login	395	396	\N
184	181	\N	edit	365	366	\N
212	211	\N	history_state	420	421	\N
185	181	\N	delete	367	368	\N
200	194	\N	logout	397	398	\N
186	181	\N	users	369	370	\N
220	211	\N	ldebugHr	436	437	\N
187	181	\N	setConn	371	372	\N
201	194	\N	mdplost	399	400	\N
188	181	\N	processLogin	373	374	\N
189	181	\N	writeLoginConfig	375	376	\N
213	211	\N	sql_explain	422	423	\N
190	181	\N	writeUserConfig	377	378	\N
202	194	\N	setConn	401	402	\N
203	194	\N	processLogin	403	404	\N
214	211	\N	setConn	424	425	\N
204	194	\N	writeLoginConfig	405	406	\N
205	194	\N	writeUserConfig	407	408	\N
215	211	\N	processLogin	426	427	\N
206	194	\N	writeLoginSession	409	410	\N
207	194	\N	ldebug	411	412	\N
216	211	\N	writeLoginConfig	428	429	\N
217	211	\N	writeUserConfig	430	431	\N
\.


--
-- Name: acos_id_seq; Type: SEQUENCE SET; Schema: public; Owner: idelibre
--

SELECT pg_catalog.setval('acos_id_seq', 220, true);


--
-- Data for Name: annexes; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY annexes (id, name, size, type, content, created, modified, projet_id, checksum, rank) FROM stdin;
\.


--
-- Data for Name: annotactions; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY annotactions (id, deltarevision_id, created, modified, type, annotation_id) FROM stdin;
\.


--
-- Data for Name: annotations; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY annotations (id, annot_client_id, imgpage_id, content, created, modified, user_id, shared, rank, date, date_modif, height, width, x, y, comment) FROM stdin;
\.


--
-- Data for Name: annotations_users; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY annotations_users (id, user_id, created, modified, annotation_id, read) FROM stdin;
\.


--
-- Data for Name: aros; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY aros (id, parent_id, model, alias, lft, rght, foreign_key) FROM stdin;
1	\N	Group	Administrateurs	1	4	26b380d8-93b2-41e9-9f68-da98c55ccb30
2	\N	Group	Acteurs	5	6	b2f7a7e4-4ab1-4d93-ac13-d7ca540b4c16
3	\N	Group	Utilisateurs	7	8	9f803280-eb21-46b3-bcec-74ffc4d2aab8
4	1	User	Administrateur	2	3	e64e47b6-9027-4d73-a49b-70fd2ee0475e
\.


--
-- Data for Name: aros_acos; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY aros_acos (id, aro_id, aco_id, _create, _read, _update, _delete) FROM stdin;
1	1	25	1	1	1	1
2	1	56	1	1	1	1
3	1	36	1	1	1	1
4	1	37	1	1	1	1
5	1	68	1	1	1	1
6	1	111	1	1	1	1
7	1	181	1	1	1	1
8	1	194	1	1	1	1
9	3	56	1	1	1	1
10	3	130	1	1	1	1
11	3	131	1	1	1	1
12	3	132	1	1	1	1
13	3	134	1	1	1	1
14	3	133	1	1	1	1
15	3	135	1	1	1	1
16	3	33	1	1	1	1
17	3	11	1	1	1	1
18	3	196	1	1	1	1
19	3	111	1	1	1	1
20	3	181	1	1	1	1
21	3	47	1	1	1	1
22	3	81	1	1	1	1
23	3	3	1	1	1	1
24	2	56	1	1	1	1
25	2	124	1	1	1	1
26	2	33	1	1	1	1
27	2	47	1	1	1	1
28	2	48	1	1	1	1
29	2	81	1	1	1	1
30	2	3	1	1	1	1
31	2	11	1	1	1	1
\.


--
-- Name: aros_acos_id_seq; Type: SEQUENCE SET; Schema: public; Owner: idelibre
--

SELECT pg_catalog.setval('aros_acos_id_seq', 31, true);


--
-- Name: aros_id_seq; Type: SEQUENCE SET; Schema: public; Owner: idelibre
--

SELECT pg_catalog.setval('aros_id_seq', 4, true);


--
-- Data for Name: convocations; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY convocations (id, seance_id, user_id, read, presence, delegation, procuration, created, modified, ar_horodatage, active, ar_horodatage_file, ar_horodatage_token, ar_received, ae_sent, ae_horodatage, ae_horodatage_file, ae_horodatage_token) FROM stdin;
\.


--
-- Data for Name: deltarevisions; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY deltarevisions (id, num, delta, created, modified, user_id) FROM stdin;
\.


--
-- Name: deltarevisions_num_seq; Type: SEQUENCE SET; Schema: public; Owner: idelibre
--

SELECT pg_catalog.setval('deltarevisions_num_seq', 1, false);


--
-- Data for Name: documents; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY documents (id, name, size, type, content, created, modified, checksum) FROM stdin;
\.


--
-- Data for Name: groups; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY groups (id, name, created, modified, admin, inseance, superuser, baseuser) FROM stdin;
26b380d8-93b2-41e9-9f68-da98c55ccb30	Administrateurs	2013-02-11 11:21:45.788881	2013-06-13 17:19:10	t	f	f	f
b2f7a7e4-4ab1-4d93-ac13-d7ca540b4c16	Acteurs	2013-02-11 11:21:45.805519	2013-06-26 13:36:31	f	t	f	t
9f803280-eb21-46b3-bcec-74ffc4d2aab8	Utilisateurs	2013-02-11 11:21:45.832553	2013-06-26 13:36:49	f	f	t	f
\.


--
-- Data for Name: imgpages; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY imgpages (id, name, size, type, content, imgpagenum, document_id, created, modified, checksum) FROM stdin;
\.


--
-- Data for Name: projets; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY projets (id, name, theme, document_id, vote, ptheme_id, seance_id, created, modified, rank) FROM stdin;
\.


--
-- Data for Name: pthemes; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY pthemes (id, name, parent_id, rght, lft) FROM stdin;
\.


--
-- Name: pthemes_id_seq; Type: SEQUENCE SET; Schema: public; Owner: idelibre
--

SELECT pg_catalog.setval('pthemes_id_seq', 1, true);


--
-- Data for Name: seances; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY seances (id, name, date_seance, rev, created, modified, type_id, document_id, alias) FROM stdin;
\.


--
-- Data for Name: types; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY types (id, name, created, modified) FROM stdin;
\.


--
-- Data for Name: types_users; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY types_users (id, type_id, user_id, created, modified) FROM stdin;
\.


--
-- Data for Name: users; Type: TABLE DATA; Schema: public; Owner: idelibre
--

COPY users (id, firstname, lastname, username, password, last_login, group_id, created, modified, active, last_logout, last_import, mail, pwdmodified, rev, rev_date, seance_sync_date) FROM stdin;
e64e47b6-9027-4d73-a49b-70fd2ee0475e	Administrateur	Administrateur	Administrateur	mot de pass non valide	\N	26b380d8-93b2-41e9-9f68-da98c55ccb30	2013-11-07 15:54:40.942895	2013-11-07 15:54:40.942895	t	\N	\N	\N	f	\N	\N	\N
\.


--
-- Name: acos_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY acos
    ADD CONSTRAINT acos_pkey PRIMARY KEY (id);


--
-- Name: annexes_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY annexes
    ADD CONSTRAINT annexes_pkey PRIMARY KEY (id);


--
-- Name: annotactions_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY annotactions
    ADD CONSTRAINT annotactions_pkey PRIMARY KEY (id);


--
-- Name: aros_acos_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY aros_acos
    ADD CONSTRAINT aros_acos_pkey PRIMARY KEY (id);


--
-- Name: aros_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY aros
    ADD CONSTRAINT aros_pkey PRIMARY KEY (id);


--
-- Name: convocations_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY seances
    ADD CONSTRAINT convocations_pkey PRIMARY KEY (id);


--
-- Name: convocusers_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY convocations
    ADD CONSTRAINT convocusers_pkey PRIMARY KEY (id);


--
-- Name: deltarevisions_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY deltarevisions
    ADD CONSTRAINT deltarevisions_pkey PRIMARY KEY (id);


--
-- Name: documents_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY documents
    ADD CONSTRAINT documents_pkey PRIMARY KEY (id);


--
-- Name: groups_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY groups
    ADD CONSTRAINT groups_pkey PRIMARY KEY (id);


--
-- Name: imgpages_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY imgpages
    ADD CONSTRAINT imgpages_pkey PRIMARY KEY (id);


--
-- Name: name_unique_idx; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY groups
    ADD CONSTRAINT name_unique_idx UNIQUE (name);


--
-- Name: projets_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY projets
    ADD CONSTRAINT projets_pkey PRIMARY KEY (id);


--
-- Name: pthemes_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY pthemes
    ADD CONSTRAINT pthemes_pkey PRIMARY KEY (id);


--
-- Name: seances_alias_key; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY seances
    ADD CONSTRAINT seances_alias_key UNIQUE (alias);


--
-- Name: types_name_unique_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY types
    ADD CONSTRAINT types_name_unique_pkey UNIQUE (name);


--
-- Name: types_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY types
    ADD CONSTRAINT types_pkey PRIMARY KEY (id);


--
-- Name: types_users_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY types_users
    ADD CONSTRAINT types_users_pkey PRIMARY KEY (id);


--
-- Name: user_mail_unique_key; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY users
    ADD CONSTRAINT user_mail_unique_key UNIQUE (mail);


--
-- Name: user_username_unique_key; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY users
    ADD CONSTRAINT user_username_unique_key UNIQUE (username);


--
-- Name: users_pkey; Type: CONSTRAINT; Schema: public; Owner: idelibre; Tablespace: 
--

ALTER TABLE ONLY users
    ADD CONSTRAINT users_pkey PRIMARY KEY (id);


--
-- Name: aro_aco_key; Type: INDEX; Schema: public; Owner: idelibre; Tablespace: 
--

CREATE UNIQUE INDEX aro_aco_key ON aros_acos USING btree (aro_id, aco_id);


--
-- Name: annexes_projet_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY annexes
    ADD CONSTRAINT annexes_projet_id_fkey FOREIGN KEY (projet_id) REFERENCES projets(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: annotactions_deltarevision_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY annotactions
    ADD CONSTRAINT annotactions_deltarevision_id_fkey FOREIGN KEY (deltarevision_id) REFERENCES deltarevisions(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: convocations_type_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY seances
    ADD CONSTRAINT convocations_type_id_fkey FOREIGN KEY (type_id) REFERENCES types(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: convocusers_convocation_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY convocations
    ADD CONSTRAINT convocusers_convocation_id_fkey FOREIGN KEY (seance_id) REFERENCES seances(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: convocusers_user_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY convocations
    ADD CONSTRAINT convocusers_user_id_fkey FOREIGN KEY (user_id) REFERENCES users(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: deltarevisions_user_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY deltarevisions
    ADD CONSTRAINT deltarevisions_user_id_fkey FOREIGN KEY (user_id) REFERENCES users(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: imgpages_document_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY imgpages
    ADD CONSTRAINT imgpages_document_id_fkey FOREIGN KEY (document_id) REFERENCES documents(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: projets_document_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY projets
    ADD CONSTRAINT projets_document_id_fkey FOREIGN KEY (document_id) REFERENCES documents(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: projets_ptheme_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY projets
    ADD CONSTRAINT projets_ptheme_id_fkey FOREIGN KEY (ptheme_id) REFERENCES pthemes(id) ON UPDATE CASCADE ON DELETE SET NULL;


--
-- Name: projets_seance_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY projets
    ADD CONSTRAINT projets_seance_id_fkey FOREIGN KEY (seance_id) REFERENCES seances(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: seances_document_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY seances
    ADD CONSTRAINT seances_document_id_fkey FOREIGN KEY (document_id) REFERENCES documents(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: types_users_type_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY types_users
    ADD CONSTRAINT types_users_type_id_fkey FOREIGN KEY (type_id) REFERENCES types(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: types_users_user_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY types_users
    ADD CONSTRAINT types_users_user_id_fkey FOREIGN KEY (user_id) REFERENCES users(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: users_group_id_fkey; Type: FK CONSTRAINT; Schema: public; Owner: idelibre
--

ALTER TABLE ONLY users
    ADD CONSTRAINT users_group_id_fkey FOREIGN KEY (group_id) REFERENCES groups(id) ON UPDATE CASCADE ON DELETE CASCADE;


--
-- Name: public; Type: ACL; Schema: -; Owner: postgres
--

REVOKE ALL ON SCHEMA public FROM PUBLIC;
REVOKE ALL ON SCHEMA public FROM postgres;
GRANT ALL ON SCHEMA public TO postgres;
GRANT ALL ON SCHEMA public TO PUBLIC;


--
-- PostgreSQL database dump complete
--

