<?php

App::uses('AppController', 'Controller');
App::uses('Security', 'Component');

/**
 * Convocations Controller
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * SVN Informations
 * $Date: 2014-05-20 14:32:26 +0200 (mar. 20 mai 2014) $
 * $Revision: 424 $
 * $Author: ssampaio $
 * $HeadURL: svn+ssh://rdubourget@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/Controller/ConvocationsController.php $
 * $Id: ConvocationsController.php 424 2014-05-20 12:32:26Z ssampaio $
 *
 * @property Convocation $Convocation
 */
class ConvocationsController extends AppController {

	/**
	 *
	 * @var type
	 */
	public $components = array('Horodatage', 'Security');

	/**
	 *
	 * @var type
	 */
	public $TSFile;

	/**
	 *
	 * @var type
	 */
	public $TSToken;

	/**
	 * Enregistre l'accusé de lecture de la convocation par l'utilisateur de l'application cliente
	 *
	 * @param string $seance_id UUID
	 * @param string $user_id UUID
	 * @throws NotFoundException
	 */
	//TODO: utiliser la classe utilitaire Horodatage
	public function setRead($seance_id, $user_id) {
		$this->layout = 'ajax';

		if (!$this->Convocation->User->exists($user_id) || !$this->Convocation->Seance->exists($seance_id)) {
			throw new NotFoundException();
		}

		$return = array(
			'success' => false,
			'errorMsg' => __d('convocation', 'setRead.error')
		);

		$convoc = $this->Convocation->find('first', array('conditions' => array('Convocation.seance_id' => $seance_id, 'Convocation.user_id' => $user_id)));

		if (empty($convoc)) {
			throw new NotFoundException();
		}

		$convoc['Convocation']['read'] = true;
		$convoc['Convocation']['ar_received'] = date('Y-m-d H:i:s');

		$baseFileName = Security::hash($convoc['Convocation']['id'] . $convoc['Convocation']['ar_horodatage']);
		$this->TSFile = new File(TMP . DS . 'horodatage' . $baseFileName);

		$tsFileContent = "i-delibRE - Timestamp\n" .
				"controller : " . $this->name . "\n" .
				"action : " . $this->action . "\n" .
				"organisation : " . CakeSession::read('Auth.collectivite.name') . " : " . CakeSession::read('Auth.collectivite.id') . "\n" .
				"user : " . CakeSession::read('Auth.User.name') . " : " . CakeSession::read('Auth.User.id') . "\n" .
				"seance : " . $seance_id . "\n" .
				"convocation : " . $convoc['Convocation']['id'] . "\n" .
				"server timestamp : " . $convoc['Convocation']['ar_received'];
		$this->TSFile->write($tsFileContent);

		try {
			if ($this->Horodatage->check() && $this->Horodatage->horodateFichier($this->TSFile->path)) {
				$resp = $this->Horodatage->responseToText;
				for ($i = 0; $i < count($resp); $i++) {
					$matches = array();
					if (preg_match("#Time stamp: (.*)#", $resp[$i], $matches)) {
						if (count($matches) > 1) {
							$date = DateTime::createFromFormat('M j H:i:s Y O', $matches[1]);
							$convoc['Convocation']['ar_horodatage'] = date('Y-m-d H:i:s', CakeTime::convert($date->getTimestamp(), null));
						}
					}
				}
				$convoc['Convocation']['ar_horodatage_file'] = $this->TSFile->read();
				$this->TSToken = new File($this->TSFile->path . '.rep.tsa');
				$convoc['Convocation']['ar_horodatage_token'] = $this->TSToken->read();
			} else {
				$convoc['Convocation']['ar_received'] = null;
				$convoc['Convocation']['ar_horodatage'] = date('Y-m-d H:i:s');
			}

			$this->Convocation->create();
			if ($this->Convocation->save($convoc)) {
				$return['success'] = true;
				$return['errorMsg'] = "";
			}

			if (!empty($this->TSFile)) {
				$this->TSFile->delete();
			}
			if ($this->Horodatage->check() && !empty($this->TSToken)) {
				$this->TSToken->delete();
			}
		} catch (Exception $exc) {
			$data['errorMsg'] = __d('convocation', 'setRead.error:' + ' ' + var_export($exc, true));
		}


		$this->set('return', $return);
	}

	/**
	 * Enregistre l'accusé d'envoi de la convocation pour l'utilisateur de l'application cliente
	 *
	 * @param string $convocation_id UUID
	 */
	public function send($convocation_id) {
		if ($this->request->is('GET') && $this->request->is('ajax')) {
			$this->autoRender = false;
			echo json_encode($this->Convocation->send($convocation_id));
		}
	}

	/**
	 * Récupération de l'image stockée en base
	 *
	 * @param type $id
	 * @throws NotFoundException
	 */
	public function getHorodatageFile($id = null) {
		$blackHole = true;

		if (!empty($id)) {
			try {
				$qd = array(
					'fields' => array(
						'Convocation.id',
						'Convocation.ar_horodatage_file'
					),
					'recursive' => -1,
					'conditions' => array(
						'Convocation.id' => $id
					)
				);
				$convocation = $this->Convocation->find('first', $qd);
				if (!empty($convocation)) {
					//preparation du fichier temporaire
					$name = time() . '_' . $convocation['Convocation']['id'];
					$tmpPath = '/tmp';
					file_put_contents($tmpPath . DS . $name, $convocation['Convocation']['ar_horodatage_file']);

					//utilisation de la vue media
					$this->set('download', true);
					$this->set('id', $name);
					$this->set('name', $name);
					$this->set('path', $tmpPath . DS);
					$this->set('mimeType', 'text/plain');


					$this->viewClass = 'Media';
					$this->autoLayout = false;

					if ($this->render() !== false) {
						$blackHole = false;
					}
				}
			} catch (ErrorException $err) {
				//log debug
				if (Configure::read('debug') > 0) {
					DevTools::ldebug($err);
				}
			}
		}

		//suppression du fichier temporaire
		if (file_exists($tmpPath . DS . $name)) {
			unlink($tmpPath . DS . $name);
		}

		//si erreur on n'envoie rien
		if ($blackHole) {
			$this->Security->blackHole($this);
		}
	}

	/**
	 * Récupération de l'image stockée en base
	 *
	 * @param type $id
	 * @throws NotFoundException
	 */
	public function getHorodatageToken($id = null) {
		$blackHole = true;

		if (!empty($id)) {
			try {
				$qd = array(
					'fields' => array(
						'Convocation.id',
						'Convocation.ar_horodatage_token'
					),
					'recursive' => -1,
					'conditions' => array(
						'Convocation.id' => $id
					)
				);
				$convocation = $this->Convocation->find('first', $qd);
				if (!empty($convocation)) {
					//preparation du fichier temporaire
					$name = mktime() . '_' . $convocation['Convocation']['id'];
					$tmpPath = '/tmp';
					file_put_contents($tmpPath . DS . $name, $convocation['Convocation']['ar_horodatage_token']);

					//utilisation de la vue media
					$this->set('cache', 0);
					$this->set('download', true);
					$this->set('id', $name);
					$this->set('name', $name);
					$this->set('path', $tmpPath . DS);
					$this->set('mimeType', 'text/plain');


					$this->viewClass = 'Media';
					$this->Layout = false;

					if ($this->render() !== false) {
						$blackHole = false;
					}
				}
			} catch (ErrorException $err) {
				//log debug
				if (Configure::read('debug') > 0) {
					DevTools::ldebug($err);
				}
			}
		}

		//suppression du fichier temporaire
		if (file_exists($tmpPath . DS . $name)) {
			unlink($tmpPath . DS . $name);
		}

		//si erreur on n'envoie rien
		if ($blackHole) {
			$this->Security->blackHole($this);
		}
	}

}
