<?php

App::uses('AppController', 'Controller');

/**
 * Documents Controller
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 * @encoding UTF-8
 *
 * SVN Informations
 * $Date: 2014-07-24 17:55:01 +0200 (jeu. 24 juil. 2014) $
 * $Revision: 536 $
 * $Author: ssampaio $
 * $HeadURL: svn+ssh://rdubourget@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/Controller/DocumentsController.php $
 * $Id: DocumentsController.php 536 2014-07-24 15:55:01Z ssampaio $
 *
 * @property Document $Document
 */
class DocumentsController extends AppController {

    /**
     * Components
     *
     * @var array
     */
    public $components = array('DbFile');

    /**
     *
     */
    public function beforeFilter() {
        parent::beforeFilter();
        $this->Auth->allow(array('getFile'));
    }

    /**
     *
     * @param type $data
     * @return type
     * @throws BadMethodCallException
     */
    private function _checkCredentials($data) {
        $return = false;

        $loginParts = explode('@', $data['login']);

        if (count($loginParts) === 2) {
            $Collectivite = ClassRegistry::init(array('class' => 'Collectivite', 'ds' => 'default'));
            $qd = array(
                'fields' => array(
                    'conn'
                ),
                'conditions' => array(
                    'login_suffix' => $loginParts[1],
                    'active' => true
                )
            );
            $resConn = $Collectivite->find('first', $qd);

            if (!empty($resConn)) {
                Configure::write('conn', $resConn['Collectivite']['conn']);
                $this->loadModel('User');
                $qd = array(
                    'recursive' => -1,
                    'fields' => array(
                        'id',
                        'firstname',
                        'lastname'
                    ),
                    'conditions' => array(
                        'username' => $loginParts[0],
                        'password' => $data['password']
                    )
                );
                $resUser = $this->User->find('first', $qd);
                $return = !empty($resUser);
            }
        }
        return $return;
    }

    /**
     * Sauvegarde des annotations venant de la tablette.
     *
     * @param string $documentUuid
     *
     *
     */
    //TODO: verifier pour suppression
    public function save_annotations($documentUuid) {
        $message = __d('convocation', 'post.error');
        $return = array(
            'action' => 'save_annotations',
            'success' => false,
            'message' => $message,
            'uuid' => $documentUuid
        );

        $success = true;

        if (!empty($_POST['annotations'])) {
            $annotations = json_decode(( isset($_POST['annotations']) ? $_POST['annotations'] : '{}'), true);

            $this->Document->begin();
            foreach ($annotations as $annotation) {
                $imgpage = $this->Document->Imgpage->find(
                        'first', array(
                    'fields' => array(
                        'Imgpage.id'
                    ),
                    'conditions' => array(
                        'Imgpage.document_id' => $documentUuid,
                        'Imgpage.imgpagenum' => $annotation['page'],
                    )
                        )
                );
                if (empty($imgpage)) {
                    $success = false;
                } else {
                    $data = array(
                        'Annotation' => array(
                            'imgpage_id' => $imgpage['Imgpage']['id'],
                            'coordx' => $annotation['coordx'],
                            'coordy' => $annotation['coordy'],
                            'content' => $annotation['content'],
                        )
                    );
                    $success = $this->Document->Imgpage->Annotation->save() && $success;
                }
            }

            if ($success) {
                $this->Document->commit();
                $return['message'] = __d('convocation', 'post.ok');
                $return['success'] = true;
            } else {
                $this->Document->rollback();
            }
        }

        $this->set(compact('return'));
    }

    /**
     *
     * @param type $id
     */
    public function getFile($id) {
        $data = array();

        if (!$this->Session->read('Auth.User.id')) { //si pas de session ouverte (par exemple en utilisant le client web / tablette)
            if (!empty($this->request->data) && !empty($this->request->data['User']) && !empty($this->request->data['User']['login']) && !empty($this->request->data['User']['password'])) {
                $data = $this->request->data;
            } else if (!empty($this->request->query['data']) && !empty($this->request->query['data']['User']) && !empty($this->request->query['data']['User']['login']) && !empty($this->request->query['data']['User']['password'])) {
                $data = $this->request->query['data'];
            }

            if (empty($data)) {
                throw new ForbiddenException();
            }

            if (!$this->_checkCredentials($data['User'])) {
                throw new ForbiddenException(__d('login', 'login.error'));
            }
        }
        $this->DbFile->getFile($id);
    }

    /**
     *
     */
    //TODO: déplacer dans ClientsController
    public function getAnnotations() {
        $this->layout = 'ajax';
        if ($this->isAjax || $this->isJsonp) {
            $result = array(
                'annotations' => $this->Document->getAnnotations($this->Session->read('Auth.User.id')),
//				'revision' => $this->Document->Imgpage->Annotation->Annotaction->Deltarevision->field('num', array('created <' => date('Y-m-d H:i:s')), 'created DESC')
//				'revision' => $this->Document->Imgpage->Annotation->Annotaction->Deltarevision->field('num', null, 'created DESC')
            );

            DevTools::ldebug($result['revision']);

            if (!empty($result['revision'])) {

                $this->loadModel('User');
                $this->User->id = CakeSession::read('Auth.User.id');
                $this->User->saveField('rev', $result['revision']);
                $this->User->saveField('rev_date', 'now()');
            }
            $this->set('result', $result);
        } else {
            $this->redirect($this->Auth->loginRedirect);
        }
    }

}
