<?php

App::uses('AppController', 'Controller');

/**
 * Pthemes Controller
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * SVN Informations
 * $Date: 2014-07-23 13:58:59 +0200 (mer. 23 juil. 2014) $
 * $Revision: 527 $
 * $Author: ssampaio $
 * $HeadURL: svn+ssh://rdubourget@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/Controller/PthemesController.php $
 * $Id: PthemesController.php 527 2014-07-23 11:58:59Z ssampaio $
 *
 * @property Ptheme $Ptheme
 */
class PthemesController extends AppController {

	/**
	 *
	 * @var type
	 */
	public $helpers = array('Tree');

	/**
	 * index method
	 *
	 * @return void
	 */
	public function index() {
		$this->Ptheme->recursive = -1;

		$this->set('controller', $this->name);
		$this->set('action', $this->action);
		$this->set('title', __d('ptheme', 'Ptheme.list'));
		$this->set('addurl', Router::url(array('action' => 'add')));
		$this->set('deleteUrl', Router::url(array('controller' => 'Pthemes', 'action' => 'delete')));
		$this->set('addtitle', __d('ptheme', 'Ptheme.add'));
		$this->set('model', $this->modelClass);

		$fields = array(
			'id' => array(
				'label' => __d('ptheme', 'Ptheme.id'),
				'class' => 'uuid',
			),
			'name' => array(
				'label' => __d('ptheme', 'Ptheme.name')
			)
		);
		$this->set('fields', $fields);

		$treelist = $this->Ptheme->generateTreeList();
		$data = array();
		foreach ($treelist as $id => $name) {
			$data[] = array('Ptheme' => array('id' => $id, 'name' => $name));
		}
		$this->set('data', $data);
	}

	/**
	 * add method
	 *
	 * @return void
	 */
	public function add() {
		if ($this->request->is('post')) {
			$this->Ptheme->create();
			if ($this->Ptheme->save($this->request->data)) {
				$this->Session->setFlash(__d('ptheme', 'Ptheme.add.ok'), 'bootstrap_flash', array('class' => 'alert alert-success'));
				$this->redirect(array('action' => 'index'));
			} else {
				$this->Session->setFlash(__d('ptheme', 'Ptheme.add.error'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
			}
		}
		$parentPthemes = $this->Ptheme->generateTreeList(null, null, null, ' -- ');
		$this->set(compact('parentPthemes'));
	}

	/**
	 * edit method
	 *
	 * @throws NotFoundException
	 * @param string $id
	 * @return void
	 */
	public function edit($id = null) {
		if (!$this->Ptheme->exists($id)) {
			throw new NotFoundException(__('Invalid ptheme'));
		}
		if ($this->request->is('post') || $this->request->is('put')) {
			if ($this->Ptheme->save($this->request->data)) {
				$this->Session->setFlash(__d('ptheme', 'Ptheme.edit.ok'), 'bootstrap_flash', array('class' => 'alert alert-success'));
				$this->redirect(array('action' => 'index'));
			} else {
				$this->Session->setFlash(__d('ptheme', 'Ptheme.edit.error'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
			}
		} else {
			$options = array('conditions' => array('Ptheme.' . $this->Ptheme->primaryKey => $id));
			$this->request->data = $this->Ptheme->find('first', $options);
		}
//		$parentPthemes = $this->Ptheme->ParentPtheme->find('threaded', array('conditions' => array('ParentPtheme.' . $this->Ptheme->ParentPtheme->primaryKey . ' !=' => $id)));
		$parentPthemes = $this->Ptheme->generateTreeList(array('Ptheme.' . $this->Ptheme->primaryKey . ' !=' => $id), null, null, ' -- ');
		$this->set(compact('parentPthemes'));
	}

	/**
	 * delete method
	 *
	 * @throws NotFoundException
	 * @param string $id
	 * @return void
	 */
	public function delete($id = null) {
		$this->Ptheme->id = $id;
		if (!$this->Ptheme->exists()) {
			throw new NotFoundException(__('Invalid ptheme'));
		}
		if ($this->Ptheme->delete()) {
			$this->Session->setFlash(__d('ptheme', 'Ptheme.delete.ok'), 'bootstrap_flash', array('class' => 'alert alert-success'));
			$this->redirect(array('action' => 'index'));
		}
		$this->Session->setFlash(__d('ptheme', 'Ptheme.delete.error'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
		$this->redirect(array('action' => 'index'));
	}

	/**
	 *
	 */
	public function getList() {
		if ($this->request->isAjax() && $this->request->is('get')) {
			$this->set('pthemes', $this->Ptheme->find('threaded'));
		} else {
			throw new NotFoundException(__d('default', 'ajax.only'));
		}
	}

}
