<?php

App::uses('AppController', 'Controller');

/**
 * Seances Controller
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * SVN Informations
 * $Date: 2014-05-20 14:32:26 +0200 (mar. 20 mai 2014) $
 * $Revision: 424 $
 * $Author: ssampaio $
 * $HeadURL: svn+ssh://rdubourget@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/Controller/SeancesController.php $
 * $Id: SeancesController.php 424 2014-05-20 12:32:26Z ssampaio $
 *
 * @property Seance $Seance
 */
class SeancesController extends AppController {

    /**
     * Components
     *
     * @var array
     */
    public $components = array('Dataview', 'Horodatage');

    /**
     * 	Identifiant de l'utilisateur effectuant une importation
     *
     * @var string (UUID)
     */
    private $_WsImportUserId;

    /**
     *
     * @throws ForbiddenException
     */
    public function beforeFilter() {
        // $this->Auth->allow('reSendConvocations');
        parent::beforeFilter();

        if ($this->action == 'add') {

            App::uses('Security', 'Utility');

            $data = array(
                'username' => filter_input(INPUT_POST, 'username'),
                'password' => Security::hash(filter_input(INPUT_POST, 'password'), null, true),
                'conn' => filter_input(INPUT_POST, 'conn')
            );

            $this->setConn($data['conn']);
            $userModel = ClassRegistry::init('User');
            $userModel->useDbConfig = $data['conn'];

            $qdUser = array(
                'recursive' => -1,
                'contain' => array(
                    'Group'
                ),
                'conditions' => array(
                    'User.username' => $data['username'],
                    'User.password' => $data['password'],
                    'User.active' => true,
                    'Group.superuser' => true
                )
            );
            $user = $userModel->find('first', $qdUser);

            if (empty($user) || !$this->Auth->login($user)) {
                throw new ForbiddenException();
            }

            $this->_WsImportUserId = $user['User']['id'];
        }
    }

    /**
     * index method
     *
     * @return void
     */
    public function index($cancel = false) {
        if ($cancel) {
            $this->Dataview->cancelSearch();
        }

        $this->Dataview->keepShowStyle();

        $dataviewInfos = $this->Dataview->processFilters();
        $this->set('critere', $dataviewInfos['critere']);

        $this->Seance->recursive = -1;
        $this->paginate = array_merge($this->paginate, array('contain' => array('Type', 'Document'), 'conditions' => $dataviewInfos['conditions']));

        $this->set('controller', Inflector::camelize($this->name));
        $this->set('action', Inflector::camelize($this->action));
        $this->set('title', __d('seance', 'Seance.list'));
        $this->set('deleteUrl', Router::url(array('controller' => 'Seances', 'action' => 'delete')));
        $this->set('addurl', false);
        $this->set('addtitle', false);
        $this->set('model', $this->modelClass);
        $fields = array(
            'id' => array('label' => __d('seance', 'Seance.id')),
            'name' => array('label' => __d('seance', 'Seance.name')),
            'alias' => array('label' => __d('seance', 'Seance.alias')),
            'date_seance' => array('datetime' => true, 'label' => __d('seance', 'Seance.date_seance')),
            'type_id' => array(
                'label' => __d('seance', 'Seance.type_id'),
                'foreignkey' => true,
                'linkedDisplayField' => $this->Seance->Type->displayField,
                'linkedModel' => 'Type',
                'linkedController' => 'types',
                'linkedAction' => 'view'
            ),
            'document_id' => array(
                'label' => __d('seance', 'Seance.document_id'),
                'foreignkey' => true,
                'linkedDisplayField' => $this->Seance->Document->displayField,
                'linkedModel' => 'Document',
                'linkedController' => 'documents',
                'linkedAction' => 'view'
            ),
            'created' => array('datetime' => true, 'label' => __d('seance', 'Seance.created')),
            'modified' => array('datetime' => true, 'label' => __d('seance', 'Seance.modified'))
        );
        $this->set('fields', $this->Dataview->processFields($fields));
        $this->set('data', $this->paginate());
    }

    /**
     *
     * @param type $id
     * @throws BadMethodCallException
     * @throws NotFoundException
     */
    public function view($userId) {
        if (empty($userId)) {
            throw new BadMethodCallException();
        }

        $seances = $this->_getSeances($userId);
        $this->set(compact("seances"));
    }

    /**
     * add method
     *
     * @return void
     */
    public function add() {
        if ($this->request->is('post') && $this->request->is('ssl')) {
            DevTools::ldebug($this->request->data, true);
            $return = $this->Seance->genSeance(filter_input_array(INPUT_POST), $_FILES);
            $this->set('return', $return);
            if ($return['success'] && !empty($this->_WsImportUserId)) {
                DevTools::ldebug($return, true);
                $this->loadModel('User');
                $this->User->id = $this->_WsImportUserId;
                $this->User->saveField('last_import', date('Y-m-d H:i:s'));
            }
        } else {
            $this->autoRender = false;
//			$this->Components->load('Security')->blackHole();
        }
    }

    /**
     * edit method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function edit($id = null) {
        if (!$this->Seance->exists($id)) {
            throw new NotFoundException(__('Invalid seance'));
        }
        if ($this->request->is('post') || $this->request->is('put')) {

            if ($this->Seance->save($this->request->data)) {
                $this->Session->setFlash(__('The seance has been saved'));
                $this->redirect(array('action' => 'index'));
            } else {
                $this->Session->setFlash(__('The seance could not be saved. Please, try again.'));
            }
        } else {
            $options = array('conditions' => array('Seance.' . $this->Seance->primaryKey => $id));
            $this->request->data = $this->Seance->find('first', $options);
        }
        $types = $this->Seance->Type->find('list');
        $documents = $this->Seance->Document->find('list');
        $this->set(compact('types', 'documents'));
    }

    /**
     * delete method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function delete($id = null) {
        $this->Seance->id = $id;
        if (!$this->Seance->exists()) {
            throw new NotFoundException(__('Invalid seance'));
        }
        $this->request->onlyAllow('post', 'delete');
        if ($this->Seance->delete()) {
            $this->Session->setFlash(__('Seance deleted'));
            $this->redirect(array('action' => 'index'));
        }
        $this->Session->setFlash(__('Seance was not deleted'));
        $this->redirect(array('action' => 'index'));
    }

    /**
     * Ajoute une séance
     *
     */
    public function ajout() {
        if (!empty($this->request->data)) {

            // debug($_FILES); die;


            $return = $this->Seance->genSeance(filter_input_array(INPUT_POST), $_FILES);
            if ($return['success']) {
                $this->Session->setFlash(__d('seance', 'Seance.add.ok'), 'bootstrap_flash', array('class' => 'alert alert-success'));
            } else {
                $strError = __d('seance', 'Seance.add.error');
                if (Configure::read('debug') > 0) {
                    $strError .= " " . json_encode($return);
                }
                $this->Session->setFlash($strError, 'bootstrap_flash', array('class' => 'alert alert-danger'));
            }


            $this->Session->write('seanceAjoutId', $return['uuid']);
            $this->redirect(array('controller' => 'Seances', 'action' => 'liste'));
        } else {
            $qdUser = array(
                'recursive' => -1,
                'fields' => array(
                    'User.id',
                    'User.firstname',
                    'User.lastname',
                    'User.active',
                    'Group.name',
                    'Group.inseance'
                ),
                'conditions' => array(
                    'User.active' => true,
                    'Group.inseance' => true
                ),
                'contain' => array(
                    'Group'
                )
            );
            $users = $this->Seance->Type->TypesUser->User->find('all', $qdUser);

            $usersToSend = array();
            foreach ($users as $user) {
                $usersToSend[$user['User']['id']] = $user['User']['firstname'] . ' ' . $user['User']['lastname'];
            }
            $this->set('users', $usersToSend);
            $this->set('types', $this->Seance->Type->find('list'));
        }
    }

    /**
     * Affiche la liste des séances
     *
     */
    public function liste() {

        // on recupere la seance qui vient d'etre enregistrée (si pas d'enregistrement vide)
        $seanceredirectId = $this->Session->read('seanceAjoutId');
        $this->Session->delete('seanceAjoutId');

        $seances = $this->Seance->find('all', array('order' => 'Seance.date_seance DESC'));
        $this->set(compact('seances'));
        $this->set(compact('seanceredirectId'));
    }

    /**
     * Affiche les détails d'une séance
     *
     * @param string $id UUID
     */
    public function details($id) {
        $this->disableCache();
        $qd = array(
            'fields' => array(
                'Seance.id',
                'Seance.name',
                'Seance.date_seance',
                'Seance.document_id'
            ),
            'contain' => array(
                'Document' => array(
                    'fields' => array(
                        'Document.id',
                        'Document.name'
                    )
                ),
                'Convocation' => array(
                    'fields' => array(
                        'Convocation.id',
                        'Convocation.read',
                        'Convocation.ar_horodatage',
                        'Convocation.active',
                        'Convocation.ae_horodatage',
                        'Convocation.seance_id'
                    ),
                    'User' => array(
                        'fields' => array(
                            "User.id",
                            "User.name"
                        ),
                        'order' => array(
                            'User.firstname',
                            'User.lastname'
                        )
                    )
                ),
                'Projet' => array(
                    'order' => array('Projet.rank'),
                    'fields' => array(
                        "Projet.id",
                        "Projet.name",
                        "Projet.document_id",
                        "Projet.seance_id",
                        "Projet.vote",
                        "Projet.ptheme_id",
                        "Projet.rank"
                    ),
                    'Document' => array(
                        'fields' => array(
                            'Document.id',
                            'Document.name'
                        )
                    ),
                    'Annex' => array(
                        'fields' => array(
                            'Annex.id',
                            'Annex.name'
                        )
                    )
                )
            ),
            'conditions' => array(
                'Seance.id' => $id
            ),
            'order' => 'Seance.date_seance DESC'
        );
        $seance = $this->Seance->find('first', $qd);

        //recupération des branches complètes de pthemes pour chaque projet
        for ($j = 0; $j < count($seance['Projet']); $j++) {
            $seance['Projet'][$j]['theme'] = '';
            if (isset($seance['Projet'][$j]['ptheme_id'])) {
                $seance['Projet'][$j]['theme'] = trim(implode(', ', Hash::extract($this->Seance->Projet->Ptheme->getPath($seance['Projet'][$j]['ptheme_id']), "{n}.Ptheme.name")));
            }
        }
        $this->set(compact('seance'));
    }

    /**
     * Suppression d'une séance
     *
     * @throws NotFoundException
     * @param string $id UUID
     * @return void
     */
    public function suppression($id) {
        $this->Seance->id = $id;

        if (!$this->Seance->exists()) {
            throw new NotFoundException(__('Invalid seance'));
        }


        if ($this->Seance->delete()) {
            $this->Session->setFlash(__d('seance', 'Seance.delete.ok'), 'bootstrap_flash', array('class' => 'alert alert-success'));
        } else {
            $this->Session->setFlash(__d('seance', 'Seance.delete.error'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
        }

        $this->redirect(array('controller' => 'Seances', 'action' => 'liste'));
    }

    /**
     * Récupère la liste des séances (+détails) pour un utilisateur donné
     *
     * @param string $userId uuid de l'utilisateur
     * @return array Liste des convocations
     * @see APP/webroot/js/update.json
     */
    private function _getSeances($userId) {
        //récupération des identifiants de convocation et de séance concernant l'identifiant de l'utilisateur donné
        $qdConvocations = array(
            'fields' => array(
                'Convocation.id',
                'Convocation.seance_id'
            ),
            'conditions' => array(
                'Convocation.user_id' => $userId,
                'Convocation.active' => true
            ),
            'recursive' => -1
        );
        $convocs = $this->Seance->Convocation->find('list', $qdConvocations);

        //récupération des informations concernant les séances auxquelles est convoqué l'utilisateur désigné par l'identifiant donné
        $qd = array(
            'fields' => array(
                'Seance.id',
                'Seance.name',
                'Seance.date_seance',
                'Seance.document_id'
            ),
            'contain' => array(
                'Document' => array(
                    'fields' => array(
                        'Document.id',
                        'Document.name'
                    ),
                    'Imgpage' => array(
                        'fields' => array(
                            'Imgpage.name',
                            'Imgpage.imgpagenum',
                            'Imgpage.document_id'
                        )
                    ),
                ),
                'Convocation' => array(
                    'fields' => array(
                        'Convocation.id',
                        'Convocation.read',
                        'Convocation.seance_id'
                    ),
                    'User' => array(
                        'fields' => array(
                            "User.id",
                            "User.firstname",
                            "User.lastname"
                        )
                    )
                ),
                'Projet' => array(
                    'fields' => array(
                        "Projet.id",
                        "Projet.name",
                        "Projet.document_id",
                        "Projet.seance_id",
                        "Projet.vote",
                        "Projet.ptheme_id"
                    ),
                    'Document' => array(
                        'fields' => array(
                            'Document.id',
                            'Document.name'
                        ),
                        'Imgpage' => array(
                            'fields' => array(
                                "Imgpage.name",
                                "Imgpage.imgpagenum",
                                "Imgpage.id",
                                "Imgpage.document_id"
                            )
                        )
                    )
                )
            ),
            'conditions' => array(
                'Seance.id' => $convocs
            )
        );
        $return = $this->Seance->find('all', $qd);

        //traitements après récupération des informations pour transfert vers client
        for ($i = 0; $i < count($return); $i++) {
            //recupération des branches complètes de pthemes pour chaque projet de chaque seance
            for ($j = 0; $j < count($return[$i]['Projet']); $j++) {
                $return[$i]['Projet'][$j]['theme'] = trim(implode(', ', Hash::extract($this->Seance->Projet->Ptheme->getPath($return[$i]['Projet'][$j]['ptheme_id']), "{n}.Ptheme.name")));
            }

            //préparation de la liste des utilisateurs pour les annotations partagées
            //FIXME: faire en sorte que le client sache lire  lui-même les utilisateurs
            $return[$i]['Users'] = array();
            foreach ($return[$i]['Convocation'] as $convoc) {
                $return[$i]['Users'][] = $convoc['User'];
            }
        }
        return $return;
    }

    /**
     *
     * @param string $id uuid
     */
    public function edition($id) {
        $this->disableCache();
        if ((empty($id) || !$this->Seance->exists($id)) && empty($this->request->data)) {
            throw new NotFoundException(__d('seance', 'Seance.notFound'));
        } else if (!empty($this->request->data)) {

            //debug($_FILES); die
            // pour le cas de la modification d'une seance deja envoyée
            if (empty($_FILES['convocation'])) {
                $_FILES['convocation'] = array(
                    'name' => '',
                    'type' => '',
                    'tmp_name' => '',
                    'error' => (int) 4,
                    'size' => (int) 0
                );
            }






            $return = $this->Seance->editSeance($this->request->data['Seance']['id'], $_POST, $_FILES);
            if ($return['success']) {



                // on verifie si la seance à deja etait envoyée
                $seance = $this->Seance->find('first', array(
                    'recursive => -1',
                    'contain' => array(
                        'Convocation'
                    ),
                    'conditions' => array(
                        'Seance.id' => $this->request->data['Seance']['id']
                    )
                ));

                // si elle est active on rafraichie la seance               
                if ($seance['Convocation'][0]['active']) {
                    $this->refreshSeance($this->request->data['Seance']['id']);
                }

                $this->Session->setFlash(__d('seance', 'Seance.edit.ok'), 'bootstrap_flash', array('class' => 'alert alert-success'));
            } else {
                $strError = __d('seance', 'Seance.edit.error');
                if (Configure::read('debug') > 0) {
                    $strError .= " " . json_encode($return);
                }
                $this->Session->setFlash($strError, 'bootstrap_flash', array('class' => 'alert alert-danger'));
            }


            // si la seance n'est pas encore envoyée               
            if (!$seance['Convocation'][0]['active']) {
                $this->Session->write('seanceAjoutId', $this->request->data['Seance']['id']);
            }

            $this->redirect(array('controller' => 'Seances', 'action' => 'liste'));
        } else {
            $qd = array(
                'fields' => array(
                    'Seance.id',
                    'Seance.name',
                    'Seance.date_seance',
                    'Seance.document_id'
                ),
                'contain' => array(
                    'Document' => array(
                        'fields' => array(
                            'Document.id',
                            'Document.name'
                        )
                    ),
                    'Convocation' => array(
                        'fields' => array(
                            'Convocation.id',
                            'Convocation.read',
                            'Convocation.ar_horodatage',
                            'Convocation.active',
                            'Convocation.ae_horodatage',
                            'Convocation.seance_id'
                        ),
                        'User' => array(
                            'fields' => array(
                                "User.id",
                                "User.name"
                            ),
                            'order' => array(
                                'User.firstname',
                                'User.lastname'
                            )
                        )
                    ),
                    'Projet' => array(
                        'order' => array('Projet.rank'),
                        'fields' => array(
                            "Projet.id",
                            "Projet.name",
                            "Projet.document_id",
                            "Projet.seance_id",
                            "Projet.vote",
                            "Projet.ptheme_id",
                            "Projet.rank",
                        ),
                        'Document' => array(
                            'fields' => array(
                                'Document.id',
                                'Document.name'
                            )
                        ),
                        'Annex' => array(
                            'fields' => array(
                                'Annex.id',
                                'Annex.name'
                            )
                        ),
                      
                    )
                ),
                'conditions' => array(
                    'Seance.id' => $id
                ),
                'order' => 'Seance.date_seance DESC'
            );
            $seance = $this->Seance->find('first', $qd);
            //debug($seance); die;
//recupération des branches complètes de pthemes pour chaque projet
            for ($j = 0; $j < count($seance['Projet']); $j++) {
                $seance['Projet'][$j]['theme'] = '';
                if (isset($seance['Projet'][$j]['ptheme_id'])) {
                    $seance['Projet'][$j]['theme'] = trim(implode(', ', Hash::extract($this->Seance->Projet->Ptheme->getPath($seance['Projet'][$j]['ptheme_id']), "{n}.Ptheme.name")));
                }
            }

            $this->set('seance', $seance);

            $qdUser = array(
                'recursive' => 2,
                'fields' => array(
                    'User.id',
                    'User.firstname',
                    'User.lastname',
                    'Group.name'
                ),
                'contain' => array(
                    'Group' => array(
                        'conditions' => array(
                            'Group.inseance' => true
                        )
                    )
                )
            );
            $users = $this->Seance->Type->TypesUser->User->find('all', $qdUser);

            $usersToSend = array();
            foreach ($users as $user) {
                $usersToSend[$user['User']['id']] = $user['User']['firstname'] . ' ' . $user['User']['lastname'];
            }
            $this->set('users', $usersToSend);
            $this->set('types', $this->Seance->Type->find('list'));
        }
    }

    /**
     *
     * @param type $seance_id
     * @throws NotFoundException
     * 
     * envoie toutes les convocations
     * 
     */
    public function sendConvocations($seance_id) {

        if (empty($seance_id) || !$this->Seance->exists($seance_id)) {
            throw new NotFoundException(__d('seance', 'Seance.notFound'));
        }
        $this->autoRender = false;
        $qd = array(
            'recursive' => -1,
            'fields' => array(
                'Seance.id'
            ),
            'contain' => array(
                'Convocation' => array(
                    'fields' => array(
                        'Convocation.id'
                    ),
                    'conditions' => array(
                        'Convocation.active' => false
                    )
                )
            ),
            'conditions' => array(
                'Seance.id' => $seance_id
            )
        );


        $results = array();

        // Recherche toutes convocations associées
        $convocs = Hash::extract($this->Seance->find('first', $qd), "Convocation.{n}.id");



        for ($i = 0; $i < count($convocs); $i++) {
            $result[] = array($i => $this->Seance->Convocation->send($convocs[$i]));
        }




        echo json_encode($result);
    }

    /**
     * 
     * @param type $seance_id
     * @throws NotFoundException
     * 
     * 
     */
    public function refreshSeance($seance_id) {
        //  DevTools::ldebug($seance_id, true);

        if (empty($seance_id) || !$this->Seance->exists($seance_id)) {
            throw new NotFoundException(__d('seance', 'Seance.notFound'));
        }
        $this->autoRender = false;
        $qd = array(
            'recursive' => -1,
            'fields' => array(
                'Seance.id'
            ),
            'contain' => array(
                'Convocation' => array(
                    'fields' => array(
                        'Convocation.id'
                    ),
                    'conditions' => array(
                        'Convocation.active' => true
                    )
                )
            ),
            'conditions' => array(
                'Seance.id' => $seance_id
            )
        );

        // debug($seance_id); die;
        $result = array();

        // Recherche toutes convocations associées
        $convocs = Hash::extract($this->Seance->find('first', $qd), "Convocation.{n}.id");


        for ($i = 0; $i < count($convocs); $i++) {
            //TODO utiliser un chemin qui ne passe pas par l'horodatage 
            //  $result[] = array($i => $this->Seance->Convocation->resend($convocs[$i], $seance_id));
            $result[] = array($i => $this->Seance->Convocation->reSend($convocs[$i]));
        }
        echo json_encode($result);
    }

}
