<?php

App::uses('AppController', 'Controller');
App::uses('CakeEmail', 'Network/Email');
App::uses('Password', 'Utility');
App::uses('SSLCertificates', 'Utility');

/**
 * Srvusers Controller
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * SVN Informations
 * $Date: 2014-12-17 18:59:13 +0100 (mer. 17 déc. 2014) $
 * $Revision: 561 $
 * $Author: rdubourget $
 * $HeadURL: svn+ssh://rdubourget@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/Controller/SrvusersController.php $
 * $Id: SrvusersController.php 561 2014-12-17 17:59:13Z rdubourget $
 *
 * @property Srvuser $Srvuser
 */
class SrvusersController extends AppController {

    /**
     * Controller components
     *
     * @access public
     * @var array
     */
    public $components = array('Dataview');

    /**
     *
     */
    public function beforeFilter() {
        parent::beforeFilter();
        $this->Auth->allow(array('login', 'logout', 'mdplost', 'get_proof')); //get_proof a supprimer (vérifier les méthodes et les éventuelles répercutions en base)
    }

    /**
     * index method
     *
     * @return void
     */
    public function index($cancel = false) {
        if ($cancel) {
            $this->Dataview->cancelSearch();
        }

        $this->Dataview->keepShowStyle();

        $dataviewInfos = $this->Dataview->processFilters();
        $this->set('critere', $dataviewInfos['critere']);

        if ($this->Session->read('Auth.env') === 'walletmanager') {
            $dataviewInfos['conditions']['Srvuser.srvrole_id'] = IDELIBRE_ADMIN_GROUP_USER_ID;
        }

        $this->Srvuser->recursive = -1;
        $this->paginate = array_merge($this->paginate, array('contain' => array('Srvrole'), 'conditions' => $dataviewInfos['conditions']));


        $this->set('controller', Inflector::camelize($this->name));
        $this->set('action', Inflector::camelize($this->action));
        $this->set('title', __d('srvuser', 'Srvuser.list'));
        $this->set('addurl', Router::url(array('action' => 'add')));
        $this->set('deleteUrl', Router::url(array('controller' => 'Srvusers', 'action' => 'delete')));
        $this->set('addtitle', __d('srvuser', 'Srvuser.add'));
        $this->set('model', $this->modelClass);
        $fields = array(
            'id' => array('label' => __d('srvuser', 'Srvuser.id')),
            'username' => array('label' => __d('srvuser', 'Srvuser.username')),
            'firstname' => array('label' => __d('srvuser', 'Srvuser.firstname')),
            'lastname' => array('label' => __d('srvuser', 'Srvuser.lastname')),
            'mail' => array('label' => __d('srvuser', 'Srvuser.mail')),
            'srvrole_id' => array(
                'label' => __d('srvuser', 'Srvuser.srvrole_id'),
                'foreignkey' => true,
                'linkedDisplayField' => $this->Srvuser->Srvrole->displayField,
                'linkedModel' => 'Srvrole',
                'linkedController' => 'srvroles',
                'linkedAction' => 'view'
            ),
            'last_login' => array('datetime' => true, 'label' => __d('srvuser', 'Srvuser.last_login')),
            'last_logout' => array('datetime' => true, 'label' => __d('srvuser', 'Srvuser.last_logout')),
            'created' => array('datetime' => true, 'label' => __d('srvuser', 'Srvuser.created')),
            'modified' => array('datetime' => true, 'label' => __d('srvuser', 'Srvuser.modified'))
        );


        if ($this->Session->read('Auth.env') != 'superadmin') {
            $this->set('forbiddenActions', array('add' => false, 'edit' => false, 'delete' => true));
        }
        $this->set('fields', $this->Dataview->processFields($fields));
        $this->set('data', $this->paginate());
    }

    public function addAdminColl($data) {

        $User = ClassRegistry::init('User');

        $conn = $this->Srvuser->Collectivite->field('conn', array('Collectivite.id' => $data['CollAdmin']['collId']));

        $User->useDbConfig = $conn;
        $User->Aro->useDbConfig = $conn;



        //verification de l'existance de cet admin
        /*   $user = $User->find('first', array(
          'conditions' => array('User.username' => $this->request->data['Srvuser']['username'])
          ));
         */

        /*
          if (!empty($user)) {
          return false;
          } else {
         */


        $user['User']['firstname'] = $data['Srvuser']['firstname'];
        $user['User']['lastname'] = $data['Srvuser']['lastname'];
        $user['User']['mail'] = $data['Srvuser']['mail'];
        $user['User']['username'] = $data['Srvuser']['username'];

        $user['User']['password'] = AuthComponent::password($this->request->data['Srvuser']['password']);
        $user['User']['confirm'] = $this->request->data['Srvuser']['password'];
        $user['User']['group_id'] = IDELIBRE_GROUP_ADMIN_ID;


        $success = $User->save($user);


        if ($success) {

            $this->Session->setFlash(__d('srvuser', 'Srvuser.add.ok'), 'bootstrap_flash', array('class' => 'alert alert-success'));
            $this->redirect(array('action' => 'index'));
        } else {
            $this->Session->setFlash(__d('srvuser', 'Srvuser.add.error'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
            $errors = $User->validationErrors;
            return($errors);
        }
    }

    /**
     * add method
     *
     * @return void
     */
    public function add() {
        // changer pour des acl
        if($this->Session->read('Auth.env') == 'baseuser'){
           //  throw new NotFoundException('acces interdit');
        }
        
        if ($this->request->is('post')) {

            if ($this->request->data['Srvuser']['srvrole_id'] == IDELIBRE_GROUP_ADMIN_ID) {
                $this->Srvuser->validationErrors = $this->addAdminColl($this->request->data);
            } else {


                $this->Srvuser->begin();
                $this->Srvuser->create();

                //traitement du mot de passe dans le cas d'un utilisateur de collectivité
                if (!empty($this->request->data['User']['newpass'])) {
                    $this->request->data['User']['password'] = $this->request->data['User']['newpass'];
                }

                //traitement du mot de passe dans le cas d'un utilisateur superadmin
                if (!empty($this->request->data['Srvuser']['password'])) {
                    $this->request->data['Srvuser']['password'] = AuthComponent::password($this->request->data['Srvuser']['password']);
                }





                //creation de l'utilisateur
                if ($this->Srvuser->save($this->request->data)) {
                    $result = array();
                    if (//dans le cas de la creation d un utilisateur (trousseau)
                            !empty($this->request->data['Srvuser']) &&
                            $this->request->data['Srvuser']['srvrole_id'] === IDELIBRE_ADMIN_GROUP_USER_ID &&
                            !empty($this->request->data['Srvuser']['createForColl'])
                    ) {



                        //creation des utilisateurs dans les collectivités choisies (lors de la creation d'un trousseau)
                        $User = ClassRegistry::init('User');
                        foreach ($this->request->data['Srvuser']['createForColl'] as $collectiveId) {

                            $conn = $this->Srvuser->Collectivite->field('conn', array('Collectivite.id' => $collectiveId));
                            $User->useDbConfig = $conn;
                            $User->Aro->useDbConfig = $conn;

                            //verification de la présence de l'utilisateur  ou verifier plutot username comme ça on peut permettre de changer le nom
                            $qdUserExists = array(
                                'conditions' => array(
                                    'User.firstname' => $this->request->data['Srvuser']['firstname'],
                                    'User.lastname' => $this->request->data['Srvuser']['lastname']
                                )
                            );
                            $user = $User->find('first', $qdUserExists);

                            if (!empty($user)) { //si l'utilisateur exsite deja, on continue
                                $User->id = $user['User']['id'];
                                $success = true;
                            } else { //sinon on creer l'utilisateur dans la collectivite
                                $user = $User->create();
                                $user['User']['firstname'] = $this->request->data['Srvuser']['firstname'];
                                $user['User']['lastname'] = $this->request->data['Srvuser']['lastname'];
                                $user['User']['mail'] = $this->request->data['Srvuser']['mail'];
                                $user['User']['username'] = $this->request->data['Srvuser']['username'];
                                $newPassword = Password::generatePassword();
                                $user['User']['password'] = AuthComponent::password($newPassword);
                                $user['User']['confirm'] = $newPassword;
                                $user['User']['group_id'] = IDELIBRE_GROUP_AGENT_ID;
                                $success = $User->save($user);
                            }

                            if (!empty($success)) { //creation des mandats correspondants aux utilisateurs créés dans les bases collectivites
                                $mandat = $this->Srvuser->Mandat->create();
                                $mandat['Mandat']['user_id'] = $User->id;
                                $mandat['Mandat']['collectivite_id'] = $collectiveId;
                                $mandat['Srvuser']['id'] = $this->Srvuser->id;
                                $successMandat = $this->Srvuser->Mandat->saveAll($mandat);
                                $result[] = !empty($successMandat);
                            }
                        }
                    }

                    if (!in_array(false, $result, true)) {
                        //success
                        $this->Session->setFlash(__d('srvuser', 'Srvuser.add.ok'), 'bootstrap_flash', array('class' => 'alert alert-success'));
                        $this->Srvuser->commit();
                        $this->redirect(array('action' => 'index'));
                    }
                } else {
                    $this->Srvuser->rollback();
                    $this->Session->setFlash(__d('srvuser', 'Srvuser.add.error'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
                }
            }
        }

        $qdSrvroles = array();
        $srvuserColls = array();
        if ($this->Session->read('Auth.env') === 'walletmanager') { // en tant que gestionnaire de trousseau
            //on ne cherche que le profil "Trousseau" pour remplir le champ select
            $qdSrvroles['conditions'] = array('id' => IDELIBRE_ADMIN_GROUP_USER_ID);

            //on cherche les collectivités auxquelles l'utilisateur à accès
            $qdSrvuserColls = array(
                'fields' => array(
                    'Collectivite.id',
                    'Collectivite.name'
                ),
                'contain' => false,
                'joins' => array(
                    $this->Srvuser->join($this->Srvuser->CollectivitesSrvuser->alias),
                    $this->Srvuser->CollectivitesSrvuser->join($this->Srvuser->CollectivitesSrvuser->Collectivite->alias),
                ),
                'conditions' => array(
                    'Srvuser.id' => $this->Session->read('Auth.User.id')
                )
            );
            $srvuserColls = $this->Srvuser->find('list', $qdSrvuserColls);
        } else if ($this->Session->read('Auth.env') === 'superadmin') { //en tant que superadmin
            //on recupere toutes les collectivites
            $srvuserColls = $this->Srvuser->Collectivite->find('list');
        }

        $srvroles = $this->Srvuser->Srvrole->find('list', $qdSrvroles);

        // on rajoute l'administrateur de collectivité dans la liste
        $srvroles[IDELIBRE_GROUP_ADMIN_ID] = "administrateur de collectivité";
        $this->set(compact('srvroles', 'srvuserColls'));
    }

    /**
     * 
     *  liste des collectivités liées à un srvuser
     */
    public function listeMandat($id) {
        $qdMandats = array(
            'contain' => false,
            'joins' => array(
                $this->Srvuser->Mandat->join($this->Srvuser->Mandat->SrvusersMandat->alias)
            ),
            'conditions' => array(
                'SrvusersMandat.srvuser_id' => $id
            ),
            'fields' => array(
                'Mandat.user_id',
                'Mandat.collectivite_id'
            )
        );
        $mandats = $this->Srvuser->Mandat->find('all', $qdMandats);
        return $mandats;
    }

    // listes des collectivité gérées par le srvuser
    public function authorizedCollectivite() {
        if ($this->Session->read('Auth.env') === 'walletmanager') { // en tant que gestionnaire de trousseau
            //on cherche les collectivités auxquelles l'utilisateur (gestionnaire) à accès
            $qdSrvuserColls = array(
                'fields' => array(
                    'Collectivite.id',
                    'Collectivite.name'
                ),
                'contain' => false,
                'joins' => array(
                    $this->Srvuser->join($this->Srvuser->CollectivitesSrvuser->alias),
                    $this->Srvuser->CollectivitesSrvuser->join($this->Srvuser->CollectivitesSrvuser->Collectivite->alias),
                ),
                'conditions' => array(
                    'Srvuser.id' => $this->Session->read('Auth.User.id')
                )
            );
            $srvuserColls = $this->Srvuser->find('list', $qdSrvuserColls);
        } else if ($this->Session->read('Auth.env') === 'superadmin') { //en tant que superadmin
            //on recupere toutes les collectivites
            $srvuserColls = $this->Srvuser->Collectivite->find('list');
        }

        return $srvuserColls;
    }

    // listes des id des collectivités gérées par le srvuser
    public function authorizedCollectiviteId() {
        if ($this->Session->read('Auth.env') === 'walletmanager') { // en tant que gestionnaire de trousseau
            //on cherche les collectivités auxquelles l'utilisateur (gestionnaire) à accès
            $qdSrvuserColls = array(
                'fields' => array(
                    'Collectivite.id',
                ),
                'contain' => false,
                'joins' => array(
                    $this->Srvuser->join($this->Srvuser->CollectivitesSrvuser->alias),
                    $this->Srvuser->CollectivitesSrvuser->join($this->Srvuser->CollectivitesSrvuser->Collectivite->alias),
                ),
                'conditions' => array(
                    'Srvuser.id' => $this->Session->read('Auth.User.id')
                )
            );
            $srvuserColls = $this->Srvuser->find('all', $qdSrvuserColls);
        } else if ($this->Session->read('Auth.env') === 'superadmin') { //en tant que superadmin
            //on recupere toutes les collectivites
            $srvuserColls = $this->Srvuser->Collectivite->find('all', array('fields' => 'Collectivite.id'));
        }

        return $srvuserColls;
    }

    public function toRemoveMandat($mandatSelected) {
        $toRemove = array();
        $authorizedCollectivite = $this->authorizedCollectiviteId();

        if (!empty($mandatSelected)) {
            foreach ($authorizedCollectivite as $authorized) {

                if (!in_array($authorized['Collectivite']['id'], $mandatSelected)) {

                    array_push($toRemove, $authorized['Collectivite']['id']);
                }
            }
        } else {
            foreach ($authorizedCollectivite as $authorized) {
                array_push($toRemove, $authorized['Collectivite']['id']);
            }
        }

        return $toRemove;
    }

    /**
     * edit method   
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function edit($id = null) {
          // changer pour des acl
        if($this->Session->read('Auth.env') == 'baseuser'){
             //throw new NotFoundException('acces interdit');
        }


        if (!$this->Srvuser->exists($id)) {
            throw new NotFoundException(__('Invalid srvuser'));
        }
        if ($this->request->is('post') || $this->request->is('put')) {

            if (!empty($this->request->data['Srvuser']['newpass'])) {
                $this->request->data['Srvuser']['password'] = $this->request->data['Srvuser']['newpass'];
            }
            if (!empty($this->request->data['Srvuser']['password'])) {
                $this->request->data['Srvuser']['password'] = AuthComponent::password($this->request->data['Srvuser']['password']);
            }

            // si on a editer un portefeuille de mandat
            if (!empty($this->request->data['Srvuser']) &&
                    $this->request->data['Srvuser']['srvrole_id'] === IDELIBRE_ADMIN_GROUP_USER_ID) {

                $result = $this->editPortefeuilleMandat($id);
            }

                $result = $this->editPortefeuilleMandat($id);
            }

            if (!empty($this->request->data['Srvuser']) &&
                    $this->request->data['Srvuser']['srvrole_id'] === IDELIBRE_ADMIN_GROUP_WALLETMANAGER_ID) {

                $result = $this->editWalletManager($id);
            }


            if (!empty($this->request->data['Srvuser']) &&
                    $this->request->data['Srvuser']['srvrole_id'] === IDELIBRE_ADMIN_GROUP_ADMIN_ID) {

                $result = $this->editSuperAdmin($id);
            }



            if ($result) {
                //success
                $this->Session->setFlash(__d('srvuser', 'Srvuser.edit.ok'), 'bootstrap_flash', array('class' => 'alert alert-success'));
                $this->redirect(array('action' => 'index'));
            } else {
                $this->Session->setFlash(__d('srvuser', 'Srvuser.edit.error'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
                $this->redirect(array('action' => 'index'));
            }
        }

        $options = array('contain' => array('Collectivite'), 'conditions' => array('Srvuser.' . $this->Srvuser->primaryKey => $id));
        $this->request->data = $this->Srvuser->find('first', $options);

        $this->loadModel("Collectivite");


//        $usersByColls = $this->Collectivite->getUsersByCollectivite();




        $activeUsersByColls = $this->Collectivite->getActiveUsersByCollectivite();




        $srvuserColls = $this->authorizedCollectivite();
        $mandats = $this->listeMandat($id);


        $qdSrvroles = array();
        if ($this->Session->read('Auth.env') === 'walletmanager') { // en tant que gestionnaire de trousseau
            //on ne cherche que le profil "Porte-feuille de mandat" pour remplir le champ select
            $qdSrvroles['conditions'] = array('id' => IDELIBRE_ADMIN_GROUP_USER_ID);
        }
        $srvroles = $this->Srvuser->Srvrole->find('list', $qdSrvroles);
        $this->set(compact('srvroles', /* 'usersByColls', */ 'activeUsersByColls', 'mandats', 'srvuserColls'));
//        $this->set(compact('srvroles', 'usersByColls', 'activeUsersByColls', 'mandats', 'srvuserColls'));
    }

    public function editSuperAdmin($id) {
        return $this->Srvuser->save($this->request->data);
    }

    // modification d'un wallet manager ! 
    public function editWalletManager($id) {


        //debug($this->request->data); die;
        // on sauvegarde les nouveaux

        $this->Srvuser->save($this->request->data);


        // si ils sont a supprimé
        $authorizedCollectiviteId = $this->authorizedCollectiviteId();
        $selectedCollectivite = Hash::extract($this->request->data, 'Collectivite.Collectivite');


        $success = true;
        foreach ($selectedCollectivite as $selected) {
            if (in_array($selected, $authorizedCollectiviteId)) {

                //$this->Srvuser->begin();

                $conditions = array(
                    'CollectivitesSrvuser.srvuser_id' => $id,
                    'CollectivitesSrvuser.collectivite_id' => $selected,
                );

                $exists = $this->Srvuser->CollectivitesSrvuser->find('first', array('fields' => array('id'), 'conditions' => $conditions));
                if (!empty($exists)) {
                    $success = $this->Srvuser->CollectivitesSrvuser->deleteAll($conditions) && $success;
                }

                //$this->Srvuser->rollback();
            }
        }
        return $success;
    }

    //TODO à la fin de chaque edit envoyer un push pour forcer un auth global !
    public function editPortefeuilleMandat($id) {



        //on sauvegarde le nouveau mot de passe si il a changer 

        if (!empty($this->request->data['Srvuser']['newpass'])) {

            $this->Srvuser->id = $id;
            $this->Srvuser->savefield('password', AuthComponent::password($this->request->data['Srvuser']['newpass']));

            //$this->request->data['User']['password'] = $this->request->data['User']['newpass'];
        }




        $success = true;
        $oldMandatsliste = $this->listeMandat($id);
        $oldMandats = Hash::extract($oldMandatsliste, '{n}.Mandat.collectivite_id');



        $User = ClassRegistry::init('User');
        if (empty($this->request->data['Srvuser']['createForColl'])) {
            $this->request->data['Srvuser']['createForColl'] = array();
        }

        foreach ($this->request->data['Srvuser']['createForColl'] as $collectiveId) {


            // si il est deja dans cette collectivité
            // on l'ajoute à la collectivité
            //recherche de la collectivité correspondante
            $conn = $this->Srvuser->Collectivite->field('conn', array('Collectivite.id' => $collectiveId));
            $User->useDbConfig = $conn;
            $User->Aro->useDbConfig = $conn;



            //verification si le srvuser est deja present dans cette collectivité
            $qdUserExists = array(//verifier plutot l'email ?? oua lros aussi l'email en plus
                'conditions' => array(
                    'User.firstname' => $this->request->data['Srvuser']['firstname'],
                    'User.lastname' => $this->request->data['Srvuser']['lastname']
                )
            );
            $user = $User->find('first', $qdUserExists);




            if (!empty($user)) { //si l'utilisateur exsite deja, on continue


                $User->id = $user['User']['id'];
                $User->saveField('mail', $this->request->data['Srvuser']['mail']);
                $success = true;
            } else {


                // ajout de l'utilisateur dans la collectivité concernée 
                $user = $User->create();
                $user['User']['firstname'] = $this->request->data['Srvuser']['firstname'];
                $user['User']['lastname'] = $this->request->data['Srvuser']['lastname'];
                $user['User']['mail'] = $this->request->data['Srvuser']['mail'];
                $user['User']['username'] = $this->request->data['Srvuser']['username'];
                $newPassword = Password::generatePassword();
                $user['User']['password'] = AuthComponent::password($newPassword); //ou avec le meme password ?
                $user['User']['confirm'] = $newPassword;
                $user['User']['group_id'] = IDELIBRE_GROUP_AGENT_ID;
                $success = $User->save($user);
		$logvalidation = $User->validationErrors;



            }

            if (!empty($success)) {


                if (!in_array($collectiveId, $oldMandats)) {
                    $mandat = $this->Srvuser->Mandat->create();
                    $mandat['Mandat']['user_id'] = $User->id;
                    $mandat['Mandat']['collectivite_id'] = $collectiveId;
                    $mandat['Srvuser']['id'] = $id;
                    $success = $this->Srvuser->Mandat->saveAll($mandat);

                    //  $result[] = !empty($successMandat);
                } else {

$this->log(__LINE__, LOG_ERROR);
                    //debug($this->request->data['Srvuser']['mail']);die;
                    $this->Srvuser->id = $id;
                    $this->Srvuser->saveField('mail', $this->request->data['Srvuser']['mail']);
                }
            }
        }
        $toRemoveMandat = $this->toRemoveMandat($this->request->data['Srvuser']['createForColl']);
        //si le mandat est present dans la liste des manats on l'enleve et on le retire de la colletivité
        foreach ($toRemoveMandat as $Mandat) {
            //debug($toRemoveMandat);
            //debug($oldMandats);
            if (in_array($Mandat, $oldMandats)) {
                // on le supprime de la bdd de la collectivité
                // on charge la connection correspondante
                $conn = $this->Srvuser->Collectivite->field('conn', array('Collectivite.id' => $Mandat));
                $User->useDbConfig = $conn;
                $User->Aro->useDbConfig = $conn;
                Configure::write('conn', $conn);
                $this->setConn($conn);
                // on recherche l'user correspondant


                $user = $User->find('first', array(//utiliser pluto l'adresse email !!!!
                    'conditions' => array(
                        'User.firstname' => $this->request->data['Srvuser']['firstname'],
                        'User.lastname' => $this->request->data['Srvuser']['lastname'])
                        )
                );


                // on nesuprime l'user de sa bdd 
                $User->delete($user['User']['id']);

                //on reprend la connection à la base admin
                $this->setConn('default');

                //on supprime le supprime de la liste des mandats

                $mandat = $this->Srvuser->Mandat->find('first', array(
                    'conditions' => array(
                        'Mandat.user_id' => $user['User']['id'],
                    )
                ));

                //debug($mandat['Mandat']['id']);
                $this->Srvuser->Mandat->delete($mandat['Mandat']['id']);
            }
        }

        return $success;
    }

    /**
     * delete method
     *
     * @throws NotFoundException
     * @throws MethodNotAllowedException
     * @param string $id
     * @return void
     */
    public function delete($id = null) {
          // changer pour des acl
        if($this->Session->read('Auth.env') == 'baseuser'){
             //throw new NotFoundException('acces interdit');
        }
        $this->Srvuser->id = $id;
        if (!$this->Srvuser->exists()) {
            throw new NotFoundException(__('Invalid srvuser'));
        }
        if ($this->Srvuser->delete()) {
            $this->Session->setFlash(__d('srvuser', 'Srvuser.delete.ok'), 'bootstrap_flash', array('class' => 'alert alert-success'));
            $this->redirect(array('action' => 'index'));
        }
        $this->Session->setFlash(__d('srvuser', 'Srvuser.delete.error'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
        $this->redirect(array('action' => 'index'));
    }

    /**
     *
     * @param type $proof
     * @param type $sign
     * @return type
     */
    private function _selectCollWithCertificate($sign) {
        $selectedColl = array();
        $this->loadModel('Collectivite');
        $certsColls = $this->Collectivite->find('all', array('fields' => array('Collectivite.conn', 'Collectivite.name', 'Collectivite.public_key', 'Collectivite.login_suffix'), 'conditions' => array('Collectivite.active' => true, 'Collectivite.use_cert' => true)));
        foreach ($certsColls as $collitem) {
            if (SSLCertificates::verifySSLSign($this->Session->read('proof'), SSLCertificates::hex2bin($sign), $collitem['Collectivite']['public_key'])) {
                $selectedColl = $collitem;
            }
        }
        $loginParams = array('userModel' => 'User', 'roleModel' => 'Group', 'login_suffix' => !empty($selectedColl) ? $selectedColl['Collectivite']['login_suffix'] : '');
        $this->Session->delete('proof');
        return array($selectedColl, $loginParams);
    }

    /**
     *
     * @param type $loginParams
     * @return type
     */
    private function _selectColl() {
        $this->loadModel('Collectivite');
        $loginParams = $this->processLogin();
        $selectedColl = $this->Collectivite->find('first', array('conditions' => array('Collectivite.active' => true, 'Collectivite.use_cert' => false, 'Collectivite.login_suffix' => $loginParams['login_suffix'])));
        return array($selectedColl, $loginParams);
    }

    /**
     * Genere une chaîne texte aléatoire de 512 caractères
     * @deprecated since version 1.1
     * @return string
     */
    private function _genProof() {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $length = 512;
        $randomString = '';
        for ($i = 0; $i < $length; $i++) {
            $randomString .= $characters[rand(0, strlen($characters) - 1)];
        }
        return $randomString;
    }

    /**
     * @deprecated since version 1.1
     */
    public function get_proof() {
        $proof = $this->_genProof();

        $this->Session->write('proof', $proof);
        $this->set('proof', $proof);

        if ($this->isJsonp) {
            $this->set('callback', $this->request->query['callback']);
        }
    }

    /**
     *
     */
    public function login() {
        //  DevTools::ldebug('login');
        $this->Session->write('ctxTitle', __d('ctx', 'ctx.login'));
        $this->layout = 'login';

        if (!empty($this->request->data)) {

            //rajoute le @admin si il n'y a rien
            $matches = array();
            (preg_match('#(.*)@(.*)$#', $this->request->data['Srvuser']['username'], $matches));

            if (empty($matches)) {
                $this->request->data['Srvuser']['username'] = $this->request->data['Srvuser']['username'] . '@admin';
            }






            list($selectedColl, $loginParams) = $this->_selectColl();

            //connexion
            $logged = false;

            //todo: pour un log sans aucun @ envoyer au writelogin() params uniquement la collectivite correspondante au login et le srvrole
            // si le login n'existe pas envoyer une erreur !


            $this->writeLoginConfig($selectedColl, $loginParams);




            if (!empty($selectedColl) || $loginParams['login_suffix'] == "default") {
                $this->Auth->authenticate = array('Form' => array('userModel' => $loginParams['userModel']));
                if (Configure::read('conn') != 'default') {
                    $this->request->data['User'] = $this->request->data['Srvuser'];
                }
                if ($this->Auth->login()) {
                    $logged = true;
                }
            }

            $userInfos = array(
                'id' => '',
                'prenom' => '',
                'nom' => '',
                'logged' => false
            );

            //traitement post connexion
            if ($logged) {
                $this->writeUserConfig();
                $this->writeLoginSession();
                $this->setConn(Configure::read('conn'));


                $userModel = $this->Session->read('Auth.userModel');
                $this->loadModel($userModel);
                $this->{$userModel}->id = $this->Session->read('Auth.User.id');
                $this->{$userModel}->saveField('last_login', 'NOW()');
                $this->Session->setFlash(__d('login', 'login.success', true), 'bootstrap_flash', array('class' => 'alert alert-success'));

                if (!$this->isAjax && !$this->isJsonp) {
                    $redirect = $this->Auth->redirect();
                } else {
                    $userInfos = array(
                        'id' => $this->{$userModel}->id,
                        'prenom' => $this->{$userModel}->field('firstname'),
                        'nom' => $this->{$userModel}->field('lastname'),
                        'logged' => $logged
                    );
                }
            } else if ($this->Session->read('Message.flash.message') != __d('default', 'login.logout')) {
                if (!$this->isAjax && !$this->isJsonp) {
                    $this->Session->setFlash(__d('login', 'login.error'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
                    $redirect = "/Srvusers/login";
                }
            }

            if ($this->isAjax || $this->isJsonp) {
                $this->set("userInfos", $userInfos);
            } else {
                $this->redirect($redirect);
            }
        }
    }

    /**
     *
     */
    public function logout() {
        if ($this->isAjax || $this->isJsonp) {
//			$this->autoRender = false;
        } else {
            $this->Session->setFlash(__d('login', 'login.logout'), 'bootstrap_flash', array('class' => 'alert alert-info'));
        }

        if ($this->Session->check('Auth.userModel')) {
            $userModel = $this->Session->read('Auth.userModel');
            $this->loadModel($userModel);
            $this->{$userModel}->id = $this->Session->read('Auth.User.id');
            $this->{$userModel}->saveField('last_logout', 'NOW()');
        }
        $this->Session->delete('Auth');
        $this->Session->destroy();

        if (!$this->isAjax && !$this->isJsonp) {
            $this->redirect($this->Auth->logout());
        } else {
            $this->set("userInfos", array('logout', 'ok'));
        }
    }

    // communique via websocket (elephantIO) pour demander une AuthGlobal
    public function WSTellGlobalAuth($userId) {

        App::import('Vendor', 'ElephantIOClient', array('file' => 'ElephantIO/Client.php'));

        if (!class_exists('ElephantIO\Client')) {
            throw new RuntimeException(__d('websocket', 'ElephantIO.load.error'));
        }



//connexion au serveur websocket
        $elephantConfig = array(
            'url' => IDELIBRE_WS_PROTOCOL . '://' . IDELIBRE_WS_HOST . ':' . IDELIBRE_WS_PORT,
            'path' => 'socket.io',
            'protocol' => 1,
            'read' => false,
            'checkSSL' => false,
            'debug' => Configure::read('debug') > 0
        );


        DevTools::ldebug('request GlobalAuth');

        $this->elephant = new ElephantIO\Client($elephantConfig['url'], $elephantConfig['path'], $elephantConfig['protocol'], $elephantConfig['read'], $elephantConfig['checkSSL'], $elephantConfig['debug']);
        $this->elephant->init();

        $toSend = new stdClass();
        $toSend->name = 'request GlobalAuth'; //FIXME: Donnée du traitement
        $toSend->data = 'request GlobalAuth'; //FIXME: Donnée du traitement
        $toSend->userId = $userId;
        $toSend->from = 'srv';

        $this->elephant->send(ElephantIO\Client::TYPE_EVENT, null, null, json_encode(array('name' => 'request GlobalAuth', 'args' => json_encode($toSend))));
    }

    /**
     * Fonction permettant la gestion du mot de passe oublié
     *
     * @access public
     * @return void
     */
    public function mdplost($from = null) {
        $this->layout = 'login';
        $this->Session->write('ctxTitle', __d('ctx', 'ctx.mdplost'));

        $this->set('from', $from);

        if (!empty($this->request->data)) {

            $from_client = (isset($this->request->data['from_client']) && $this->request->data['from_client'] == 1);
            $redirect = $from_client ? '/idelibre_client' : $this->referer();
            $redirectSuccess = $from_client ? '/idelibre_client' : '/Srvusers/login';

            $this->loadModel('Collectivite');
            $conns = $this->Collectivite->find('list', array('fields' => array('Collectivite.conn', 'Collectivite.name'), 'conditions' => array('Collectivite.active' => true)));
            $this->set('conns', $conns);

            // Récupération du suffix permettant de connaître la collectivité sur laquelle se connecter
            $loginParams = $this->processLogin();
            $this->loadModel($loginParams['userModel']);



            if (empty($loginParams['login_suffix'])) {
                $this->Session->setFlash(__d('mdplost', 'save_error_suffix'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
            } else {
                $qdIsActive = array(
                    'conditions' => array(
                        'Collectivite.active' => true,
                        'Collectivite.login_suffix' => $loginParams['login_suffix']
                    )
                );
                $selectedColl = $this->Collectivite->find('first', $qdIsActive);


                if (!empty($selectedColl)) {
                    // Connexion à la collectivité en question pour rechercher l'utilisateur en question
                    $this->setConn($selectedColl['Collectivite']['conn']);
                    $this->{$loginParams['userModel']}->useDbConfig = $selectedColl['Collectivite']['conn'];

                    $this->{$loginParams['userModel']}->begin();
                    $qdUser = array(
                        'fields' => array(
                            $loginParams['userModel'] . '.id',
                            $loginParams['userModel'] . '.username',
                            $loginParams['userModel'] . '.active',
                            $loginParams['userModel'] . '.mail'
                        ),
                        'conditions' => array(
                            $loginParams['userModel'] . '.username' => $this->request->data['Srvuser']['username'],
                            $loginParams['userModel'] . '.mail' => $this->request->data['Srvuser']['mail']
                        ),
                        'contain' => false
                    );
                    $user = $this->{$loginParams['userModel']}->find('first', $qdUser);



                    // Si l'utilisateur existe, on vérifie le couple login/mot de passe
                    if (empty($user)) {
                        $this->Session->setFlash(__d('mdplost', 'save_error_couple'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
                        $this->{$loginParams['userModel']}->rollback();
                        $this->redirect($redirect);
                    } else if (!$user[$loginParams['userModel']]['active']) {
                        $this->Session->setFlash(__d('mdplost', 'save_error_inactif'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
                        $this->{$loginParams['userModel']}->rollback();
                        $this->redirect($redirect);
                    } else {
                        // génération du nouveau mot de passe
                        $newpass = Password::generatePassword();

                        $success = false;
                        $errorMessage = 'save_error';
                        $this->{$loginParams['userModel']}->id = $user[$loginParams['userModel']]['id'];
                        if ($this->{$loginParams['userModel']}->saveField('password', AuthComponent::password($newpass))) {
                            if ($this->{$loginParams['userModel']}->saveField('pwdmodified', true)) {
                                $success = true;
                                // Paramètres mail
                                try {
                                    $Email = new CakeEmail('motdepasseoublie');
                                    $Email->to(Configure::read('debug') == 0 ? $this->{$loginParams['userModel']}->field('mail') : $Email->from());
                                    $Email->subject('Changement de mot de passe dans i-delibRE');
                                    $mailBody = "Bonjour,\nsuite à votre demande, veuillez trouver ci-dessous vos nouveaux identifiants: " . "\n" . "  Rappel de votre identifiant : " . $user[$loginParams['userModel']]['username'] . "@" . $selectedColl['Collectivite']['login_suffix'] . "\n" . "  Votre nouveau mot de passe : " . $newpass . "\n\nA votre première connexion, il vous sera demandé de modifier ce mot de passe.";
                                    $result = $Email->send($mailBody);
                                    $success = !empty($result) && $success;
                                } catch (Exception $e) {
                                    $this->log($e->getMessage(), LOG_ERROR);
                                    $success = false;
                                    $errorMessage = 'save_error_mail';
                                }
                            }
                        }

                        //Sauvegarde du nouveau mot de passe de l'utilisateur et envoi du mail
                        if ($success) {
                            $this->{$loginParams['userModel']}->commit();
                            $this->Session->setFlash(__d('mdplost', 'save_success'), 'bootstrap_flash', array('class' => 'alert alert-info'));
                            $this->redirect($redirectSuccess);
                        } else {
                            $this->{$loginParams['userModel']}->rollback();
                            $this->Session->setFlash(__d('mdplost', $errorMessage), 'bootstrap_flash', array('class' => 'alert alert-danger'));
                        }
                    }
                }
            }
        }
    }

}
