<?php

App::uses('AppController', 'Controller');

/**
 * Types Controller
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * SVN Informations
 * $Date: 2013-10-21 16:27:59 +0200 (lun. 21 oct. 2013) $
 * $Revision: 301 $
 * $Author: ssampaio $
 * $HeadURL: svn+ssh://rdubourget@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/Controller/TypesController.php $
 * $Id: TypesController.php 301 2013-10-21 14:27:59Z ssampaio $
 *
 * @property Type $Type
 */
class TypesController extends AppController {

	/**
	 * Components
	 *
	 * @var array
	 */
	public $components = array('Dataview');

	/**
	 * index method
	 *
	 * @return void
	 */
	public function index($cancel = false) {

		if ($cancel) {
			$this->Dataview->cancelSearch();
		}

		$this->Dataview->keepShowStyle();

		$dataviewInfos = $this->Dataview->processFilters();
		$this->set('critere', $dataviewInfos['critere']);

		$this->Type->recursive = -1;
		$this->paginate = array_merge($this->paginate, array('conditions' => $dataviewInfos['conditions']));

		$this->set('controller', Inflector::camelize($this->name));
		$this->set('action', Inflector::camelize($this->action));
		$this->set('title', __d('type', 'Type.list'));
		$this->set('addurl', Router::url(array('action' => 'add')));
		$this->set('deleteUrl', Router::url(array('controller' => 'Types', 'action' => 'delete')));
		$this->set('addtitle', __d('type', 'Type.add'));
		$this->set('model', $this->modelClass);
		$fields = array(
			'id' => array('label' => __d('type', 'Type.id')),
			'name' => array('label' => __d('type', 'Type.name')),
			'created' => array('datetime' => true, 'label' => __d('type', 'Type.created')),
			'modified' => array('datetime' => true, 'label' => __d('type', 'Type.modified'))
		);

		$this->set('fields', $this->Dataview->processFields($fields));
		$this->set('data', $this->paginate());
	}

	/**
	 * add method
	 *
	 * @return void
	 */
	public function add() {
		if ($this->request->is('post')) {
			$this->Type->create();
			if ($this->Type->saveAll($this->request->data)) {
				$this->Session->setFlash(__d('type', 'Type.add.ok'), 'bootstrap_flash', array('class' => 'alert alert-success'));
				$this->redirect(array('action' => 'index'));
			} else {
				$this->Session->setFlash(__d('type', 'Type.add.error'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
			}
		}
		$qd = array(
			'contain' => array(
				$this->Type->User->Group->alias
			),
			'conditions' => array(
				'Group.inseance' => true,
				'Group.baseuser' => true,
				'Group.superuser' => false,
				'Group.admin' => false
			)
		);
		$users = $this->Type->User->find('list', $qd);
		$this->set(compact('users'));
	}

	/**
	 * edit method
	 *
	 * @throws NotFoundException
	 * @param string $id
	 * @return void
	 */
	public function edit($id = null) {
		if (!$this->Type->exists($id)) {
			throw new NotFoundException(__('Invalid type'));
		}
		if ($this->request->is('post') || $this->request->is('put')) {
			if ($this->Type->saveAll($this->request->data)) {
				$this->Session->setFlash(__d('type', 'Type.edit.ok'), 'bootstrap_flash', array('class' => 'alert alert-success'));
				$this->redirect(array('action' => 'index'));
			} else {
				$this->Session->setFlash(__d('type', 'Type.edit.error'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
			}
		} else {
			$data = $this->Type->TypesUser->find('list', array('conditions' => array('TypesUser.type_id' => $id), 'fields' => array('TypesUser.id', 'TypesUser.user_id')));
			$data = array('User' => array('User' => array_values($data)));
			$options = array('conditions' => array('Type.' . $this->Type->primaryKey => $id));
			$data = Hash::merge($data, $this->Type->find('first', $options));
			$this->request->data = $data;
		}
		$qd = array(
			'fields' => array(
				'User.id',
				'User.name'
			),
			'contain' => array(
				$this->Type->User->Group->alias
			),
			'conditions' => array(
				'Group.inseance' => true,
				'Group.baseuser' => true,
				'Group.superuser' => false,
				'Group.admin' => false
			)
		);
		$users = $this->Type->User->find('list', $qd);
		$this->set(compact('users', 'selected'));
	}

	/**
	 * delete method
	 *
	 * @throws NotFoundException
	 * @param string $id
	 * @return void
	 */
	public function delete($id = null) {
		$this->Type->id = $id;
		if (!$this->Type->exists()) {
			throw new NotFoundException(__('Invalid type'));
		}
		if ($this->Type->delete()) {
			$this->Session->setFlash(__d('type', 'Type.delete.ok'), 'bootstrap_flash', array('class' => 'alert alert-success'));
			$this->redirect(array('action' => 'index'));
		}
		$this->Session->setFlash(__d('type', 'Type.delete.error'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
		$this->redirect(array('action' => 'index'));
	}

	/**
	 * Récupération de la liste des utilisateurs liés à un type
	 *
	 * @param string $id UUID
	 * @throws NotFoundException
	 */
	public function users($id = null) {
		if (!$this->request->is('ajax') || !$this->request->is('get') || !$this->Type->exists($id)) {
			throw new NotFoundException();
		}
		$this->layout = '';
		$this->set('users', $this->Type->getUsersFromTypeId($id));
	}

}
