<?php

App::uses('AppController', 'Controller');

/**
 * Users Controller
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 *
 * SVN Informations
 * $Date: 2014-12-17 18:59:13 +0100 (mer. 17 déc. 2014) $
 * $Revision: 561 $
 * $Author: rdubourget $
 * $HeadURL: svn+ssh://rdubourget@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/Controller/UsersController.php $
 * $Id: UsersController.php 561 2014-12-17 17:59:13Z rdubourget $
 *
 * @property User $User
 */
class UsersController extends AppController {

    /**
     * Components
     *
     * @var array
     */
    public $components = array('Dataview');

    /**
     *
     */
    public function beforeFilter() {
        parent::beforeFilter();
        $this->Auth->allow(array('login', 'logout'));
    }

    /**
     * index method
     *
     * @return void
     */
    public function index($cancel = false) {
        if ($cancel) {
            $this->Dataview->cancelSearch();
        }

        $this->Dataview->keepShowStyle();

        $dataviewInfos = $this->Dataview->processFilters();
        $this->set('critere', $dataviewInfos['critere']);

        $this->User->recursive = -1;
        $this->paginate = array_merge($this->paginate, array('contain' => array('Group'), 'conditions' => $dataviewInfos['conditions']));

        $this->set('controller', Inflector::camelize($this->name));
        $this->set('action', Inflector::camelize($this->action));
        $this->set('title', __d('user', 'User.list'));
        $this->set('addurl', Router::url(array('action' => 'add')));
        $this->set('deleteUrl', Router::url(array('controller' => 'Users', 'action' => 'delete')));
        $this->set('addtitle', __d('user', 'User.add'));
        $this->set('model', $this->modelClass);

        $fields = array(
            'id' => array('label' => __d('user', 'User.id')),
            'username' => array('label' => __d('user', 'User.username')),
            'firstname' => array('label' => __d('user', 'User.firstname')),
            'lastname' => array('label' => __d('user', 'User.lastname')),
            'mail' => array('label' => __d('user', 'User.mail')),
            'group_id' => array(
                'label' => __d('user', 'User.group_id'),
                'foreignkey' => true,
                'linkedDisplayField' => $this->User->Group->displayField,
                'linkedModel' => 'Group',
                'linkedController' => 'groups',
             //   'linkedAction' => 'view'
            ),
            'active' => array('boolean' => true, 'label' => __d('user', 'User.active')),
            'last_login' => array('datetime' => true, 'label' => __d('user', 'User.last_login')),
            'last_logout' => array('datetime' => true, 'label' => __d('user', 'User.last_logout')),
            'created' => array('datetime' => true, 'label' => __d('user', 'User.created')),
            'modified' => array('datetime' => true, 'label' => __d('user', 'User.modified'))
        );
        $this->set('fields', $this->Dataview->processFields($fields));
        $this->set('data', $this->paginate());
    }

    /**
     * add method
     *
     * @return void
     */
    public function add($from = '') {
       
//throw new NotFoundException('moment indisponible');
 if ($this->request->is('post')) {
            $this->User->create();

            if (!empty($this->request->data['User']['password'])) {
                $this->request->data['User']['password'] = AuthComponent::password($this->request->data['User']['password']);
            }

            $success = false;
            $this->User->begin();
            
            if ($this->User->save($this->request->data)) {
                $this->User->commit();
                $this->Session->setFlash(__d('user', 'User.add.ok'), 'bootstrap_flash', array('class' => 'alert alert-success'));
                $this->redirect(array('action' => 'index'));
            } else {
                $this->User->rollback();
                $this->Session->setFlash(__d('user', 'User.add.error'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
            }
        }
        $groups = $this->User->Group->find('list');
        
        // enleve la possibilité de crrer un nouvel acteur
        unset($groups[IDELIBRE_GROUP_AGENT_ID]);
        $this->set(compact('groups', 'from'));
    }

    /**
     * edit method
     *
     * @throws NotFoundException
     * @param string $id
     * @return void
     */
    public function edit($id = null) {
 //throw new NotFoundException('moment indisponible');
       
 if (!$this->User->exists($id)) {
            throw new NotFoundException(__('Invalid user'));
        }
        if ($this->request->is('post') || $this->request->is('put')) {

	
	$this->User->id = $id;
	if($this->User->field('group_id') == IDELIBRE_GROUP_AGENT_ID){
	    $this->Session->setFlash(__d('user', 'User.edit.error'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
	    $this->redirect(array('action' => 'index'));
	}

	
            if (!empty($this->request->data['User']['newpass'])) {
                $this->request->data['User']['password'] = $this->request->data['User']['newpass'];
            }

            if (!empty($this->request->data['User']['password'])) {
                $this->request->data['User']['password'] = AuthComponent::password($this->request->data['User']['password']);
            }

            if ($this->User->save($this->request->data)) {
                $this->Session->setFlash(__d('user', 'User.edit.ok'), 'bootstrap_flash', array('class' => 'alert alert-success'));
                $this->redirect(array('action' => 'index'));
            } else {
                $this->Session->setFlash(__d('user', 'User.edit.error'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
            }
        } else {
            $options = array('conditions' => array('User.' . $this->User->primaryKey => $id));
            $this->request->data = $this->User->find('first', $options);
        }
        $groups = $this->User->Group->find('list');
       
	 unset($groups[IDELIBRE_GROUP_AGENT_ID]);

	 $this->set(compact('groups', 'userconfs'));
    }

    /**
     * delete method
     *
     * @throws NotFoundException
     * @throws MethodNotAllowedException
     * @param string $id
     * @return void
     */
    public function delete($id = null) {
        $this->User->id = $id;
        if (!$this->User->exists()) {
            throw new NotFoundException(__('Invalid user'));
        }


	  if($this->User->field('group_id') == IDELIBRE_GROUP_AGENT_ID){
            $this->Session->setFlash(__d('user', 'User.delete.error'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
            $this->redirect(array('action' => 'index'));
        }



        if ($this->User->delete()) {
            $this->Session->setFlash(__d('user', 'User.delete.ok'), 'bootstrap_flash', array('class' => 'alert alert-success'));
            $this->redirect(array('action' => 'index'));
        }
        $this->Session->setFlash(__d('user', 'User.delete.error'), 'bootstrap_flash', array('class' => 'alert alert-danger'));
        $this->redirect(array('action' => 'index'));
    }

    /**
     *
     */
    public function login() {
        $this->redirect('/Srvusers/login');
    }

    /**
     *
     */
    public function logout() {
        $this->redirect('/Srvusers/logout');
    }

    /**
     *
     */
    public function mdplost() {
        $this->redirect('/Srvusers/mdplost');
    }

}
