<?php

/**
 * Annotation Model
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 * @encoding UTF-8
 *
 * SVN Informations
 * $Date: 2014-06-05 12:04:00 +0200 (jeu. 05 juin 2014) $
 * $Revision: 445 $
 * $Author: ssampaio $
 * $HeadURL: svn+ssh://rdubourget@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/Model/Annotation.php $
 * $Id: Annotation.php 445 2014-06-05 10:04:00Z ssampaio $
 *
 * @property Imgpage $Imgpage
 */
App::uses('AppModel', 'Model');

class Annotation extends AppModel {

    /**
     * Data sent by client
     *
     * @var array
     */
    private $_data;

    /**
     * Last client revision
     *
     * @var integer
     */
    private $_clientRevision;

    /**
     * Annotations sent by client
     *
     * @var array
     */
    private $_annots;

    /**
     * What to process
     *
     * @var array
     */
    private $_mergeTodo = array();

    /**
     *
     * @var type
     */
    private $_resultEval;

    /**
     *
     * @var type
     */
    private $_userActionNeeded;

    /**
     * Last delta to compare whith client delta
     *
     * @var type
     */
    private $_lastDelta = array(
        'create' => array(),
        'update' => array(),
        'delete' => array()
    );

    /**
     * Validation rules
     *
     * @var array
     */
    public $validate = array(
        'user_id' => array(
            'uuid' => array(
                'rule' => array('uuid'),
            //'message' => 'Your custom message here',
            //'allowEmpty' => false,
            //'required' => false,
            //'last' => false, // Stop validation after this rule
            //'on' => 'create', // Limit validation to 'create' or 'update' operations
            ),
        ),
        'imgpage_id' => array(
            'uuid' => array(
                'rule' => array('uuid'),
            //'message' => 'Your custom message here',
            //'allowEmpty' => false,
            //'required' => false,
            //'last' => false, // Stop validation after this rule
            //'on' => 'create', // Limit validation to 'create' or 'update' operations
            ),
        ),
        'coordx' => array(
            'numeric' => array(
                'rule' => array('numeric'),
            //'message' => 'Your custom message here',
            //'allowEmpty' => false,
            //'required' => false,
            //'last' => false, // Stop validation after this rule
            //'on' => 'create', // Limit validation to 'create' or 'update' operations
            ),
        ),
        'coordy' => array(
            'numeric' => array(
                'rule' => array('numeric'),
            //'message' => 'Your custom message here',
            //'allowEmpty' => false,
            //'required' => false,
            //'last' => false, // Stop validation after this rule
            //'on' => 'create', // Limit validation to 'create' or 'update' operations
            ),
        ),
    );

    //The Associations below have been created with all possible keys, those that are not needed can be removed

    /**
     * belongsTo associations
     *
     * @var array
     */
    public $belongsTo = array(
        'Imgpage' => array(
            'className' => 'Imgpage',
            'foreignKey' => 'imgpage_id',
            'conditions' => '',
            'fields' => '',
            'order' => ''
        ),
        'Owner' => array(
            'className' => 'User',
            'foreignKey' => 'user_id',
            'conditions' => '',
            'fields' => '',
            'order' => ''
        )
    );

    /**
     * hasMany associations
     *
     * @var array
     */
//    public $hasMany = array(
//        'Annotaction' => array(
//            'className' => 'Annotaction',
//            'foreignKey' => 'annotation_id',
//            'dependent' => false,
//            'conditions' => '',
//            'fields' => '',
//            'order' => '',
//            'limit' => '',
//            'offset' => '',
//            'exclusive' => '',
//            'finderQuery' => '',
//            'counterQuery' => ''
//        )
//    );

    /**
     * hasAndBelongsToMany associations
     *
     * @var array
     */
    public $hasAndBelongsToMany = array(
        'User' => array(
            'className' => 'User',
            'joinTable' => 'annotations_users',
            'foreignKey' => 'annotation_id',
            'associationForeignKey' => 'user_id',
            'unique' => 'keepExisting',
            'conditions' => '',
            'fields' => '',
            'order' => '',
            'limit' => '',
            'offset' => '',
            'finderQuery' => '',
            'deleteQuery' => '',
            'insertQuery' => '',
            'with' => 'AnnotationsUser'
        )
    );

    /**
     * Checks coherence of sent data
     *
     * @param array $this->_data
     * @return array
     */
    private function _checkData() {
        $return = array(
            'success' => true,
            'message' => ''
        );
        //verification des données
        for ($i = 0; $i < count($this->_data['delta']['create']); $i++) {
            if (!isset($this->_annots[$this->_data['delta']['create'][$i]])) {
                $return['success'] = false;
                $return['message'] .= (!empty($return['message']) ? ', ' : '') . __d('sync', 'Sync.data.send.error.delta.create');
            }
        }
//		for ($i = 0; $i < count($this->_data['delta']['update']); $i++) {
//			if (!isset($this->_annots[$this->_data['delta']['update'][$i]])) {
//				$return['success'] = false;
//				$return['message'] .= (!empty($return['message']) ? ', ' : '') . __d('sync', 'Sync.data.send.error.delta.update');
//			}
//		}
//		for ($i = 0; $i < count($this->_data['delta']['delete']); $i++) {
//			if (!isset($this->_annots[$this->_data['delta']['delete'][$i]])) {
//				$return['success'] = false;
//				$return['message'] .= (!empty($return['message']) ? ', ' : '') . __d('sync', 'Sync.data.send.error.delta.delete');
//			}
//		}
        return $return;
    }

    /**
     * Evaluates what kind of action to do for each annotation
     *
     * @param array $this->_data
     * @return array
     */
    private function _evalData() {
        $result = array(
            'create' => array(
                'success' => false,
                'message' => ''
            ),
            'update' => array(
                'success' => false,
                'message' => ''
            ),
            'delete' => array(
                'success' => false,
                'message' => ''
            )
        );

        //evaluation des creations
        try {
            for ($i = 0; $i < count($this->_data['delta']['create']); $i++) {
                $this->_mergeTodo[$this->_data['delta']['create'][$i]] = $this->_eval($this->_annots[$this->_data['delta']['create'][$i]], 'create');
            }
            $result['create']['success'] = true;
        } catch (Exception $exc) {
            $result['create']['message'] = $exc->getTraceAsString();
        }

        DevTools::ldebug($this->_data['delta']['update']);

        //evalution des mises a jour
        try {
            for ($i = 0; $i < count($this->_data['delta']['update']); $i++) {
                if (!empty($this->_annots[$this->_data['delta']['update'][$i]])) { //on ne prend pas en compte le changement de status read
                    $this->log('update', 'debug');
                    $this->_mergeTodo[$this->_data['delta']['update'][$i]] = $this->_eval($this->_annots[$this->_data['delta']['update'][$i]], 'update');
                } else {
                    //FIXME: il faut changer la logique et créer une entrée spéciale dans le delta
                    $this->_setAnnotationRead($this->_data['delta']['update'][$i]);
                    $this->log('read', 'debug');
                }
            }
            $result['update']['success'] = true;
        } catch (Exception $exc) {
            $result['update']['message'] = $exc->getTraceAsString();
        }


        //evaluation des suppressions
        //comme on n'envoi aucune donnée dans $this->_data['annotations'] pour la suppression, on simule le résultat
        try {
            for ($i = 0; $i < count($this->_data['delta']['delete']); $i++) {
                $this->_mergeTodo[$this->_data['delta']['delete'][$i]] = $this->_eval(array('id' => $this->_data['delta']['delete'][$i]), 'delete');
            }
            $result['delete']['success'] = true;
        } catch (Exception $exc) {
            $result['delete']['message'] = $exc->getTraceAsString();
        }

        return $result;
    }

    /**
     * accusé de lecture d une annotation
     * @param type $annotation_id
     */
    private function _setAnnotationRead($annotation_id) {
        $annotUserId = $this->AnnotationsUser->field('id', array('AnnotationsUser.user_id' => CakeSession::read('Auth.User.id'), 'AnnotationsUser.annotation_id' => $annotation_id));
        $this->AnnotationsUser->id = $annotUserId;
        $this->AnnotationsUser->saveField('read', true);
    }

    /**
     * Evaluates what kind of action to do for an annotation
     *
     * @param array $annotation
     * @param string $action
     * @return array
     */
    private function _eval($annotation, $action) {

        $keys = array(
            'create' => 'c',
            'update' => 'u',
            'delete' => 'd'
        );

        $return = array($keys[$action] => $annotation['id']);

        $concerned_create = array();
        for ($i = 0; $i < count($this->_lastDelta['create']); $i++) {
            if ($this->_lastDelta['create'][$i]['Annotaction']['annotation_id'] == $annotation['id']) {
                $concerned_create[] = $this->_lastDelta['create'][$i]['Annotaction']['annotation_id'];
            }
        }
        $return['c' . $keys[$action]] = $concerned_create;

        $concerned_update = array();
        for ($i = 0; $i < count($this->_lastDelta['update']); $i++) {
            if ($this->_lastDelta['update'][$i]['Annotaction']['annotation_id'] == $annotation['id']) {
                $concerned_update[] = $this->_lastDelta['update'][$i]['Annotaction']['annotation_id'];
            }
        }
        $return['u' . $keys[$action]] = $concerned_update;

        $concerned_delete = array();
        for ($i = 0; $i < count($this->_lastDelta['delete']); $i++) {
            if ($this->_lastDelta['delete'][$i]['Annotaction']['annotation_id'] == $annotation['id']) {
                $concerned_delete[] = $this->_lastDelta['delete'][$i]['Annotaction']['annotation_id'];
            }
        }
        $return['d' . $keys[$action]] = $concerned_delete;

        return $return;
    }

    /**
     * Checks conflicts
     *
     * @return array
     */
    private function _checkUserActionNeeded() {
        $return = array('success' => false, 'state' => array(), 'message' => __d('sync', 'Sync.noConflict'));

        $chkUU = Hash::extract($this->_mergeTodo, '{s}.uu');
        $chkUD = Hash::extract($this->_mergeTodo, '{s}.ud');
        $chkDU = Hash::extract($this->_mergeTodo, '{s}.du');


        //verification des conflits DU (demande de maj d'une annotation supprimée)
        $i = 0;
        $checkingDU = true;
        $countDU = count($chkDU);
        while ($checkingDU && $i < $countDU) {
            if (!empty($chkDU[$i])) {
                $return['success'] = true;
                $return['state'][] = __d('sync', 'Sync.conflict.du');
                $checkingDU = false;
            }
            $i++;
        }

        //verification des conflits UU (demande de maj d'une annotation modifiée)
        $i = 0;
        $checkingUU = true;
        $countUU = count($chkUU);
        while ($checkingUU && $i < $countUU) {
            if (!empty($chkUU[$i])) {
                if ($this->_isOldClientRevision()) { //on verifie si des données n'ont pas été mise à jour entre temps
                    $return['success'] = true;
                    $return['state'][] = __d('sync', 'Sync.conflict.uu');
                    $checkingUU = false;
                }
            }
            $i++;
        }

        //verification des conflits UD (demande de suppression d'une annotation modifiée)
        $i = 0;
        $checkingUD = true;
        $countUD = count($chkUD);
        while ($checkingUD && $i < $countUD) {
            if (!empty($chkUD[$i])) {
                if ($this->_isOldClientRevision()) { //on verifie si des données n'ont pas été mise à jour entre temps
                    $return['success'] = true;
                    $return['state'][] = __d('sync', 'Sync.conflict.ud');
                    $checkingUD = false;
                }
            }
            $i++;
        }

        if ($return['success']) {
            $return['message'] = __d('sync', 'Sync.conflict.askUserAction');
        }


        if (Configure::read('debug') > 0) {
            $return['ctrl'] = array('chk UU' => $chkUU, 'chk UD' => $chkUD, 'chk DU' => $chkDU);
        }

        return $return;
    }

    /**
     * Checks if client sent revision number is older than server revision number
     *
     * @return boolean
     */
    private function _isOldClientRevision() {
        return (int) $this->_clientRevision < (int) Hash::extract($this->_lastDelta, 'create.0.Deltarevision.num');
    }

    /**
     * Stores data in temporary file (waiting for user reponse)
     *
     * @param array $this->_data
     * @return array
     */
    private function _store() {
        $return = array('success' => false);
        $dataToSore = array(
            'data' => $this->_data,
            'lastDelta' => $this->_lastDelta,
            'resultEval' => $this->_resultEval,
            'mergeTodo' => $this->_mergeTodo,
            'userActionNeeded' => $this->_userActionNeeded
        );

        try {
            file_put_contents('/tmp/' . IDELIBRE_SYNC_STORE_TOKEN_PREFIX . CakeSession::id(), bzcompress(json_encode($dataToSore)));
            $return['success'] = true;
            $return['message'] = __d('sync', 'Sync.store.ok');
            $return['token'] = CakeSession::id();
        } catch (Exception $exc) {
            $return['message'] = __d('sync', 'Sync.store.error') . ' : ' . $exc->getTraceAsString();
        }

        return $return;
    }

    /**
     * Restore previously stored data
     *
     * @param string $filename
     * @param boolean $delete
     * @return array
     */
    private function _restore($token = null, $delete = false) {
        $return = array('success' => false);

        if (!empty($token)) {
            if (!is_file('/tmp/' . IDELIBRE_SYNC_STORE_TOKEN_PREFIX . $token)) {
                $return['message'] = __d('sync', 'Sync.restore.error.token.notExists');
            } else {
                $dataToRestore = array();
                try {
                    $dataToRestore = json_decode(bzdecompress(file_get_contents('/tmp/' . IDELIBRE_SYNC_STORE_TOKEN_PREFIX . $token)), true);
                } catch (Exception $exc) {
                    $return['message'] = __d('sync', 'Sync.restore.error') . ' : ' . $exc->getTraceAsString();
                }

                if (!empty($dataToRestore)) {
                    $this->_data = $dataToRestore['data'];
                    $this->_lastDelta = $dataToRestore['lastDelta'];
                    $this->_resultEval = $dataToRestore['resultEval'];
                    $this->_mergeTodo = $dataToRestore['mergeTodo'];
                    $this->_userActionNeeded = $dataToRestore['userActionNeeded'];
                    $return['success'] = true;
                    $return['message'] = __d('sync', 'Sync.restore.ok');
                }

                if ($delete) {
                    $return['delete'] = ' ' . __d('sync', 'Sync.restore.error.token.delete.ok');
                    if (!unlink('/tmp/' . IDELIBRE_SYNC_STORE_TOKEN_PREFIX . $token)) {
                        $return['delete'] = ' ' . __d('sync', 'Sync.restore.error.token.delete.error');
                    }
                }
            }
        } else {
            $return['message'] = __d('sync', 'Sync.restore.error.token.void');
        }
        return $return;
    }

    /**
     * Drop previously stored data
     *
     * @param string $token
     * @return array
     */
    private function _drop($token = null) {
        $return = array('success' => false);
        if (!empty($token)) {
            if (!is_file('/tmp/' . IDELIBRE_SYNC_STORE_TOKEN_PREFIX . $token)) {
                $return['message'] = __d('sync', 'Sync.drop.error.token.notExists');
            } else {
                $return['message'] = ' ' . __d('sync', 'Sync.drop.error.token.delete.ok');
                if (!unlink('/tmp/' . IDELIBRE_SYNC_STORE_TOKEN_PREFIX . $token)) {
                    $return['message'] = ' ' . __d('sync', 'Sync.drop.error.token.delete.error');
                } else {
                    $return['success'] = true;
                }
            }
        } else {
            $return['message'] = __d('sync', 'Sync.drop.error.token.void');
        }
        return $return;
    }

    /**
     * Entry point for sync
     *
     * @param string $userId UUID
     * @param array $this->_data
     * @return array
     */
    public function sync($userId, $clientData) {
        $return = array();
        $this->_data = $clientData;
        $proceed = false;

        if (isset($this->_data['delta']) && /* isset($this->_data['annotations']) && */ isset($this->_data['revision'])) { //cas normal
            //récupération de la révision client
            $this->_clientRevision = $this->_data['revision'];

            //preparation des annotations
            for ($i = 0; $i < count($this->_data['annotations']); $i++) {
                $this->_annots[$this->_data['annotations'][$i]['id']] = $this->_data['annotations'][$i];
            }

            $isDataOk = $this->_checkData($this->_data);
            if ($isDataOk['success']) {
                //recupération de la précédente révision
                $this->_lastDelta['create'] = $this->Annotaction->Deltarevision->getLastDelta($userId, 'C');
                $this->_lastDelta['update'] = $this->Annotaction->Deltarevision->getLastDelta($userId, 'U');
                $this->_lastDelta['delete'] = $this->Annotaction->Deltarevision->getLastDelta($userId, 'D');

                //évaluation des données
                $this->_resultEval = $this->_evalData($this->_data);

                //evaluation de la demande d action client
                $this->_userActionNeeded = $this->_checkUserActionNeeded();

                if ($this->_userActionNeeded['success']) { //si conflit, on demande a l utilisateur
                    $stored = $this->_store(); //on stocke
//					$return['success'] = true;
                    $return['message'] = $this->_userActionNeeded['message'];
                    $return['token'] = !empty($stored['token']) ? $stored['token'] : null; //et on prepare le message retour avec le token de restauration
                } else { //sinon on demande le traitement des données envoyées
                    $proceed = true;
                }
            } else {
                $return['message'] = $isDataOk['message'];
            }
        } else if (isset($this->_data['response']) && isset($this->_data['token'])) { //apres reponse de l utilisateur
            if ($this->_data['response'] == "ok") { //reponse positive
                $result = $this->_restore($this->_data['token'], true); //on recupere les infos stockées
                if ($result['success']) {
                    $proceed = true; //on demande le traitement des infos
                } else {
                    $return['message'] = $result['message'];
                }
            } else { //reponse negative
                $result = $this->_drop($this->_data['token']); //on suprrime les infos stockées
                $return = array_merge($return, $result);
            }
        }

        if ($proceed) { //lancement du traitement
            $result = $this->_process($userId);
            $return = array_merge($return, $result);
        }

        return $return;
    }

    /**
     * Process data : save modifications
     *
     *
     * @param type $user_id
     * @return type
     */
    private function _process($userId) {
        $return = array('success' => false, 'message' => 'processing');

        $return[] = $this->_mergeTodo;
        $return[] = $this->_data;

        //preparation des annotations
        $annotations = array();
        for ($i = 0; $i < count($this->_data['annotations']); $i++) {
            $annotations[$this->_data['annotations'][$i]['id']] = $this->_data['annotations'][$i];
        }

        $this->begin();
        //creation de la revision
        $deltarevision = $this->Annotaction->Deltarevision->create();
        $deltarevision['Deltarevision']['delta'] = json_encode($this->_data['delta']);
        $deltarevision['Deltarevision']['user_id'] = $userId;

        if ($this->Annotaction->Deltarevision->save($deltarevision)) {
            $result['Deltarevision'] = array(
                'id' => $this->Annotaction->Deltarevision->id,
                'num' => $this->Annotaction->Deltarevision->field('num')
            );
        }

        //enregistrements des annotations
        $result = array();
        foreach ($this->_mergeTodo as $annot_id => $todo) {
            $type = '';
            //traitement de l'annotation
            $saveAnnotationResult = false;
            if (isset($todo['c']) && $todo['c'] == $annot_id) { //create
                $this->create();
                $annot = $this->_convertAnnotation($annotations[$annot_id]);
                $annot['Annotation']['read'] = true; //on note les annotations que l on a créer comme lues
                $saveAnnotationResult = $this->save($annot);
                $type = 'C';
            } else if (isset($todo['u']) && $todo['u'] == $annot_id) { //update
                $this->create();
                $annot = $this->_convertAnnotation($annotations[$annot_id]);
                $saveAnnotationResult = $this->save($annot);
                $type = 'U';
            } else if (isset($todo['d']) && $todo['d'] == $annot_id) { //delete
                $saveAnnotationResult = $this->delete($annot_id, false);
                $type = 'D';
            }

            //creation de la lisaison Annotations -> Deltarevisions
            $annotaction = $this->Annotaction->create();
            $annotaction['Annotaction']['deltarevision_id'] = $this->Annotaction->Deltarevision->id;
            $annotaction['Annotaction']['annotation_id'] = $annot_id;
            $annotaction['Annotaction']['type'] = $type;
            $saveAnnotactionResult = $this->Annotaction->save($annotaction);

            $result[$annot_id] = array(
                'success' => !empty($saveAnnotationResult) && !empty($saveAnnotactionResult),
                'Annotation' => Configure::read('debug') > 0 ? $saveAnnotationResult : !empty($saveAnnotationResult),
                'Annotaction' => Configure::read('debug') > 0 ? $saveAnnotactionResult : !empty($saveAnnotactionResult)
            );
        }
        $return[] = $result;


        if (!in_array(false, Hash::extract($result, '{n}.{s}.success'), true)) {
            $return['success'] = true;
            $this->commit();
        } else {
            $this->rollback();
        }

        return $return;
    }

    private function _convertAnnotation($annotationFromJson) {


        $return = array(
            'Annotation' => array(
                'id' => !empty($annotationFromJson['id']) ? $annotationFromJson['id'] : '',
                'x' => $annotationFromJson['x'],
                'y' => $annotationFromJson['y'],
                'height' => $annotationFromJson['height'],
                'width' => $annotationFromJson['width'],
                'content' => $annotationFromJson['content'],
                'comment' => $annotationFromJson['comment'],
                'shared' => $annotationFromJson['shared'],
                'rank' => $annotationFromJson['rank'],
                'date' => date('Y-m-d H:i:s', mktime(0, 0, (int) ($annotationFromJson['date'] / 1000))),
                'date_modif' => date('Y-m-d H:i:s', mktime(0, 0, (int) ($annotationFromJson['date_modif'] / 1000))),
                'imgpage_id' => $annotationFromJson['imgpage_id'],
                'user_id' => $annotationFromJson['user_id'],
                'annot_client_id' => !empty($annotationFromJson['annot_client_id']) ? $annotationFromJson['annot_client_id'] : ''
            ),
            'User' => array(
                'User' => $annotationFromJson['users']
            )
        );

        return $return;
    }

    /**
     * Créer une annotation
     *
     * @param array $annotation
     * @return mixed id or null
     */
//	private function _createAnnotation($annotation) {
//		$return = null;
//		if (!empty($result)) {
//			$return = $this->id;
//		}
//
//		return $return;
//	}

    /**
     * Créer une nouvelle annotation et remplace l'identifiant par celui envoyé par le client
     *
     * @param array $annotation
     */
//	private function _updateAnnotation($annotation) {
//		$return = false;
//		$id = $this->_createAnnotation($annotation);
//
//		if (!empty($id)) {
//			$this->id = $id;
//			$return = $this->saveField('id', $annotation['real_id']);
//		}
//
//		return $return;
//	}

    /**
     *
     * @param type $annot
     * @return type
     */
    public function createJsonAnnot($annot) {
        $this->create();

        $annotToSave = array(
            'Annotation' => array(
//                'id' => null,
                'x' => $annot->x,
                'y' => $annot->y,
                'height' => $annot->hauteur,
                'width' => $annot->largeur,
                'content' => $annot->texteChamps,
                'comment' => $annot->texteInformations,
                'shared' => $annot->type,
//                'rank' => $annotationFromJson['rank'],
//                'date' => date('Y-m-d H:i:s', mktime(0, 0, (int) ($annot->date / 1000))),
//                'date_modif' => date('Y-m-d H:i:s', mktime(0, 0, (int) ($annot->date_modif / 1000))),
                'imgpage_id' => $annot->imgpage_id,
                'user_id' => $annot->id_utilisateur,
                'annot_client_id' => $annot->id
            ),
//            'User' => $annot->listePartage
        );

        return $this->save($annotToSave);
    }

    /**
     *
     * @param type $annot
     * @return type
     */
    public function updateJsonAnnot($annot) {
        $this->create();
        $annotToSave = array(
            'Annotation' => array(
                'id' => $annot->id,
                'x' => $annot->x,
                'y' => $annot->y,
                'height' => $annot->hauteur,
                'width' => $annot->largeur,
                'content' => $annot->texteChamps,
                'comment' => $annot->texteInformations,
                'shared' => $annot->type,
//                'rank' => $annotationFromJson['rank'],
//                'date' => date('Y-m-d H:i:s', mktime(0, 0, (int) ($annot->date / 1000))),
//                'date_modif' => date('Y-m-d H:i:s', mktime(0, 0, (int) ($annot->date_modif / 1000))),
                'imgpage_id' => $annot->imgpage_id,
                'user_id' => $annot->id_utilisateur,
//                'annot_client_id' => '' //FIXME: le client n'envoie pas son ancien id de creation
            ),
        );
        return $this->save($annotToSave);
    }

    /**
     *
     * @param type $annotId
     * @param type $users
     * @return boolean
     */
    public function share($annotId, $users) {
        $return = false;
        $this->begin();
        $result = array();

        foreach ($users as $user) {

            if (!empty($user[2])) { //valeur de la checkbox dans la liste de partage de l'annotation
                $linkId = $this->AnnotationsUser->field('id', array('annotation_id' => $annotId, 'user_id' => $user[1]));
                $this->AnnotationsUser->create();
                if (!empty($linkId)) { //si modification on reinitialise l indicateur de lecture de l annotation
                    $data = array('AnnotationsUser' => array(
                            'id' => $linkId,
                            'read' => false
                        )
                    );
                } else { //sinon on partage l annotation aux nouveaux utilisateur
                    $data = array('AnnotationsUser' => array(
                            'annotation_id' => $annotId,
                            'user_id' => $user[1],
                            'read' => false
                        )
                    );
                }
                $res = $this->AnnotationsUser->save($data);
                $result[] = !empty($res);
            }
        }

        if (!in_array(false, $result, true)) {
            $return = $users;
            $this->commit();
        } else {
            $this->rollback();
        }
        return $return;
    }

    /**
     *
     * @param type $annotId
     * @param type $users
     * @return type
     */
    public function unshare($annotId, $users) {
        $return = false;
        $this->begin();

        $result = array();
        $currentShares = $this->getSharedUsers($annotId);

        foreach ($users as $user) {
            if (empty($user[2])) { //valeur de la checkbox dans la liste de partage de l'annotation
                foreach ($currentShares as $link => $share) {
                    if ($user[1] == $share) {
                        $result[$share] = $this->AnnotationsUser->delete($link);
                    }
                }
            }
        }

        if (!in_array(false, $result, true)) {
            $this->commit();
            $return = array_keys($result);
        } else {
            $this->rollback();
        }

        return $return;
    }

    /**
     *
     * @param type $annotationId
     * @return type
     */
    public function getSharedUsers($annotationId) {
        $qd = array(
            'recursive' => -1,
            'fields' => array(
                'id',
                'user_id'
            ),
            'conditions' => array(
                'annotation_id' => $annotationId
            )
        );

        return $this->AnnotationsUser->find('list', $qd);
    }

    /**
     *
     * @param type $userId
     * @param type $annotationId
     */
    public function setRead($userId, $annotationId) {
        $annotationUser = $this->AnnotationsUser->find('first', array('conditions' => array('user_id' => $userId, 'annotation_id' => $annotationId, 'read' => false)));
        $return = false;
        if (!empty($annotationUser)) {

            $annotationUser['AnnotationsUser']['read'] = true;

            DevTools::ldebug($annotationUser);


            $this->AnnotationsUser->create();
            $return = $this->AnnotationsUser->save($annotationUser);
        }
        return $return;
    }

}
