<?php

/**
 * Collectivite Model
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 * @encoding UTF-8
 *
 * SVN Informations
 * $Date: 2014-07-30 12:00:41 +0200 (mer. 30 juil. 2014) $
 * $Revision: 542 $
 * $Author: ssampaio $
 * $HeadURL: svn+ssh://rdubourget@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/Model/Collectivite.php $
 * $Id: Collectivite.php 542 2014-07-30 10:00:41Z ssampaio $
 *
 * @property Srvuser $Srvuser
 */
App::uses('AppModel', 'Model');

class Collectivite extends AppModel {

    /**
     *
     * @var type
     */
    public $useDbConfig = 'default';

    /**
     *
     * @var type
     */
    public $validationDomain = 'collectivite';

    /**
     * Validation rules
     *
     * @var array
     */
    public $validate = array(
        'name' => array(
            'notempty' => array(
                'rule' => array('notempty'),
                'message' => 'Collectivite.name.notempty'
            ),
            'isUnique' => array(
                'rule' => array('isUnique'),
                'message' => 'Collectivite.name.isUnique'
            )
        ),
        'conn' => array(
            'notempty' => array(
                'rule' => array('notempty'),
                'message' => 'Collectivite.conn.notempty'
            ),
            'isUnique' => array(
                'rule' => array('isUnique'),
                'message' => 'Collectivite.conn.isUnique'
            )
        ),
        'login_suffix' => array(
            'notempty' => array(
                'rule' => array('notempty'),
                'message' => 'Collectivite.login_suffix.notempty'
            ),
            'isUnique' => array(
                'rule' => array('isUnique'),
                'message' => 'Collectivite.login_suffix.isUnique'
            )
        )
    );

    //The Associations below have been created with all possible keys, those that are not needed can be removed

    /**
     * hasAndBelongsToMany associations
     *
     * @var array
     */
    public $hasAndBelongsToMany = array(
        'Srvuser' => array(
            'className' => 'Srvuser',
            'joinTable' => 'collectivites_srvusers',
            'foreignKey' => 'collectivite_id',
            'associationForeignKey' => 'srvuser_id',
            'unique' => 'keepExisting',
            'conditions' => '',
            'fields' => '',
            'order' => '',
            'limit' => '',
            'offset' => '',
            'finderQuery' => '',
            'deleteQuery' => '',
            'insertQuery' => '',
            'with' => 'CollectivitesSrvuser'
        )
    );

    /**
     *
     * @return type
     */
    public function getUsersByCollectivite() {
        $return = array();
        $collectivites = $this->find('all', array('fields' => array('id', 'conn', 'name', 'active')));

        foreach (Set::extract('{n}.Collectivite', $collectivites) as $coll) {
            $User = ClassRegistry::init('User');
            $User->useDbConfig = $coll['conn'];

            $qd = array(
                'fields' => array(
                    'User.id',
                    '("User"."firstname" || \' \' || "User"."lastname") AS "User__fullname"'
                ),
                'conditions' => array(
                    'User.group_id' => IDELIBRE_GROUP_AGENT_ID
                )
            );
            $users = $User->find('all', $qd);
            if (!empty($users)) {
                array_walk($users, array($this, '_concatenate_ids'), $coll['id']);
                $return[$coll['name'] . (!$coll['active'] ? ' (non active)' : '')] = Set::combine($users, '{n}.User.id', '{n}.User.fullname');
            }
        }
        return $return;
    }

    /**
     *
     * @return type
     */
    public function getActiveUsersByCollectivite() {
        $return = array();
        $collectivites = $this->find('all', array('fields' => array('id', 'conn', 'name', 'active')));

        foreach (Set::extract('{n}.Collectivite', $collectivites) as $coll) {
            $User = ClassRegistry::init('User');
            $User->useDbConfig = $coll['conn'];

            $qd = array(
                'fields' => array(
                    'User.id',
                    'User.active'
                ),
                'conditions' => array(
                    'User.group_id' => IDELIBRE_GROUP_AGENT_ID
                )
            );
            $users = $User->find('all', $qd);
            if (!empty($users)) {
                array_walk($users, array($this, '_concatenate_ids'), $coll['id']);

                if (!$coll['active']) {

                    array_walk($users, array($this, '_desactive_users'));
                }

                $return[$coll['name'] . (!$coll['active'] ? ' (non active)' : '')] = Set::combine($users, '{n}.User.id', '{n}.User.active');
            }
        }
        return $return;
    }

    /**
     *
     * @param type $user
     * @param type $key
     * @param type $collectiviteId
     */
    private function _concatenate_ids(&$user, $key, $collectiviteId) {
        $user['User']['id'] = $collectiviteId . "=" . $user['User']['id'];
    }

    /**
     *
     * @param type $user
     * @param type $key
     * @param type $collectiviteId
     */
    private function _desactive_users(&$user, $key) {
        $user['User']['active'] = false;
    }

}
