<?php

/**
 * Document Model
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 * @encoding UTF-8
 *
 * SVN Informations
 * $Date: 2014-04-16 10:38:43 +0200 (mer. 16 avril 2014) $
 * $Revision: 384 $
 * $Author: ssampaio $
 * $HeadURL: svn+ssh://rdubourget@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/Model/Document.php $
 * $Id: Document.php 384 2014-04-16 08:38:43Z ssampaio $
 *
 * @property Convocation $Convocation
 * @property Projet $Projet
 * @property Imgpage $Imgpage
 */
App::uses('AppModel', 'Model');
App::uses('Folder', 'Utility');
App::uses('File', 'Utility');

class Document extends AppModel {

	/**
	 *
	 * @var type
	 */
	public $actsAs = array('DbFile');

	/**
	 * Display field
	 *
	 * @var string
	 */
	public $displayField = 'name';

	/**
	 *
	 * @var type
	 */
	private $_getAnnotationsReturn = array();

	/**
	 * Validation rules
	 *
	 * @var array
	 */
	public $validate = array(
		'name' => array(
			'notempty' => array(
				'rule' => array('notempty'),
			//'message' => 'Your custom message here',
			//'allowEmpty' => false,
			//'required' => false,
			//'last' => false, // Stop validation after this rule
			//'on' => 'create', // Limit validation to 'create' or 'update' operations
			),
		),
		'size' => array(
			'numeric' => array(
				'rule' => array('numeric'),
			//'message' => 'Your custom message here',
			//'allowEmpty' => false,
			//'required' => false,
			//'last' => false, // Stop validation after this rule
			//'on' => 'create', // Limit validation to 'create' or 'update' operations
			),
		),
		'type' => array(
			'notempty' => array(
				'rule' => array('notempty'),
			//'message' => 'Your custom message here',
			//'allowEmpty' => false,
			//'required' => false,
			//'last' => false, // Stop validation after this rule
			//'on' => 'create', // Limit validation to 'create' or 'update' operations
			),
		),
	);

	//The Associations below have been created with all possible keys, those that are not needed can be removed

	/**
	 * hasOne associations
	 *
	 * @var array
	 */
	public $hasOne = array(
		'Seance' => array(
			'className' => 'Seance',
			'foreignKey' => 'document_id',
			'conditions' => '',
			'fields' => '',
			'order' => ''
		),
		'Projet' => array(
			'className' => 'Projet',
			'foreignKey' => 'document_id',
			'conditions' => '',
			'fields' => '',
			'order' => ''
		)
	);

	/**
	 * hasMany associations
	 *
	 * @var array
	 */
	public $hasMany = array(
		'Imgpage' => array(
			'className' => 'Imgpage',
			'foreignKey' => 'document_id',
			'dependent' => false,
			'conditions' => '',
			'fields' => '',
			'order' => '',
			'limit' => '',
			'offset' => '',
			'exclusive' => '',
			'finderQuery' => '',
			'counterQuery' => ''
		)
	);

	/**
	 * Conversion et enregistrement des Imgpages liées.
	 *
	 * @param array $data
	 * @return boolean
	 */
	public function convertToImgpage($documentId, $filename) {
		$success = true;
		$cmd = 'gs -q -dNOPAUSE -dBATCH -sDEVICE=png256 -r150 -dEPSCrop -sOutputFile=' . $filename . '__page__%03d.png ' . $filename;
		passthru(escapeshellcmd($cmd));

		$dir = dirname($filename);
		$basename = basename($filename);

		$Folder = new Folder($dir);
		$Folder->sort = true;
		$files = $Folder->find("{$basename}__page__.*.png");

		foreach ($files as $file) {
			preg_match("#.*__page__([0-9]*)\.png#", $file, $matches);
			$imgpage = $this->Imgpage->create();
			$imgpage['Imgpage']['document_id'] = $documentId;
			$imgpage['Imgpage']['imgpagenum'] = (int) $matches[1];
			$imgpage['fileToStore'] = array('filename' => $Folder->pwd() . DS . $file);
			$success = $this->Imgpage->save($imgpage) && $success;
			if (is_file($Folder->pwd() . DS . $file)) {
				unlink($Folder->pwd() . DS . $file);
			}
		}

		if (is_file($filename)) {
			unlink($filename);
		}

		return $success;
	}

	/**
	 * Récupération des annotations après synchronisation
	 *
	 * @return array
	 */
	public function getAnnotations() {
		$this->_process($this->_getOwn());
		$this->_process($this->_getShared());
		return $this->_getAnnotationsReturn;
	}

	/**
	 * Récupération des annotation personnelles de l'utlisateur connecté
	 *
	 * @return array
	 */
	private function _getOwn() {
		$qd = array(
			'recursive' => -1,
			'contain' => false,
			'joins' => array(
				$this->join($this->Seance->alias),
				$this->Seance->join($this->Seance->Convocation->alias),
				$this->join($this->Imgpage->alias),
				$this->Imgpage->join($this->Imgpage->Annotation->alias),
				$this->Imgpage->Annotation->join($this->Imgpage->Annotation->Owner->alias)
			),
			'fields' => array(
				'Document.id',
				'Imgpage.id',
				'Imgpage.imgpagenum',
				'Owner.id',
				'Owner.firstname',
				'Owner.lastname',
				'Annotation.id',
				'Annotation.user_id',
				'Annotation.shared',
				'Annotation.x',
				'Annotation.y',
				'Annotation.y',
				'Annotation.height',
				'Annotation.width',
				'Annotation.date',
				'Annotation.date_modif',
				'Annotation.comment',
				'Annotation.content'
			),
			'conditions' => array(
				'Annotation.user_id' => CakeSession::read('Auth.User.id')
			),
			'order' => array(
				'Document.id',
				'Imgpage.imgpagenum'
			)
		);
		return $this->find('all', $qd);
	}

	/**
	 * Récupération des annotations partagées de l'utlisateur connecté
	 *
	 * @return array
	 */
	private function _getShared() {
		$qd = array(
			'recursive' => -1,
			'contain' => false,
			'joins' => array(
				$this->join($this->Seance->alias),
				$this->Seance->join($this->Seance->Convocation->alias),
				$this->join($this->Imgpage->alias),
				$this->Imgpage->join($this->Imgpage->Annotation->alias),
				$this->Imgpage->Annotation->join($this->Imgpage->Annotation->AnnotationsUser->alias),
				$this->Imgpage->Annotation->AnnotationsUser->join($this->Imgpage->Annotation->AnnotationsUser->User->alias),
			),
			'fields' => array(
				'Document.id',
				'Imgpage.id',
				'Imgpage.imgpagenum',
				'User.id',
				'User.username',
				'User.firstname',
				'User.lastname',
				'Annotation.id',
				'Annotation.user_id',
				'Annotation.shared',
				'Annotation.x',
				'Annotation.y',
				'Annotation.y',
				'Annotation.height',
				'Annotation.width',
				'Annotation.date',
				'Annotation.date_modif',
				'Annotation.comment',
				'Annotation.content',
				'AnnotationsUser.read'
			),
			'conditions' => array(
				'AnnotationsUser.user_id' => CakeSession::read('Auth.User.id')
			),
			'order' => array(
				'Document.id',
				'Imgpage.imgpagenum'
			)
		);
		return $this->find('all', $qd);
	}

	/**
	 *
	 * @param type $pgTimestamp
	 * @return type
	 */
	private function _postgres2JsTimestamp($pgTimestamp) {
		$tmp = explode(' ', $pgTimestamp);
		$date = explode('-', $tmp[0]);
		$time = explode(':', $tmp[1]);
		return intval(mktime($time[0], $time[1], $time[2], $date[1], $date[2], $date[0])) * 1000;
	}

	/**
	 * Conversion d'une annotation en base vers une annotation pour le client
	 *
	 * @see APP/webroot/js/annotation.json
	 *
	 * @param array $annotation
	 * @param int $imgpagenum
	 * @return array
	 */
	private function _convertAnnotation($annotation, $imgpagenum, $read = true) {
		return array(
			"id" => $annotation['id'],
			"id_utilisateur" => $annotation['user_id'],
			"page" => $imgpagenum,
			"type" => $annotation['shared'],
			"x" => $annotation['x'],
			"y" => $annotation['y'],
			"largeur" => $annotation['width'],
			"hauteur" => $annotation['height'],
			"date" => $this->_postgres2JsTimestamp($annotation['date']),
			"date_modif" => $this->_postgres2JsTimestamp($annotation['date_modif']),
			"lue" => $read,
			"texteInformations" => $annotation['comment'],
			"texteChamps" => $annotation['content']
		);
	}

	/**
	 *
	 * @param type $document_id
	 * @param type $imgpage_id
	 */
	private function _constructAnnotationsReturn($document_id, $imgpage_id) {
		//contruction du tableau retour (documents)
		if (!isset($this->_getAnnotationsReturn[$document_id])) {
			$this->_getAnnotationsReturn[$document_id] = array('id' => $document_id, 'pages' => array());
		}

		//contruction du tableau retour (imgpages)
		if (!isset($this->_getAnnotationsReturn[$document_id]['pages'][$imgpage_id])) {
			$this->_getAnnotationsReturn[$document_id]['pages'][$imgpage_id] = array('id' => $imgpage_id, 'annotations' => array());
		}
	}

	/**
	 * Récupère les utilisateurs concernés par le partage d'une annotation
	 *
	 * @param array $annotation
	 * @return array
	 */
	private function _getUsersToShare($annotation) {
		$qd = array(
			'conditions' => array(
				'Annotation.id' => $annotation['id']
			),
			'contain' => array(
				'User' => array(
					'fields' => array(
						'User.id',
						'("User"."firstname" || \' \' || "User"."lastname") AS "User__fullname"',
						'1 AS "User__sharedstatus"'
					)
				)
			)
		);

		$data = $this->Imgpage->Annotation->find('first', $qd);
		return Hash::remove($data['User'], '{n}.AnnotationsUser');
	}

	/**
	 * Conversion des données relative aux annotations récupérées par $this->_getShared et $this->_getOwn
	 *
	 * @param array $annots
	 */
	private function _process($annots) {
		for ($i = 0; $i < count($annots); $i++) {
			$document_id = $annots[$i]['Document']['id'];
			$imgpage_id = $annots[$i]['Imgpage']['id'];
			$imgpagenum = $annots[$i]['Imgpage']['imgpagenum'];
			$this->_constructAnnotationsReturn($document_id, $imgpage_id);
			$read = isset($annots[$i]['AnnotationsUser']['read']) ? $annots[$i]['AnnotationsUser']['read'] : true;
			$annot = $this->_convertAnnotation($annots[$i]['Annotation'], $imgpagenum, $read);
			$annot['listePartage'] = $this->_getUsersToShare($annots[$i]['Annotation']);
			$this->_getAnnotationsReturn[$document_id]['pages'][$imgpage_id]['annotations'][] = $annot;
		}
	}

}
