<?php

/**
 * Mandat Model
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 * @encoding UTF-8
 *
 * SVN Informations
 * $Date: 2013-10-21 17:57:34 +0200 (lun., 21 oct. 2013) $
 * $Revision: 302 $
 * $Author: ssampaio $
 * $HeadURL: svn+ssh://ssampaio@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/Model/Imgpage.php $
 * $Id: Imgpage.php 302 2013-10-21 15:57:34Z ssampaio $
 *
 * @property Srvuser $Srvuser
 */
App::uses('AppModel', 'Model');

class Mandat extends AppModel {

    /**
     *
     * @var type
     */
    public $useDbConfig = 'default';

    /**
     * Validation rules
     *
     * @var array
     */
    public $validate = array(
        'user_id' => array(
            'uuid' => array(
                'rule' => array('uuid')
            )
        ),
        'collectivite_id' => array(
            'uuid' => array(
                'rule' => array('uuid')
            )
        )
    );

    /**
     * hasAndBelongsToMany associations
     *
     * @var array
     */
    public $hasAndBelongsToMany = array(
        'Srvuser' => array(
            'className' => 'Srvuser',
            'joinTable' => 'srvusers_mandats',
            'foreignKey' => 'mandat_id',
            'associationForeignKey' => 'srvuser_id',
            'unique' => 'keepExisting',
            'conditions' => '',
            'fields' => '',
            'order' => '',
            'limit' => '',
            'offset' => '',
            'finderQuery' => '',
            'with' => 'SrvusersMandat'
        )
    );

    /**
     *
     * @param array $data
     * @return boolean
     */
    public function link($data) {
        if (!is_array($data)) {
            throw new BadMethodCallException();
        }
        $results = array();
        $mandats = $data['Srvuser']['mandat_colls'];

        $this->begin();

        /**
         * creation des mandats manquants
         */
        $srvusers_mandats = array();
        for ($i = 0; $i < count($mandats); $i++) {
            $mandat = $this->create();

            $tmp = explode('=', $mandats[$i]);
            $mandat['Mandat']['collectivite_id'] = $tmp[0];
            $mandat['Mandat']['user_id'] = $tmp[1];

            $checkMandat = $this->find('first', array('conditions' => array('user_id' => $mandat['Mandat']['user_id'], 'collectivite_id' => $mandat['Mandat']['collectivite_id']))); //verification de la presence du mandat
            if (empty($checkMandat)) {//si nouveau mandat, on enregistre
                $saved = $this->save($mandat);
                if (!empty($saved)) {
                    $srvusers_mandats[] = $saved['Mandat']['id'];
                    $results[] = true;
                } else {
                    $results[] = false;
                }
            } else {
                $srvusers_mandats[] = $checkMandat['Mandat']['id'];
            }
        }

        /**
         * suppression des anciens mandats
         */
        //on recupere tous les mandats enregistres en base
        $srvusersMandatsFound = $this->SrvusersMandat->find('all', array('conditions' => array('srvuser_id' => $data['Srvuser']['id'])));
        foreach ($srvusersMandatsFound as $item){
            if (!in_array($item['SrvusersMandat']['mandat_id'], $srvusers_mandats)){
                $results[] = $this->delete($item['SrvusersMandat']['mandat_id']);
            }
        }
        //on supprime ceux non présents dans le request data

        /**
         * liaisons srvusers_mandats
         */
        //suppression des srvusers_mandats
        //on verifie que l utilsateur à un portefeuille de mandats

        if (!empty($srvusersMandatsFound) && $this->SrvusersMandat->deleteAll(array('srvuser_id' => $data['Srvuser']['id'])) || empty($srvusersMandatsFound)) {
            //creation des nouveaux srvusers_mandats
            $srvusers_mandats = array_unique($srvusers_mandats);
            for ($i = 0; $i < count($srvusers_mandats); $i++) {
                $toSave = $this->SrvusersMandat->create();
                $toSave['SrvusersMandat']['srvuser_id'] = $data['Srvuser']['id'];
                $toSave['SrvusersMandat']['mandat_id'] = $srvusers_mandats[$i];

                $srvusers_mandats_saved = $this->SrvusersMandat->save($toSave);

                $results[] = !empty($srvusers_mandats_saved);
            }
        } else {
            $results[] = false;
        }

        $return = false;
        if (!in_array(false, $results, true)) {
            $return = true;
            $this->commit();
        } else {
            $this->rollback();
            
        }

        return $return;
    }

}
