<?php

/**
 * Ptheme Model
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 * @encoding UTF-8
 *
 * SVN Informations
 * $Date: 2014-07-23 13:58:59 +0200 (mer. 23 juil. 2014) $
 * $Revision: 527 $
 * $Author: ssampaio $
 * $HeadURL: svn+ssh://rdubourget@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/Model/Ptheme.php $
 * $Id: Ptheme.php 527 2014-07-23 11:58:59Z ssampaio $
 *
 * @property Ptheme $ParentPtheme
 * @property Ptheme $ChildPtheme
 * @property Convocation $Convocation
 */
App::uses('AppModel', 'Model');

class Ptheme extends AppModel {

    /**
     * Display field
     *
     * @var string
     */
    public $displayField = 'name';

    /**
     *
     * @var type
     */
    public $actsAs = array('Tree');

    /**
     *
     * @var type
     */
    public $validationDomain = 'ptheme';

    /**
     * Validation rules
     *
     * @var array
     */
    public $validate = array(
        'name' => array(
            'notempty' => array(
                'rule' => array('notempty'),
                'message' => 'Ptheme.name.notempty',
            ),
            'isUnique' => array(
                'rule' => array('isUnique'),
                'message' => 'Ptheme.name.isUnique'
            )
        )
    );

    //The Associations below have been created with all possible keys, those that are not needed can be removed

    /**
     * belongsTo associations
     *
     * @var array
     */
    public $belongsTo = array(
        'ParentPtheme' => array(
            'className' => 'Ptheme',
            'foreignKey' => 'parent_id',
            'conditions' => '',
            'fields' => '',
            'order' => ''
        )
    );

    /**
     * hasMany associations
     *
     * @var array
     */
    public $hasMany = array(
        'ChildPtheme' => array(
            'className' => 'Ptheme',
            'foreignKey' => 'parent_id',
            'dependent' => false,
            'conditions' => '',
            'fields' => '',
            'order' => '',
            'limit' => '',
            'offset' => '',
            'exclusive' => '',
            'finderQuery' => '',
            'counterQuery' => ''
        ),
        'Projet' => array(
            'className' => 'Projet',
            'foreignKey' => 'ptheme_id',
            'dependent' => false,
            'conditions' => '',
            'fields' => '',
            'order' => '',
            'limit' => '',
            'offset' => '',
            'exclusive' => '',
            'finderQuery' => '',
            'counterQuery' => ''
        )
    );

    /**
     * Recherche une branche de l'arbre des themes et la crée si elle n'existe pas
     *
     * @param string $strPtheme concatenation d'une branche de l'arbre des themes (separateur : voir IDELIBRE_WSIMPORT_THEME_SEPARATOR dans idelibre.inc.php, valeur par defaut : ",")
     * @return integer (false if error)
     */
    public function treeCreate($strPtheme) {
        $parentId = null;
        $exploded = explode(IDELIBRE_WSIMPORT_THEME_SEPARATOR, $strPtheme);

        foreach ($exploded as $item) {
            $result = $this->find('first', array('conditions' => array('Ptheme.name' => $item)));
            if (empty($result)) {
                $ptheme = $this->create();
                $ptheme['Ptheme']['name'] = $item;
                $ptheme['Ptheme']['parent_id'] = $parentId;
                $this->save($ptheme);
                $parentId = $this->id;
            } else {
                $parentId = $result['Ptheme']['id'];
            }
        }
        return is_null($parentId) ? false : $parentId;
    }

}
