<?php

/**
 * Seance Model
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 * @encoding UTF-8
 *
 * SVN Informations
 * $Date: 2014-07-23 13:58:59 +0200 (mer. 23 juil. 2014) $
 * $Revision: 527 $
 * $Author: ssampaio $
 * $HeadURL: svn+ssh://rdubourget@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/Model/Seance.php $
 * $Id: Seance.php 527 2014-07-23 11:58:59Z ssampaio $
 *
 * @property Document $Document
 * @property Convocation $Convocation
 * @property Projet $Projet
 */
App::uses('AppModel', 'Model');

//App::uses('Security', 'Utility');

class Seance extends AppModel {

    /**
     * Display field
     *
     * @var string
     */
    public $displayField = 'name';

    /**
     * Permet de stocker la séance à créer lors de la génération
     *
     * @var array
     */
    private $_seanceToCreate;

    /**
     * Permet de stocker les données $_POST pour traitement lors de la génération
     *
     * @var array
     */
    private $_post;

    /**
     * Permet de stocker les données $_FILES pour traitement lors de la génération
     *
     * @var array
     */
    private $_docs;

    /**
     * Permet de stocker les utilisateurs asssociés à un type lors de la génération
     *
     * @var array
     */
    private $_usersFromType;

    /**
     * Permet de stocker l'id du groupe "Représentatn" lors de la génération
     *
     * @var string UUID
     */
    private $_representantGroupId;

    /**
     * Liste des utilisateurs non associés à un type lors de la génération
     *
     * @var array
     */
    private $_usersNotAssociated;

    /**
     * Liste des utilisateurs non enregistrés en base lors de la génération
     *
     * @var array
     */
    private $_usersNotInDb;

    /**
     * Liste des utilisateurs enregistrés en base lors de la génération
     *
     * @var array
     */
    private $_usersInDb;

    /**
     * Contient les numéros de projets triés par ordre croissant
     *
     * @var array
     */
    private $_projetsRanks;

    /**
     *
     * @var type
     */
    private $_json;

    /**
     *
     * @var type
     */
    public $virtualFields = array(
        'fullname' => 'Seance.name || \' \' || Seance.date_seance'
    );

    /**
     * Validation rules
     *
     * @var array
     */
    public $validate = array(
        'name' => array(
            'notempty' => array(
                'rule' => array('notempty')
            )
        ),
        'alias' => array(
            'notempty' => array(
                'rule' => array('notempty'),
            //'message' => 'Your custom message here',
            //'allowEmpty' => false,
            //'required' => false,
            //'last' => false, // Stop validation after this rule
            //'on' => 'create', // Limit validation to 'create' or 'update' operations
            ),
            'isUnique' => array(
                'rule' => array('isUnique')
            )
        ),
        'date_seance' => array(
            'datetime' => array(
                'rule' => array('datetime'),
                //'message' => 'Your custom message here',
                'allowEmpty' => false,
            //'required' => false,
            //'last' => false, // Stop validation after this rule
            //'on' => 'create', // Limit validation to 'create' or 'update' operations
            ),
        ),
        'document_id' => array(
            'uuid' => array(
                'rule' => array('uuid'),
                //'message' => 'Your custom message here',
                'allowEmpty' => false,
            //'required' => false,
            //'last' => false, // Stop validation after this rule
            //'on' => 'create', // Limit validation to 'create' or 'update' operations
            ),
        ),
        'type_id' => array(
            'uuid' => array(
                'rule' => array('uuid'),
                //'message' => 'Your custom message here',
                'allowEmpty' => false,
            //'required' => false,
            //'last' => false, // Stop validation after this rule
            //'on' => 'create', // Limit validation to 'create' or 'update' operations
            ),
        ),
    );

    //The Associations below have been created with all possible keys, those that are not needed can be removed

    /**
     * belongsTo associations
     *
     * @var array
     */
    public $belongsTo = array(
        'Type' => array(
            'className' => 'Type',
            'type_id',
            'conditions' => '',
            'fields' => '',
            'order' => ''
        ),
        'Document' => array(
            'className' => 'Document',
            'foreignKey' => 'document_id',
            'conditions' => '',
            'fields' => '',
            'order' => ''
        ),
    );

    /**
     * hasMany associations
     *
     * @var array
     */
    public $hasMany = array(
        'Convocation' => array(
            'className' => 'Convocation',
            'foreignKey' => 'seance_id',
            'dependent' => false,
            'conditions' => '',
            'fields' => '',
            'order' => '',
            'limit' => '',
            'offset' => '',
            'exclusive' => '',
            'finderQuery' => '',
            'counterQuery' => ''
        ),
        'Projet' => array(
            'className' => 'Projet',
            'foreignKey' => 'seance_id',
            'dependent' => false,
            'conditions' => '',
            'fields' => '',
            'order' => '',
            'limit' => '',
            'offset' => '',
            'exclusive' => '',
            'finderQuery' => '',
            'counterQuery' => ''
        )
    );

    /**
     *
     * @param array $options
     */
    public function beforeSave($options = array()) {
        parent::beforeSave($options);
        if (empty($this->data['Seance']['name'])) {
            $this->data['Seance']['alias'] = Inflector::slug($this->Type->field('name', array('Type.id' => $this->data['Seance']['type_id'])) . $this->data['Seance']['date_seance']);
            $this->data['Seance']['name'] = $this->Type->field('name', array('Type.id' => $this->data['Seance']['type_id']));
        }
    }

    /**
     * verification de l'éxistence d'une convocation en fonction de son nom et de sa date
     *
     * @param string $name
     * @param date $date
     * @return mixed (false si la convocation n'existe pas, uuid de la convocation si elle existe)
     */
    public function seanceExists($name, $date) {
        $qdSeance = array(
            'conditions' => array(
                'Seance.name' => $name,
                'Seance.date_seance' => $date,
            )
        );
        $result = $this->find('first', $qdSeance);

        $return = false;
        if (!empty($result)) {
            $return = $result['Seance']['id'];
        }
        return $return;
    }

    /**
     * Traitement des fichiers envoyés
     *
     * @param array $files
     * @return array
     */
    public function processUploadedDocuments($files) {
        $documents = array();
        $convocDocument = array();
        foreach ($files as $key => $file) {
            if ($file['error'] == 0) {
                $uuid = String::uuid();
                $tmp = explode('_', $key);

//				$ext = substr($file['name'], strrpos($file['name'], '.') + 1);
                $filename = CONVOC_UPLOAD_DIR . DS . $uuid . '__' . $file['name'];

                $model = '';
                if ((count($tmp) == 3 && end($tmp) == 'rapport') || (count($tmp) == 1 && $tmp[0] == 'convocation')) { //si le document est un rapport de projet de délibération
                    $model = 'Document';
                } else if (count($tmp) == 4 && end($tmp) == 'annexe') { // si le document est une annexe
                    $model = 'Annex';
                }

                if (move_uploaded_file($file["tmp_name"], $filename)) {
                    $Model = ClassRegistry::init($model);
                    $document = $Model->create();
                    $document[$Model->alias]['id'] = $uuid;
                    $document[$Model->alias]['name'] = $filename;
                    $document[$Model->alias]['type'] = $file['type'];
                    $document[$Model->alias]['size'] = $file['size'];

                    if ($Model->save($document)) {
                        $document['tmp'] = $tmp;
                        if ($key == 'convocation') {
                            $convocDocument = $document;
                        } else {
                            $documents[] = $document;
                        }
                    }
                }
            }
        }
        return array('convocation' => $convocDocument, 'documents' => $documents);
    }

    /**
     * Génération d'une séance et des convocations si la liste des Représentant et des projets (incluant les documents) ne sont pas vides.
     *
     * @param array $post
     * @param array $docs
     * @return array
     */
    public function genSeance($post, $docs) {
        //initialisation du message de retour
        $return = array(
            'success' => false,
            'code' => 'Seance.add.error',
            'message' => __d('seance', 'Seance.add.error'),
            'uuid' => null
        );

        $this->_docs = $docs;
        $this->_post = $post;

        if (empty($this->_post['jsonData'])) {
            $return['code'] = 'Seance.add.error.jsonData.void';
            $return['message'] = __d('seance', 'Seance.add.error.jsonData.void');
        } else {
            //preparation des données
            $this->_post = json_decode($this->_post['jsonData'], true);

            //DevTools::ldebug($this->_post);
            //Traitement des informations de base pour la séance
            try {
                $this->_genBaseSeance();
            } catch (Exception $exc) {
                $return['code'] = 'Seance.add.error.genBaseSeance';
                $return['message'] = __d('seance', 'Seance.add.error.genBaseSeance') . ' ' . $exc;
            }

            //traitement des acteurs (ajout si besoin et liaison avec la seance)
            try {
                $this->_setUsers();
            } catch (Exception $exc) {
                $return['code'] = 'Seance.add.error.setUsers';
                $return['message'] = __d('seance', 'Seance.add.error.setUsers') . ' ' . $exc;
            }

            //creation et association du document principal (convocation)
            try {
                $this->_seanceToCreate['Document']['id'] = $this->_createDoc('Document', $docs['convocation']);
            } catch (Exception $exc) {
                $return['code'] = 'Seance.add.error.createDoc.convocation';
                $return['message'] = __d('seance', 'Seance.add.error.createDoc.convocation') . ' ' . $exc;
            }

            //traitement des projets
            try {
                $this->_setProjets();
            } catch (Exception $exc) {
                $return['code'] = 'Seance.add.error.setProjets';
                $return['message'] = __d('seance', 'Seance.add.error.setProjets') . ' ' . $exc;
            }

            //enregistrment global de la seance
            try {
                $resultSaveSeance = $this->saveAll($this->_seanceToCreate, array('deep' => true));
                //verification des erreurs
                if ($resultSaveSeance) {
                    $return['success'] = true;
                    $return['code'] = 'Seance.add.ok';
                    $return['message'] = __d('seance', 'Seance.add.ok');
                    $return['uuid'] = $this->id;
                };
            } catch (Exception $exc) {
                //Detection de l'erreur de duplication de la date de seance (PostgreSQL)
                if (preg_match('#7:.*#', $this->getDataSource()->lastError())) {
//					$return['message'] = __d('seance', 'Seance.add.error.alreadyExists');
                    $return['code'] = 'Seance.post.noop';
                    $return['message'] = __d('seance', 'Seance.post.noop');
                }
            }
        }
        return $return;
    }

    /**
     * Génération d'une séance et des convocations si la liste des Représentant et des projets (incluant les documents) ne sont pas vides.
     *
     * @param array $post
     * @param array $docs
     * @return array
     */
    public function editSeance($seanceId, $post, $docs) {
        //initialisation du message de retour
        $return = array(
            'success' => false,
            'message' => __d('seance', 'Seance.edit.error'),
            'uuid' => null
        );

        $this->_docs = $docs;
        $this->_post = $post;

        $docsToDrop = array();

        if (empty($seanceId) || !$this->exists($seanceId)) {
            $return['message'] = __d('seance', 'Seance.notFound');
        } else if (empty($this->_post['jsonData'])) {
            $return['message'] = __d('seance', 'Seance.edit.error.jsonData.void');
        } else {
            $this->_post = json_decode($this->_post['jsonData'], true);
            $this->_seanceToCreate = $this->find('first', array('contain' => array('Projet' => array('fields' => array('Projet.id'))), 'conditions' => array('Seance.id' => $seanceId)));
            
            
            
            // on incremente le numero de revision
            $this->_seanceToCreate["Seance"]["rev"] +=1;
            
            // creation et association du document principal (convocation)
            if ($docs['convocation']['error'] == 0) {
                try {
                    $docsToDrop[] = $this->_seanceToCreate['Seance']['document_id'];
                    unset($this->_seanceToCreate['Seance']['document_id']);
                    $this->_seanceToCreate['Document']['id'] = $this->_createDoc('Document', $docs['convocation']);
                } catch (Exception $exc) {
                    $return['message'] = __d('seance', 'Seance.edit.error.createDoc.convocation') . ' ' . $exc;
                }
            }

            try {
               // projet present dans la bdd
                $oldProjets = Hash::extract($this->_seanceToCreate['Projet'], '{n}.id');
         //debug($this->_seanceToCreate);
                $this->_setProjets();                            
         //debug($this->_seanceToCreate); die;
                $newProjets = Hash::extract($this->_seanceToCreate['Projet'], '{n}.id');
                $this->_dropProjets($oldProjets, $newProjets);
            } catch (Exception $exc) {
                $return['message'] = __d('seance', 'Seance.edit.error.setProjets') . ' ' . $exc;
            }

            //enregistrment global de la seance
            try {
                $resultSaveSeance = $this->saveAll($this->_seanceToCreate, array('deep' => true));
                //verification des erreurs
                if ($resultSaveSeance) {
                    $this->_dropDocs($docsToDrop);

                    $return['success'] = true;
                    $return['message'] = __d('seance', 'Seance.edit.ok');
                    $return['uuid'] = $this->id;
                };
            } catch (Exception $exc) {
                //Detection de l'erreur de duplication de la date de seance (PostgreSQL)
                if (preg_match('#7:.*#', $this->getDataSource()->lastError())) {
//					$return['message'] = __d('seance', 'Seance.add.error.alreadyExists');
                    $return['message'] = __d('seance', 'Seance.post.noop');
                }
            }
        }
        return $return;
    }

    /**
     *
     * @param type $docsToDrop
     */
    private function _dropDocs($docsToDrop) {
        for ($i = 0; $i < count($docsToDrop); $i++) {
            $this->Projet->Document->delete($docsToDrop[$i]);
        }
    }

    /**
     *
     * @param type $old
     * @param type $new
     */
    private function _dropProjets($old, $new) {
        $work = array_diff($old, $new);
        
       
        // TODO : le faire avec un hash::extract
        $tab = array();
        foreach ($work as $key => $value) {
            array_push($tab, $value);
        }
        
        
        debug($work);
        for ($i = 0; $i < count($tab); $i++) {
            debug($this->Projet->delete($tab[$i]));
        }
        
        
    }

    /**
     *
     */
    public function WSDeleteSeance($seanceId, $userId) {
        
        App::import('Vendor', 'ElephantIOClient', array('file' => 'ElephantIO/Client.php'));

        if (!class_exists('ElephantIO\Client')) {
            throw new RuntimeException(__d('websocket', 'ElephantIO.load.error'));
        }

//connexion au serveur websocket
        $elephantConfig = array(
            'url' => IDELIBRE_WS_PROTOCOL . '://' . IDELIBRE_WS_HOST . ':' . IDELIBRE_WS_PORT,
            'path' => 'socket.io',
            'protocol' => 1,
            'read' => false,
            'checkSSL' => false,
            'debug' => Configure::read('debug') > 0
        );

        $this->elephant = new ElephantIO\Client($elephantConfig['url'], $elephantConfig['path'], $elephantConfig['protocol'], $elephantConfig['read'], $elephantConfig['checkSSL'], $elephantConfig['debug']);
        $this->elephant->init();

        $toSend = new stdClass();
        $toSend->name = 'seance push delete'; //FIXME: Donnée du traitement
        $toSend->data = array('seanceId' => $seanceId); //FIXME: Donnée du traitement
        $toSend->userId = $userId;
        $toSend->from = 'srv';

        $this->elephant->send(ElephantIO\Client::TYPE_EVENT, null, null, json_encode(array('name' => 'seance push delete', 'args' => json_encode($toSend))));
    }

    /**
     *
     * @param type $id
     * @param type $cascade
     */
    public function delete($id = null, $cascade = true) {
        if ($id === null) {
            $id = $this->id;
        }
        $qd = array(
            'fields' => array(
                'Convocation.id',
                'Convocation.user_id'
            ),
            'conditions' => array(
                'Convocation.seance_id' => $id,
                'Convocation.active' => true
            )
        );
        $userIds = array_values($this->Convocation->find('list', $qd));

        $resultDelete = parent::delete($id, $cascade);

        if ($resultDelete && !empty($userIds)) {
            foreach ($userIds as $userId){
                $this->WSDeleteSeance($id, $userId);
            }
        }

        return $resultDelete;
    }


    /**
     * creation de la séance à enregistrer et traitement des informations de base de la séance
     *
     * @throws CakeException
     */
    private function _genBaseSeance() {
        //verification de la présence de la date
        if (empty($this->_post['date_seance'])) {
            throw new CakeException(__d('seance', 'Seance.genError.genBaseSeance.date_seance'));
        }

        //detection du type de séance
        $typeId = null;
        if (!empty($this->_post['type_id'])) { //cas du formulaire i-delibRE
            $typeId = $this->_post['type_id'];
        } else if (!empty($this->_post['type_seance'])) { //cas du web-service
            $typeId = $this->Type->getTypeIdFromName($this->_post['type_seance'], true); //recherche de l'identifiant du type (creation si le type n'existe pas)
        }

        if (!$this->Type->exists($typeId)) {
            throw new CakeException(__d('seance', 'Seance.genError.genBaseSeance.type_id'));
        }

        $this->_seanceToCreate = $this->create();
        $this->_seanceToCreate['Seance']['type_id'] = $typeId;
        $this->_seanceToCreate['Seance']['date_seance'] = $this->_normalizeDate($this->_post['date_seance']);
    }

    /**
     *
     * @param type $strDate
     * @return type
     */
    private function _normalizeDate($strDate) {
        $strDateTime = explode(' ', $strDate);
        $date = explode('-', $strDateTime[0]);
        $time = explode(':', $strDateTime[1]);
        return date('Y-m-d H:i:s', mktime($time[0], $time[1], $time[2], $date[1], $date[2], $date[0]));
    }

    /**
     * Traitement des utilisateurs associés à la scéance
     */
    private function _setUsers() {
        //récupération des utilisateurs en fonction du type de séance
        $this->_getUsersFromType();
        //récupération de l'id du groupe "Représentant"
        $this->_getRepresentantGroupId();

//        DevTools::ldebug($this->_post);
        //récupération des utilisateurs envoyés par le formulaire ou le web-service
        if (is_string($this->_post['acteurs_convoques'])) {
            $users = json_decode($this->_post['acteurs_convoques'], true);
//$this->log( var_export($users, true ), LOG_ERROR );            
            
        } else if (is_array($this->_post['acteurs_convoques'])) {
            $users = $this->_post['acteurs_convoques'];
        }

        // récupération des utilisateurs non enregistrés en base du formulaire
        $this->_usersInDb = array();
        $this->_usersNotInDb = $this->Convocation->User->getNotInDB($users, $this->_usersInDb);
//$this->log( var_export($this->_usersNotInDb, true ), LOG_ERROR );                    
        // récupération des utilisateurs non associés du formulaire (utilisateurs non assignés et non enregistrés)
        $notAssociated = $this->Type->getNotAssociated($this->_seanceToCreate['Seance']['type_id'], $this->_usersInDb);
//$this->log( __LINE__, LOG_ERROR );
        $freshCreatedUsers = $this->Convocation->User->createUsers($this->_usersNotInDb, $this->_representantGroupId);

        $this->_usersNotAssociated = array_merge($notAssociated, $freshCreatedUsers);

        //on associe les nouveaux utilisateurs et ceux non associés au type
        $this->Type->associateUsers($this->_seanceToCreate['Seance']['type_id'], $this->_usersNotAssociated);

        //association de chaque user à la séance via la table convocations
        $users = $this->Type->getUsersFromTypeId($this->_seanceToCreate['Seance']['type_id']);


        $this->_seanceToCreate['Convocation'] = array();
        foreach ($users as $user) {
            $this->_seanceToCreate['Convocation'][] = array('user_id' => $user['id']);


        }

//		$this->_seanceToCreate['Convocation']['User'] = $this->Type->getUsersFromTypeId($this->_seanceToCreate['Seance']['type_id']);
    }

    /**
     * traitement des projets
     */
    private function _setProjets() {
        $this->_seanceToCreate['Projet'] = array();
        if (!empty($this->_post['projets'])) {
            foreach ($this->_post['projets'] as $projet) {

                //traitement des annexes
                $tabAnnexes = array();
                foreach ($projet['annexes'] as $annexe) {
                    $tabAnnexes[] = array(
                        'id' => $this->_createDoc('Annex', $this->_docs['projet_' . $projet['ordre'] . '_' . $annexe['ordre'] . '_annexe']),
                        'rank' => $annexe['ordre']
                    );
                }

                //creation du projet
                if (isset($this->_docs['projet_' . $projet['ordre'] . '_rapport'])) {
                    $this->_seanceToCreate['Projet'][] = array(
                        'name' => $projet['libelle'],
                        'ptheme_id' => $this->Projet->Ptheme->treeCreate($projet['theme']),
                        'document_id' => $this->_createDoc('Document', $this->_docs['projet_' . $projet['ordre'] . '_rapport']),
                        'rank' => $projet['ordre'],
                        'Annex' => $tabAnnexes
                    );
                } else {
                    $this->_seanceToCreate['Projet'][] = array(
                        'id' => $projet['uuid'],
                        'rank' => $projet['ordre']
                    );
                }
            }
        }
    }

    /**
     * Récupération des utilisateurs du type de séance
     */
    private function _getUsersFromType() {
        $this->_usersFromType = $this->Type->getUsersFromTypeId($this->_seanceToCreate['Seance']['type_id']);
    }

    /**
     * Récupération de l'id du groupe "Représentant"
     */
    private function _getRepresentantGroupId() {
        $this->_representantGroupId = $this->Convocation->User->Group->field('Group.id', array('Group.name' => 'Acteur'));
    }

    /**
     * Créer un document enregistré en base
     *
     * @param string $modelAlias
     * @param array $uploadedFile file item from $_FILES
     * @return mixed return false on error, id (or uuid) else
     */
    private function _createDoc($modelAlias, $uploadedFile) {
        $return = false;
        $document = $this->Projet->{$modelAlias}->create();
        $document[$modelAlias]['name'] = $uploadedFile['name'];
        $document['httpSentFile'] = $uploadedFile;

        if ($this->Projet->{$modelAlias}->save($document)) {
            if ($modelAlias == "Document") {
                $this->Projet->Document->convertToImgpage($this->Projet->{$modelAlias}->id, $uploadedFile['tmp_name']);
            }
            $return = $this->Projet->{$modelAlias}->id;
        }
        return $return;
    }

    /**
     *
     * @return type
     */
    public function getConvocs() {
        $qd = array(
            'contain' => array(
                'Convocation' => array(
                    'fields' => array(
                        'Convocation.read',
                        'Convocation.ar_received',
                        'Convocation.ar_horodatage',
                        'Convocation.active',
                        'Convocation.ae_sent',
                        'Convocation.ae_horodatage',
                        'Convocation.presence',
                        'Convocation.delegation',
                        'Convocation.created'
                    ),
                    'User' => array(
                        'fields' => array('User.firstname', 'User.lastname', 'User.name'),
                        'order' => array('User.firstname', 'User.lastname')
                    )
                )
            ),
            'recursive' => -1,
            'fields' => array(
                'Seance.name',
                'Seance.date_seance'
            ),
            'order' => array('Seance.date_seance desc')
        );
        return $this->find('all', $qd);
    }

}
