<?php

/**
 * User Model
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * PHP version 5
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 * @encoding UTF-8
 *
 * SVN Informations
 * $Date: 2014-12-17 18:59:13 +0100 (mer. 17 déc. 2014) $
 * $Revision: 561 $
 * $Author: rdubourget $
 * $HeadURL: svn+ssh://rdubourget@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/Model/User.php $
 * $Id: User.php 561 2014-12-17 17:59:13Z rdubourget $
 *
 * @property Group $Group
 * @property Convocation $Convocation
 */
App::uses('AppModel', 'Model');
App::uses('AuthComponent', 'Controller/Component');
App::uses('Password', 'Utility');

class User extends AppModel {

    /**
     *
     * @var type
     */
    public $actsAs = array('Acl' => array('type' => 'requester'), 'Password');

    /**
     *
     * @var type
     */
    public $displayField = 'username';

    /**
     *
     * @var type
     */
    public $validationDomain = 'user';

    /**
     *
     * @var type
     */
    public $virtualFields = array(
        'name' => 'User.firstname || \' \' || User.lastname'
    );

    /**
     *
     * @return null
     */
    public function parentNode() {
        if (!$this->id && empty($this->data)) {
            return null;
        }
        if (isset($this->data['User']['group_id'])) {
            $groupId = $this->data['User']['group_id'];
        } else {
            $groupId = $this->field('group_id');
        }
        if (!$groupId) {
            return null;
        } else {
            return array('Group' => array('id' => $groupId));
        }
    }

    /**
     * Validation rules
     *
     * @var array
     */
    public $validate = array(
        'firstname' => array(
            'notempty' => array(
                'rule' => array('notempty'),
                'message' => 'User.firstname.notempty'
            ),
        ),
        'lastname' => array(
            'notempty' => array(
                'rule' => array('notempty'),
                'message' => 'User.lastname.notempty'
            ),
        ),
        'username' => array(
            'notempty' => array(
                'rule' => array('notempty'),
                'message' => 'User.username.notempty'
            ),
            'isUnique' => array(
                'rule' => array('isUnique'),
                'message' => 'User.username.isUnique'
            )
        ),
        'mail' => array(
            'email' => array(
                'rule' => array('email'),
                'message' => 'User.mail.invalid'
            ),
            'notempty' => array(
                'rule' => array('notempty'),
                'message' => 'User.mail.notempty'
            ),
            'isUnique' => array(
                'rule' => array('isUnique'),
                'message' => 'User.mail.isUnique'
            )
        ),
        'password' => array(
            'notempty' => array(
                'rule' => array('notempty'),
                'message' => 'User.password.notempty'
            ),
            'checkIdenticalValues' => array(
                'rule' => array('checkIdenticalValues', 'confirm'),
                'message' => 'User.password.check'
            )
        ),
        'group_id' => array(
            'uuid' => array(
                'rule' => array('uuid'),
                'message' => 'User.group_id.notempty'
            ),
        )
    );

    //The Associations below have been created with all possible keys, those that are not needed can be removed

    /**
     * belongsTo associations
     *
     * @var array
     */
    public $belongsTo = array(
        'Group' => array(
            'className' => 'Group',
            'foreignKey' => 'group_id',
            'conditions' => '',
            'fields' => '',
            'order' => ''
        )
    );

    /**
     * hasMany associations
     *
     * @var array
     */
    public $hasMany = array(
        'Convocation' => array(
            'className' => 'Convocation',
            'foreignKey' => 'user_id',
            'dependent' => false,
            'conditions' => '',
            'fields' => '',
            'order' => '',
            'limit' => '',
            'offset' => '',
            'exclusive' => '',
            'finderQuery' => '',
            'counterQuery' => ''
        ),
        'Client' => array(
            'className' => 'Client',
            'foreignKey' => 'user_id',
            'dependent' => false,
            'conditions' => '',
            'fields' => '',
            'order' => '',
            'limit' => '',
            'offset' => '',
            'exclusive' => '',
            'finderQuery' => '',
            'counterQuery' => ''
        ),
        'Owned' => array(
            'className' => 'Annotation',
            'foreignKey' => 'user_id',
            'dependent' => false,
            'conditions' => '',
            'fields' => '',
            'order' => '',
            'limit' => '',
            'offset' => '',
            'exclusive' => '',
            'finderQuery' => '',
            'counterQuery' => ''
        ),
        'Deltarevision' => array(
            'className' => 'Deltarevision',
            'foreignKey' => 'user_id',
            'dependent' => false,
            'conditions' => '',
            'fields' => '',
            'order' => '',
            'limit' => '',
            'offset' => '',
            'exclusive' => '',
            'finderQuery' => '',
            'counterQuery' => ''
        )
    );

    /**
     * hasAndBelongsToMany associations
     *
     * @var array
     */
    public $hasAndBelongsToMany = array(
        'Type' => array(
            'className' => 'Type',
            'joinTable' => 'types_users',
            'foreignKey' => 'user_id',
            'associationForeignKey' => 'type_id',
            'unique' => 'keepExisting',
            'conditions' => '',
            'fields' => '',
            'order' => '',
            'limit' => '',
            'offset' => '',
            'finderQuery' => '',
            'deleteQuery' => '',
            'insertQuery' => '',
            'with' => 'TypesUser'
        ),
        'Annotation' => array(
            'className' => 'Annotation',
            'joinTable' => 'annotations_users',
            'foreignKey' => 'user_id',
            'associationForeignKey' => 'annotation_id',
            'unique' => 'keepExisting',
            'conditions' => '',
            'fields' => '',
            'order' => '',
            'limit' => '',
            'offset' => '',
            'finderQuery' => '',
            'deleteQuery' => '',
            'insertQuery' => '',
            'with' => 'AnnotationsUser'
        )
    );

    /**
     * Retourne les utilisateurs non présents en base
     *
     * @param array $users
     * @param array $usersInDb permet de récupérer les utilisateurs présents en base
     * @return array User
     */
    public function getNotInDB($users, &$usersInDb = array()) {
        $return = array();
        foreach ($users as $user) {
            $check = $this->find('first', array('conditions' => array('firstname ilike' => $user['Acteur']['prenom'], 'lastname ilike' => $user['Acteur']['nom'])));
            if (empty($check)) {
                $return[] = $user;
            } else {
                $usersInDb[] = $check;
            }
        }
        return $return;
    }

    /**
     * Créer les utilisateurs de la liste $users et retourne le tableau contenant tous les utilisateurs créés
     *
     * @param array $users
     * @param integer $groupId
     * @return array
     */
    public function createUsers($users, $groupId) {
        $result = array();

        foreach ($users as $user) {
            $username = strtolower(substr(Inflector::slug($user['Acteur']['prenom']), 0, 1) . '.' . Inflector::slug($user['Acteur']['nom']));

            $userToSave = $this->create();
            $userToSave['User']['firstname'] = $user['Acteur']['prenom'];
            $userToSave['User']['lastname'] = $user['Acteur']['nom'];
            if (!empty($user['Acteur']['email'])) {
                $userToSave['User']['mail'] = $user['Acteur']['email'];
            }
            $userToSave['User']['username'] = $username;

            $newPassword = Password::generatePassword();
            $userToSave['User']['password'] = AuthComponent::password($newPassword);
            $userToSave['User']['confirm'] = $newPassword;

            $userToSave['User']['group_id'] = $groupId;
            //TODO: a prendre en compte plus tard
            // $user['Acteur']['salutation'];
            // $user['Acteur']['titre']
            // $user['Acteur']['ville']
            // $user['Acteur']['cp']

            try {
                if ($this->save($userToSave)) {
                    $userToSave['User']['id'] = $this->id;
                    $result[] = $userToSave;
                } else {
                    debug($this->validationErrors);
                }
            } catch (Exception $e) {
                $this->log($e->getMessage());
                $this->log(__LINE__, LOG_ERROR);
                throw $e;
            }
        }
        return $result;
    }

    public function getLastSync($userId = null) {
        $return = '';
        if (empty($userId)) {
            if (CakeSession::check('Auth.User.id')) {
                $userId = CakeSession::read('Auth.User.id');
            }
        }
        return $return;
    }

    /**
     *
     */
    public function isSyncNeeded($userId = null) {
//		$syncNeeded = false;

        if (empty($userId)) {
            if (CakeSession::check('Auth.User.id')) {
                $userId = CakeSession::read('Auth.User.id');
            }
        }

        if (!empty($userId)) {
            $result = array(
                'newConvocation' => false,
                'newOwnedAnnotation' => false,
                'newSharedAnnotation' => false
            );


            $qdUsers = array(
                'recursive' => -1,
                'contain' => array(
                    'Owned' => array(
                        'fields' => array(
                            'Owned.id'
                        )
                    ),
                    'Annotation' => array(
                        'fields' => array(
                            'Annotation.id'
                        )
                    ),
                    "Group" => array(
                        'fields' => array(
                            'Group.id'
                        )
                    ),
                    "Deltarevision" => array(
                        'fields' => array(
                            'Deltarevision.id',
                            'Deltarevision.num',
                            'Deltarevision.created'
                        ),
                        'order' => array('Deltarevision.created DESC'),
                        'limit' => 1
                    ),
                    'Convocation' => array(
                        'fields' => array(
                            'Convocation.id',
                            'Convocation.created',
                            'Convocation.active',
                            'Convocation.read'
                        ),
                        'conditions' => array(
                            'Convocation.active' => true
                        ),
                        'order' => array('Convocation.created DESC')
                    )
                ),
                'fields' => array(
                    'User.id',
                    'User.rev',
                    'User.rev_date',
                    'User.seance_sync_date',
                ),
                'conditions' => array(
                    'Group.inseance' => true,
                    'Group.baseuser' => true,
                    'User.id' => $userId
                )
            );
            $user = $this->find('first', $qdUsers);


            $lastClientRev = !empty($user['User']['rev']) ? (int) $user['User']['rev'] : 0; //derniere revision de l'utilisateur
            $lastClientSync = !empty($user['User']['seance_sync_date']) ? strtotime($user['User']['seance_sync_date']) : 0; //derniere synchro de l'utilisateur


            if (!empty($user)) {
                /**
                 * Detection de la synchro initiale
                 */
                if (empty($user['Deltarevision'])) { //pas de synchronisation effectuée
                    if (!empty($user['Convocation'])) { //il y au moins une convocation active
                        if ($lastClientSync != 0) { // si synchro on verifie nouvelles annotations
//                            $result['newOwnedAnnotation'] = $this->_checkAnnotationSync($lastClientRev, Hash::extract($user['Owned'], '{n}.id'));
//                            $result['newSharedAnnotation'] = $this->_checkAnnotationSync($lastClientRev, Hash::extract($user['Annotation'], '{n}.id'));
                        }
                    }
                } else { //deja des synchros effectuées
                    $lastConvoc = !empty($user['Convocation'][0]['created']) ? strtotime($user['Convocation'][0]['created']) : 0; //derniere convocation active
                    $result['newConvocation'] = $lastConvoc > $lastClientSync;
//                    $result['newOwnedAnnotation'] = $this->_checkAnnotationSync($lastClientRev, Hash::extract($user['Owned'], '{n}.id'));
//                    $result['newSharedAnnotation'] = $this->_checkAnnotationSync($lastClientRev, Hash::extract($user['Annotation'], '{n}.id'));
                }
            }
        }

        return $result;
    }

    /**
     *
     * @param int $lastRev
     * @param array $annotationsIds
     * @return type
     */
    private function _checkAnnotationSync($lastRev, $annotationsIds) {
        /**
         * Détection des annotations récentes non-synchronisées
         */
        //préparation de la récupération de la dernière révision concernant les annotations de l'utilisateur
        $qdAnnotRev = array(
            'recursive' => -1,
            'contain' => false,
            'joins' => array(
                $this->Annotation->Annotaction->join($this->Annotation->Annotaction->Deltarevision->alias)
            ),
            'fields' => array(
                'Deltarevision.num'
            ),
            'conditions' => array(
                'Annotaction.annotation_id' => $annotationsIds
            ),
            'order' => array('Deltarevision.created DESC'),
            'limit' => 1
        );
        $annotRev = $this->Annotation->Annotaction->find('first', $qdAnnotRev); //on récupère la dernière revision
        //si la denière revision concernant les annotations et plus récente que la dernière revision concernant l'utilisateur
        return !empty($annotRev) && ($annotRev['Deltarevision']['num'] > $lastRev);
    }

}
