/**
 *
 * webroot/idelibre_client/assets/js/appUI/contextes/ctx_document.js
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * JavaScript (HTML 5)
 *
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 * @encoding UTF-8
 *
 * SVN Informations
 * $Date: 2014-07-29 14:45:51 +0200 (mar. 29 juil. 2014) $
 * $Revision: 537 $
 * $Author: genot $
 * $HeadURL: svn+ssh://rdubourget@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/webroot/idelibre_client/assets/js/appUI/contexts/ctx_document.js $
 * $Id: ctx_document.js 537 2014-07-29 12:45:51Z genot $
 */

/**
 * Document context singleton
 */

var ctxDocument = $.extend(true, {}, new appContext(), {
    name: 'document',
    fullname: 'Consultation d&#039;un document',
    document: {
        content: {
            url: 'documents.js',
            process: function() {
                //  gestion des évenements de resize
                //  mise en mémoire de la largeur actuelle de la page
                var memLargeur = Math.min($(window).width() * 0.8, 1654);

                /**
                 *  @event resize
                 *  @description Redimension de la fenêtre dédiée aux annotations lors d'un évènement "resize"
                 **/
                $(window).on('resize', function() {
                    //  lors d'un redimensionnement de la page, calcul des changements de dimension nécessaires pour le cadre du document
                    var largeur = Math.min($(window).width() * 0.8, 1654);
                    var hauteur = 1.41 * largeur;
                    var x = ($(window).width() - largeur) / 2;
                    //  si la largeur de la page a été modifiée alors une les changements sont appliqués
                    //  (sinon il n'y a pas de modification de la taille du cadre donc pas de changement)
                    if (memLargeur !== largeur) {
                        memLargeur = largeur;
                        Annotation.setConfig({
                            fenetre: {x: x, y: 50, largeur: largeur, hauteur: hauteur},
                            zoom_limite_hauteur: hauteur - $(window).height() * 0.9
                        });
                        //  mise à jour de la scrollbar
                        scrollRedim();
                        Annotation.divIdelibre();
                    }
                });
                //  lancement de la fonction de configuration des boutons
                this.setActionBtns();

                //  initialisation des variables du document
                var doc_uuid = appUI.action.uuid;
                var notification_uuid;
                var annots;

                var finalAnnots = [];

                //  Tableau des images de la séance
                var imgpages = [];

                //  Tableau des utilisateurs partageant la même séance
                var users = [];

                //  Récupération de toutes les séances de l'utilisateur
                var dataSeances = appUI.storage.core.get('seances');

                //  Récupération des images de la séance courante et des utilisateurs associés
                for (var i in dataSeances) {
                    for (var j in dataSeances[i].Projet) {
                        if (dataSeances[i].Projet[j].document_id === appUI.action.uuid) {
                            users = dataSeances[i].Users;
                            imgpages = dataSeances[i].Projet[j].Document.Imgpage;
                        }
                    }
                }

                appUI.nbPagesProjetTotales = imgpages.length;
                //  Mise à jour de la liste de partage de l'utilisateur courant.
                appUI.actShareUsers = users;




                /**
                 *  @function dlImg
                 *  @param {JSObject} element
                 *  @param {Integer} index
                 *  @param {Array} array
                 *  @description Charge les images d'un tableau d'images donné en paramètre depuis la base de données locale
                 *  @return {Void}
                 **/
                function dlImg(element, index, array) {
                    var newPage = new Image();
                    appUI.idb.get(element.id, function(err, doc) {
                        if (err) {
                            console.log(err);
                        } else {
                            finalAnnots.push({
                                id: element.id,
                                imgpagenum: element.imgpagenum,
                                image: 'data:image/png;base64,' + doc.data
                            });
                        }
                    });
                }
                imgpages.forEach(dlImg);
                appUI.finalAnnots = finalAnnots;
            },

            /**
             *  @function setActionBtns
             *  @description Associe à chaque bouton du menu spécifique au contexte document leurs actions.
             *  @return {Void}
             */
            setActionBtns: function() {
                //  Création de la scrollbar du document

                //  variable permettant de savoir si un scroll est en cours
                var memScroll = false;
                //  création du bouton de scroll vers le haut du document
                var docNavScrollUp = $('<span></span>').prop('id', 'docNavScrollUp').addClass('btn').addClass('btn-inverse').html($('<i></i>').addClass('icon-chevron-up').addClass('icon-white')).css({
                    position: 'absolute',
                    'z-index': 50,
                    top: '50px',
                    right: '10px'
                });
                //  création du bouton de scroll vers le bas du document
                var docNavScrollDown = $('<span></span>').prop('id', 'docNavScrollDown').addClass('btn').addClass('btn-inverse').html($('<i></i>').addClass('icon-chevron-down').addClass('icon-white')).css({
                    position: 'absolute',
                    'z-index': 50,
                    top: $(window).height() - 65 + 'px',
                    right: '10px'
                });
                //  création du bouton mobile permettant d'indiquer le niveau du scroll et de le changer
                var docNavScrollCurseur = $('<span></span>').prop('id', 'docNavScrollCurseur').addClass('btn').addClass('btn-inverse').html($('<i></i>').addClass('icon-align-justify').addClass('icon-grey')).css({
                    position: 'absolute',
                    'z-index': 50,
                    top: '80px',
                    right: '10px'
                });

                //  gestion des évènements souris/doigt génériques liés au scrolling
                var noScroll = false;
                //  variable de test du scroll par drag and drop du bouton indicateur
                var curseurScrolling = false; 
                //  variable servant au calcul du scroll en fonction du zoom
                var scrollDim = 0; 
                //  variable permettant de stocker la coordonée Y de la sourie (la x ne sert pas pour le scrolling, celui-ci étant uniquement vertical)
                var mouseY = 0; 
                var lastDistance = 0;
                //  permet de gérer la désactivation du scroll pendant un pinchzoom
                var timerNoScroll = null; 

                /**
                *   Options pour un scroll stable (voir doc. HammerJS)
                **/
                var options = {
                    dragLockToAxis: true,
                    dragBlockHorizontal: true
                };

                var element = window.document.body;

                /**
                 *  @function touch
                 *  @param {JSObject} element
                 *  @param {JSObject} options
                 *  @description Ecouteur des évènements tactiles via Hammer.JS
                 **/
                var touch = new Hammer(element, options);
                touch.on("drag dragright dragleft dragend pinchin pinchout", function(event) {
                    manageMultitouch(event);
                });

                /**
                 *  @function manageMultiTouch
                 *  @param {JSObject} ev
                 *  @description Gestion et répartition des évènements tactiles
                 **/
                function manageMultitouch(ev) {
                    var config = Annotation.getConfig();
                    var svg = $('svg');
                    var svgLeft = ($(window).width() - Math.min($(window).width() * 0.8, 1654)) / 2;

                    switch (ev.type) {
                        case 'drag':
                            //  On sauvegarde la position verticale du document.
                            mouseY = ev.gesture.center.pageY;
                            //  On vérifie que la bibliothèque Annotation est bien chargée et que le nombre de doigts sur la tablette n'excède pas 1.
                            if ((typeof (Annotation) !== 'undefined') && ev.gesture.touches.length!==2) {
                                if (scrollDim === 0)
                                    scrollDim = ($(window).height() - 175) / config.zoom_limite_hauteur;
                                //  Quand on est en train de scroller dans la barre de scroll...
                                if (curseurScrolling === true) {
                                    //  Limitations du curseur (pour qu'il ne passe ni au-dessus, ni en-dessous des boutons fléchés dédiés au scroll.
                                    if (mouseY < 100)
                                        mouseY = 100;
                                    if (mouseY > ($(window).height() - 75))
                                        mouseY = ($(window).height() - 75);
                                    //  Suivi du curseur
                                    var scrollCursor = window.document.getElementById("docNavScrollCurseur");
                                    scrollCursor.style.top = mouseY-20 + 'px';
                                    Annotation.regler_zoom(config.zoom_courant_x, (mouseY - 100) / ($(window).height() - 175) * config.zoom_limite_hauteur * config.zoom_courant_valeur, config.zoom_courant_valeur, false);
                                }
                                //... Sinon mise à jour du curseur si on scroll la page directement.
                                else {
                                    //  Mise à jour de la position du curseur.
                                    actionScrollCurseur();
                                }
                            }

                            break;
                        case 'dragright':
                            //  On vérifie qu'on manipule le document principal et non pas les annotations, on détecte le click sur la balise rect et image (évènement détecté variable selon les navigateurs) ainsi que le nombre de doigts sur la tablette.
                            if ((ev.target === svg.children("rect")[0] || ev.target === svg.children("image")[0]) && ev.gesture.touches.length!==2) {
                                //  On opère le décalage si aucune action de création ou d'édition d'annotations n'est en cours.
                                if (!config.modification_en_cours&& !config.annotations.mode_annotation && ((config.zoom_courant_x + config.fenetre.largeur / config.zoom_courant_valeur) === config.fenetre.largeur + config.zoom_limite_largeur / config.zoom_courant_valeur || config.zoom_courant_x === 0)) {
                                    svg.css('left', svgLeft + ev.gesture.distance + 'px');

                                    //  On met la page précédente si le décalage vers la droite est assez prononcé.
                                    if (ev.gesture.distance > svgLeft * 3) {
                                        prev();
                                    }
                                }
                            }
                            break;
                        case 'dragleft':
                            //  On vérifie qu'on manipule le document principal et non pas les annotations, on détecte le click sur la balise rect et image (évènement détecté variable selon les navigateurs) ainsi que le nombre de doigts sur la tablette.
                            if ((ev.target === svg.children("rect")[0] || ev.target === svg.children("image")[0]) && ev.gesture.touches.length!==2) {
                                //  On opère le décalage si aucune action de création ou d'édition d'annotations n'est en cours.
                                if (!config.modification_en_cours && !config.annotations.mode_annotation && ((config.zoom_courant_x + config.fenetre.largeur / config.zoom_courant_valeur) === config.fenetre.largeur + config.zoom_limite_largeur / config.zoom_courant_valeur || config.zoom_courant_x === 0)) {
                                    svg.css('left', (svgLeft - ev.gesture.distance) + 'px');

                                    //  On met la page suivante si le décalage vers la gauche est assez prononcé.
                                    if (ev.gesture.distance > svgLeft * 3) {
                                        next();
                                    }
                                }
                            }
                            break;
                        case 'dragend':
                            curseurScrolling = false;
                            //  Dès qu'on lâche la page, on applique une animation pour recentrer le document.
                            svg.animate({
                                left: svgLeft + 'px'
                            }, 100);

                            if($("#divAnnotation").children().length === 0) {
                                var eltDetache = svg.detach();
                                $("#divAnnotation").append(eltDetache);
                            }
                            break;
                        case 'pinchin':
                            // Sur un pinchin, on dézoome.
                            if (Math.abs(ev.gesture.distance - lastDistance) > 3){
                                Annotation.zoom_moins();
                                //  Mise à jour du niveau de zoom dans le champs de texte de sélection du zoom
                                docZoomActionsDefValeur.prop('value', Math.round(Annotation.getConfig().zoom_courant_valeur * 10) / 10);
                            }
                            lastDistance = ev.gesture.distance;
                            break;
                        case 'pinchout':
                            //  Sur un pinchout, on zoome.
                            if (Math.abs(ev.gesture.distance - lastDistance) > 3){
                                Annotation.zoom_plus();
                                //  Mise à jour du niveau de zoom dans le champs de texte de sélection du zoom
                                docZoomActionsDefValeur.prop('value', Math.round(Annotation.getConfig().zoom_courant_valeur * 10) / 10);
                            }
                            lastDistance = ev.gesture.distance;
                            break;
                    }
                }

                //  bind des évênements de scroll à l'aide de la molette de la souris
                jQuery(document).bind('mousewheel wheel', function(e) {
                    //  vérification du sens de rotation de la souris et scroll en conséquence
                    if (e.originalEvent.wheelDelta > 0 || e.originalEvent.deltaY < 0) {
                        Scroll(-1);
                    }
                    else {
                        Scroll(1);
                    }
                });

                /**
                 *  @function actionScrollCurseur
                 *  @description Mise à jour la position de l'indicateur de scrolling.
                 *  @return {Void}
                 **/
                var actionScrollCurseur = function() {
                    var config = Annotation.getConfig();
                    var ratioAppli = (config.zoom_courant_y) / ((config.fenetre.hauteur * (config.zoom_courant_valeur - 1) + config.zoom_limite_hauteur) / config.zoom_courant_valeur);
                    docNavScrollCurseur.css('top', 80 + ($(window).height() - 175) * ratioAppli + 'px');
                };

                /**
                 *  @function Scroll
                 *  @param {Integer} sens
                 *  @description Scroll le document vers le haut ou le bas en fonction de l'entier donné en argument (-1 haut, 1 bas).
                 *  @return {Void}
                 **/
                var Scroll = function(sens) {
                    if (!noScroll) {
                        //  calcul de la distance à défiler sur le document
                        var scrollVal = Math.min($(window).width() * 0.8, 1654) / 40;
                        //  récupération des infos du document pour en extraire ses dimensions et son niveau de zoom
                        var config = Annotation.getConfig();
                        if (scrollDim === 0)
                            scrollDim = ($(window).height() - 175) / config.zoom_limite_hauteur;
                        //  scroll du document
                        Annotation.regler_zoom(config.zoom_courant_x, config.zoom_courant_y + (scrollVal * sens), config.zoom_courant_valeur, true);
                        //  mise à jour de l'indicateur de scrolling
                        actionScrollCurseur();
                        //  si un des deux boutons de scrolling est gardé enfoncé alors le scrolling est bouclé
                        setTimeout(function() {
                            if (memScroll) {
                                Scroll(sens);
                            }
                        }, 150);
                    }
                };

                /**
                 *  @function StopScroll
                 *  @description Arrête le scrolling en cours du document.
                 *  @return {Void}
                 **/
                var StopScroll = function() {
                    memScroll = false;
                    /* une double actualisation de l'indicateur est faite pour éviter des problèmes d'erreur de placement de celui-ci lorsque l'utilisateur effectuait un drag and drop dessus */
                    actionScrollCurseur();
                    setTimeout(function() {
                        actionScrollCurseur();
                    }, 150);
                };

                /**
                 *  @function scrollRedim
                 *  @description Redimensionne la scrollbar en fonction de la taille de la fenêtre.
                 *  @return {Void}
                 **/
                scrollRedim = function() {
                    //  récupération des informations du document pour obtenir le niveau de scroll/zoom
                    config = Annotation.getConfig();
                    scrollDim = ($(window).height() - 175) / (config.zoom_limite_hauteur);
                    //  placement des boutons de la scrollbar
                    docNavScrollDown.css({
                        position: 'absolute',
                        'z-index': 50,
                        top: $(window).height() - 65 + 'px',
                        right: '10px'
                    });
                    docNavScrollUp.css({
                        position: 'absolute',
                        'z-index': 50,
                        top: '50px',
                        right: '10px'
                    });
                    //  placement de l'indicateur
                    actionScrollCurseur();
                    //  affichage de la scrollbar
                    $('#scrollbar').css('display', 'block');
                    //  calcul de la taille du document par rapport à la taille de la fenêtre
                    //  si le document est plus petit que la fenêtre alors la scrollbar n'est plus nécessaire, elle est donc masquée
                    if ((Math.min($(window).width() * 0.8, 1654) * 1.41 + 70) < $(window).height() && parseFloat(config.zoom_courant_valeur) === 1) {
                        Annotation.setConfig({zoom_limite_hauteur: 0});
                        $('#scrollbar').css('display', 'none');
                    }
                };
                //  Document pages navigation
                //  Création du champs de texte permettant de choisir la page du document à afficher
                var docPageActionsDefValeur = $('<input />')
                .prop('type', 'number')
                .prop('name', 'zoom')
                .prop('min', 1)
                .prop('step', 1)
                .prop('value', 1)
                .prop('title', 'Page')
                .prop('id', 'indicPage')
                .css('width', '25px');


                //  Création du bouton de validation du champs de texte de sélection de la page
                var docPageActionsDef = $('<span></span>').prop('title', 'Aller à la page')
                .addClass('btn')
                .addClass('btn-inverse')
                .html($('<i></i>').addClass('icon-ok').addClass('icon-white'));

                //  Définition de l'action du bouton de validation de sélection de la page
                docPageActionsDef.on("click", function() {
                    //  récupération du contenu du champs de texte et vérification de son contenu
                    //  (seuls des chiffres sont attendus)
                    var page = parseInt(docPageActionsDefValeur.prop('value'));
                    if (/^[0-9]+$/.test(docPageActionsDefValeur.prop('value')) && page >= 1 && page <= Annotation.getConfig().pages_nombre) {
                        //  affichage de la page désirée
                        Annotation.afficher_page(page);
                        Annotation.divIdelibre();
                        docPageActionsDefValeur.prop('value', page);
                    } else {
                        docPageActionsDefValeur.prop('value', Annotation.getConfig().page_en_cours);
                    }
                    //  réinitialisation du zoom et fermeture du panneau latéral gauche
                    docZoomActionsDefValeur.prop('value', 1);
                    $('#right-panel .menu-close-btn').trigger('click');
                });

                //  Création du bouton page précédente
                var docNavActionsPrev = $('<span></span>')
                .attr('id', 'btnPrev')
                .prop('title', 'Page précédente')
                .addClass('btn')
                .addClass('btn-inverse')
                .html($('<i></i>').addClass('icon-chevron-left').addClass('icon-white'));

                /**
                 *  @function prev
                 *  @description Définition de l'action du bouton page précédente
                 *  @return {Void}
                 **/
                var prev = function() {
                    //  vérification de la page actuelle
                    var config = Annotation.getConfig();
                    if (config.page_en_cours > 1) {
                        //  lancement de l'animation de changement de page
                        $('#divAnnotation').animate({
                            left: window.innerWidth + 'px'
                        }, 300);
                        setTimeout(function() {
                            Annotation.afficher_page(config.page_en_cours - 1);
                            Annotation.divIdelibre();
                            $('#divAnnotation').css('left', -window.innerWidth + 'px').animate({
                                left: '0px'
                            }, 300);
                            //  mise à jour des niveaux de zoom et du numéro de page dans les champs de texte du menu
                            docPageActionsDefValeur.prop('value', Annotation.getConfig().page_en_cours);
                            docZoomActionsDefValeur.prop('value', 1);
                            docNavScrollCurseur.css('top', '80px');
                        }, 301);
                        $('#right-panel .menu-close-btn').trigger('click');
                    }
                };
                docNavActionsPrev.on("click", function() {
                    if (!this.inUse) {
                        var self = this;
                        this.inUse = true;
                        prev();
                        setTimeout(function() {
                            self.inUse = false;
                        }, 500);
                    }
                });

                //  Création du bouton de page suivante
                var docNavActionsNext = $('<span></span>')
                .attr('id', 'btnNext')
                .prop('title', 'Page suivante')
                .addClass('btn')
                .addClass('btn-inverse')
                .html($('<i></i>').addClass('icon-chevron-right').addClass('icon-white'));

                /**
                 *  @function next
                 *  @description Définition de l'action du bouton page suivante
                 *  @return {Void}
                 **/
                var next = function() {
                    //  vérification de la page actuelle
                    var config = Annotation.getConfig();
                    if (config.page_en_cours < config.pages_nombre) {
                        //  lancement de l'animation de changement de page
                        $('#divAnnotation').animate({
                            left: -window.innerWidth + 'px'
                        }, 300);
                        setTimeout(function() {
                            Annotation.afficher_page(config.page_en_cours + 1);
                            Annotation.divIdelibre();
                            $('#divAnnotation').css('left', window.innerWidth + 'px').animate({
                                left: '0px'
                            }, 300);
                            //  mise à jour des niveaux de zoom et du numéro de page dans les champs de texte du menu
                            docPageActionsDefValeur.prop('value', Annotation.getConfig().page_en_cours);
                            docZoomActionsDefValeur.prop('value', 1);
                            docNavScrollCurseur.css('top', '80px');
                        }, 301);
                        $('#right-panel .menu-close-btn').trigger('click');
                    }
                };
                docNavActionsNext.on("click", function() {
                    if (!this.inUse) {
                        var self = this;
                        this.inUse = true;
                        next();
                        setTimeout(function() {
                            self.inUse = false;
                        }, 500);
                    }
                });

                //  ajout des boutons de navigation dans un même groupe de boutons
                var docPageInputAppend = $('<span></span>').css('margin-right', '5px').addClass('visible-desktop').addClass('input-append').append(docPageActionsDefValeur).append(docPageActionsDef).append(docNavActionsPrev).append(docNavActionsNext);

                //  Création du bouton de mode navigation sans ajout d'annotation
                var docNavActionsStopModeAnnotations = $('<span></span>')
                .prop('title', 'Mode navigation')
                .attr('id', 'modeNavigation');

                docNavActionsStopModeAnnotations.css('width', '60px')
                .addClass('btn')
                .addClass('btn-inverse')
                .addClass('active')
                .html($('<i></i>').addClass('icon-hand-up').addClass('icon-white'))
                //  définition de l'action du bouton de mode navigation
                .on("mousedown touchstart", function() {
                    //  fermeture des panneaux latéraux
                    $('#left-panel .menu-close-btn').trigger('click');
                    $('#right-panel .menu-close-btn').trigger('click');
                    //  désactivation de l'ajout d'annotation
                    Annotation.definir_creation_annotation(false);
                    Annotation.choisir_mode_annotation(false);
                    //  réglage de l'état enfoncé/libéré des boutons d'annotations
                    docNavActionsStopModeAnnotations.addClass('active');
                    docNavActionsAddAnnotComm.removeClass('active');
                    docNavActionsAddAnnotPers.removeClass('active');
                });
                //  Création du bouton d'annotation partagée
                var docNavActionsAddAnnotComm = $('<span></span>')
                .prop('title', 'Mode annotation patagée')
                .attr('id', 'modeAnnotPart');

                docNavActionsAddAnnotComm.css('width', '60px')
                .addClass('btn')
                .addClass('btn-success')
                .html($('<i></i>').addClass('icon-edit'))
                .append($('<i></i>').addClass('icon-user'))
                .append($('<i></i>').addClass('icon-user'))
                .append($('<i></i>').addClass('icon-user'))
                //  définition de l'action du bouton d'annotation partagée
                .on("mousedown touchstart", function() {
                    //  fermeture des panneaux latéraux
                    $('#left-panel .menu-close-btn').trigger('click');
                    $('#right-panel .menu-close-btn').trigger('click');
                    //  activation des modes annotation et partage
                    Annotation.definir_creation_annotation(true);
                    Annotation.choisir_mode_annotation(true);
                    //  réglage de l'état enfoncé/libéré des boutons d'annotations
                    docNavActionsStopModeAnnotations.removeClass('active');
                    docNavActionsAddAnnotComm.addClass('active');
                    docNavActionsAddAnnotPers.removeClass('active');
                });
                //  Création du bouton d'annotation personnelle
                var docNavActionsAddAnnotPers = $('<span></span>')
                .prop('title', 'Mode annotation privée')
                .prop('id', 'modeAnnot');

                docNavActionsAddAnnotPers.css('width', '60px')
                .addClass('btn')
                .addClass('btn-primary')
                .html($('<i></i>').addClass('icon-edit')).append($('<i></i>').addClass('icon-user'))
                //  définition de l'action du bouton d'annotation personnelle
                .on("mousedown touchstart", function() {
                    //  fermeture des panneaux latéraux
                    $('#left-panel .menu-close-btn').trigger('click');
                    $('#right-panel .menu-close-btn').trigger('click');
                    //  activation du mode annotation et désactivation du mode partage
                    Annotation.definir_creation_annotation(true);
                    Annotation.choisir_mode_annotation(false);
                    //  réglage de l'état enfoncé/libéré des boutons d'annotations
                    docNavActionsStopModeAnnotations.removeClass('active');
                    docNavActionsAddAnnotComm.removeClass('active');
                    docNavActionsAddAnnotPers.addClass('active');
                });

                //  Création du bouton zoom plus
                var docZoomActionsZoomIn = $('<span></span>')
                .prop('title', 'Augmenter le zoom');

                docZoomActionsZoomIn.addClass('btn')
                .addClass('btn-inverse')
                .html($('<i></i>').addClass('icon-zoom-in').addClass('icon-white'))
                //  définition de l'action du bouton zoom plus
                .on("mousedown touchstart", function() {
                    //  vérification de l'état de zoom, si une animation de zoom est déjà en
                    //  cours alors le bouton n'a aucun effet
                    if (!this.inUse) {
                        var self = this;
                        this.inUse = true;
                        //  lancement du zoom
                        Annotation.zoom_plus();
                        //  mise à jour du niveau de zoom dans le champs de zoom du menu du document
                        docZoomActionsDefValeur.prop('value', Math.round(Annotation.getConfig().zoom_courant_valeur * 10) / 10);
                        scrollRedim();
                        setTimeout(function() {
                            self.inUse = false;
                        }, 500);
                    }
                });

                //  Création du bouton zoom moins
                var docZoomActionsZoomOut = $('<span></span>')
                .prop('title', 'Diminuer le zoom');

                docZoomActionsZoomOut.addClass('btn')
                .addClass('btn-inverse')
                .html($('<i></i>').addClass('icon-zoom-out').addClass('icon-white'))
                //  définition de l'action du bouton zoom plus
                .on("mousedown touchstart", function() {
                    //  vérification de l'état de zoom, si une animation de zoom est déjà en
                    //  cours alors le bouton n'a aucun effet
                    if (!this.inUse) {
                        var self = this;
                        this.inUse = true;
                        Annotation.zoom_moins();
                        //  mise à jour du niveau de zoom dans le champs de zoom du menu du document
                        docZoomActionsDefValeur.prop('value', Math.round(Annotation.getConfig().zoom_courant_valeur * 10) / 10);
                        scrollRedim();
                        setTimeout(function() {
                            self.inUse = false;
                        }, 500);
                    }
                });

                //  Création du champs de texte permettant d'indiquer et modifier le niveau de zoom
                var docZoomActionsDefValeur = $('<input />')
                .prop('type', 'number')
                .prop('name', 'zoom')
                .prop('min', 1)
                .prop('max', 2)
                .prop('step', 0.1)
                .css('width', '35px')
                .val(1);

                //  Création du bouton de validation du zoom souhaité
                var docZoomActionsDef = $('<span></span>')
                .prop('title', 'Choisir le zoom');

                docZoomActionsDef.addClass('btn')
                .addClass('btn-inverse')
                .html($('<i></i>').addClass('icon-search').addClass('icon-white'))
                //  définition de l'action du bouton de validation du zoom
                .on("mousedown touchstart", function() {
                    //  vérification du zoom actuel et du contenu du champs
                    //  (on attend un seulement un float compris entre 1 et 2)
                    var config = Annotation.getConfig();
                    if (/^[0-9.]+$/.test(docZoomActionsDefValeur.prop('value'))) {
                        var zoom = parseFloat(docZoomActionsDefValeur.prop('value'));
                        if (zoom < 1)
                            zoom = 1;
                        if (zoom > 2)
                            zoom = 2;
                        //  modification du niveau de zoom
                        Annotation.regler_zoom(config.zoom_courant_x, config.zoom_courant_y, zoom, true);
                        docZoomActionsDefValeur.prop('value', Math.round(zoom * 10) / 10);
                    }
                    else {
                        docZoomActionsDefValeur.prop('value', Math.round(config.zoom_courant_valeur * 10) / 10);
                    }
                    //  réajustement de la scrollbar
                    scrollRedim();
                });
                //  ajout des boutons de zoom dans un même groupe de bouton
                var docZoomInputAppend = $('<span></span>').css('margin-left', '5px').addClass('visible-desktop').addClass('input-append').append(docZoomActionsDefValeur).append(docZoomActionsDef).append(docZoomActionsZoomIn).append(docZoomActionsZoomOut);

                //  menu responsive
                //  création d'un menu simplifié lorsque la fenêtre de l'application est trop petite pour afficher le menu complet
                var dropUp = $('<div></div>').addClass('hidden-desktop').addClass('dropup').addClass('dropdown');
                var dropUpBtn = $('<a></a>')
                .addClass('btn')
                .addClass('btn-inverse')
                .addClass('dropdown-toggle')
                .css('margin-left', '5px')
                .prop('data-toggle', 'dropdown')
                .append($('<span></span>').addClass('caret'))
                .dropdown();
                dropUp.append(dropUpBtn);
                var dropUpMenu = $('<ul></ul>').addClass('dropdown-menu').prop('role', 'menu');
                dropUpMenu.append($('<li></li>').append($('<a></a>').prop('tabindex', '-1').prop('href', '#').html('Page pr&eacute;c&eacute;dente')).click(function() {
                    docNavActionsPrev.trigger('mousedown');
                }));
                dropUpMenu.append($('<li></li>').append($('<a></a>').prop('tabindex', '-1').prop('href', '#').html('Page suivante')).click(function() {
                    docNavActionsNext.trigger('mousedown');
                }));
                dropUpMenu.append($('<li></li>').addClass('divider'));
                dropUpMenu.append($('<li></li>').append($('<a></a>').prop('tabindex', '-1').prop('href', '#').html('Augmenter le zoom')).click(function() {
                    docZoomActionsZoomIn.trigger('mousedown');
                }));
                dropUpMenu.append($('<li></li>').append($('<a></a>').prop('tabindex', '-1').prop('href', '#').html('Diminuer le zoom')).click(function() {
                    docZoomActionsZoomOut.trigger('mousedown');
                }));
                dropUpMenu.append($('<li></li>').addClass('divider'));

                dropUp.append(dropUpMenu);

                appUI.toolbox
                .empty()
                .append(docPageInputAppend)
                .append($('<span></span>').addClass('btn-group').append(docNavActionsStopModeAnnotations).append(docNavActionsAddAnnotPers).append(docNavActionsAddAnnotComm))
                .append(docZoomInputAppend)
                .append($('<span></span>').addClass('btn-group'))
                .append(dropUp);
                //set scrollbar
                $('body').append($('<div></div>').prop('id', 'scrollbar'));
                $('#scrollbar').append(docNavScrollUp).append(docNavScrollDown).append(docNavScrollCurseur);


                var docNavScrollUpTouch = new Hammer(window.document.getElementById("docNavScrollUp"), options);
                docNavScrollUpTouch.on("touch release", function(event) {
                    switch (event.type) {
                        case 'touch':
                            memScroll = true;
                            Scroll(-1);
                            break;
                        case 'release':
                            StopScroll();
                            break;
                    }
                });

                var docNavScrollDownTouch = new Hammer(window.document.getElementById("docNavScrollDown"), options);
                docNavScrollDownTouch.on("touch release", function(event) {
                    switch (event.type) {
                        case 'touch':
                            memScroll = true;
                            Scroll(1);
                            break;
                        case 'release':
                            StopScroll();
                            break;
                    }
                });

                var docNavScrollCurseurTouch = new Hammer(window.document.getElementById("docNavScrollCurseur"), options);
                docNavScrollCurseurTouch.on("drag", function(event) {
                    switch (event.type) {
                        case 'drag':
                            mouseY = event.gesture.center.pageY;
                            curseurScrolling = true;
                            break;
                    }
                });
            }
        }
    },
    /*  leftPanel
     *
     *  Cet objet permet d'ajouter le contenu du paneau latéral gauche.
     */
    leftPanel: {
        btnName: 'Liste des annotations',
        content: {
            url: 'list_notes.js',
            process: function() {
                //  vidage du précédent contenu
                $('#left-panel .content').empty();
                var noAnnot = true;

                //  vérification du chargement complet du document
                if (typeof (Annotation) !== 'undefined') {
                    //  récupération de la liste des annotations du document courant
                    var listeAnnotations = Annotation.getSauvegarde();
                    //  exploration de chaque page du document
                    if (typeof (listeAnnotations) === 'undefined') {
                        listeAnnotations = {};
                    }
                    for (var i = 0; i < listeAnnotations.length; i++) {
                        if (typeof listeAnnotations[i] !== "undefined") {
                            if (listeAnnotations[i].length > 0) {
                                //  si la page contient au moins une annotation elle apparait dans le panneau
                                noAnnot = false;
                                var titrePage = $('<h4></h4>').html($('<span></span>')).append('Page : ' + (parseInt(i) + 1));
                                $('#left-panel .content').append(titrePage);
                                var listePage = $('<ul></ul>');
                                $('#left-panel .content').append(listePage);
                                //  dans le cas ou la page apparait dans le panneau on ajoute la liste de ses annotations dans le panneau
                                for (var j = 0; j < listeAnnotations[i].length; j++) {
                                    var config = Annotation.getConfig();
                                    //  création du lien vers l'annotation
                                    var texteAnnotation = (listeAnnotations[i][j].texteChamps.length === 0) ? ("(annotation vide)") : ((listeAnnotations[i][j].texteChamps.length <= 30) ? (listeAnnotations[i][j].texteChamps) : (listeAnnotations[i][j].texteChamps.substr(0, 30) + "..."));


                                    var annotationColor = 'blue';
                                    if (listeAnnotations[i][j].type){
                                        annotationColor = listeAnnotations[i][j].id_utilisateur === appUI.accounts.get(appUI.user.account_id).userId ? 'green' : 'orange';
                                    }

                                    var closeElt = $('<a></a>').addClass('close').append('x').data('listeAnnotations', listeAnnotations).data('i', i).data('j', j).click(function() {
                                        var that = $(this);
                                        if (confirm("Supprimer cette annotation?") == true) {
                                            Annotation.supprimer_annotation(that.data('listeAnnotations')[that.data('i')][that.data('j')].id);
                                            that.parent().remove();
                                        }
                                    });
                                    var test = $('<li></li>').data('listeAnnotations', listeAnnotations).data('i', i).data('j', j).addClass(annotationColor).html($('<a href="#"></a>').append(texteAnnotation))
                                    .click(function() {
                                        //  définition de l'action du lien
                                        var that = $(this);
                                        Annotation.trouver_annotation(that.data('listeAnnotations')[that.data('i')][that.data('j')].id, false);
                                        Annotation.divIdelibre();
                                    });
                                    if (annotationColor !== 'orange') {
                                        test.append(closeElt);
                                    }
                                    //  ajout du lien dans la page
                                    listePage.append(test);
                                }
                            }
                        }
                    }
                }

                if (noAnnot) {
                    //  dans le cas ou aucune annotation n'est présente un message générique est affiché
                    $('#left-panel .content').append($('<h4></h4>').html('Aucune annotation.'));
                }

            }
        }
    },
    /*  rightPanel
     *
     *  Cet objet permet d'ajouter le contenu du paneau latéral droit.
     */
    rightPanel: {
        btnName: 'Liste des marque-pages',
        content: {
            url: 'list_bookmarks.js',
            process: function() {
                var self = this;
                //  vidage du précédent contenu
                $('#right-panel .content').empty();
                var noBookmarks = true;

                //  vérification du chargement complet du document
                if (typeof (Annotation) !== 'undefined') {
                    try {
                        var config = Annotation.getConfig();
                    } catch (err) {
                    }
                    if (typeof (config) !== 'undefined') {
                        //  vérification de la présence de marques pages en mémoire
                        if (appUI.storage.core.get("bookmarks" + appUI.action.uuid) === null) {
                            //  si aucun marque page n'est présent on initialise les marques pages de ce document et on les enregistre dans le localStorage
                            var bookmarks = new Array();
                            for (var i = 0; i < config.pages_nombre; i++) {
                                bookmarks[i] = 0;
                            }
                            appUI.storage.core.set("bookmarks" + appUI.action.uuid, JSON.stringify(bookmarks));
                        }
                        else {
                            //  sinon on charge les marques pages précédemment enregistrés
                            var bookmarks = JSON.parse(appUI.storage.core.get("bookmarks" + appUI.action.uuid));
                        }

                        //  Création du champs d'ajout d'un nouveau marque page
                        var defBookmarkValueNote = $('<h4>Marquer la page</h4>');
                        var defBookmarkValue = $('<input />')
                        .prop('type', 'number')
                        .prop('name', 'zoom')
                        .prop('min', 1)
                        .prop('step', 1)
                        .prop('value', config.page_en_cours) // cette donnée value ne doit pas être passée en 'data' car il s'agit du value par défaut de l'input
                        .prop('title', 'Choisir la page à marquer')
                        .css('width', '35px')
                        .attr('id', 'bookmarkValue');

                        //  Création du bouton de validation d'ajout d'un nouveau marque page
                        var defBookmark = $('<span></span>').prop('title', 'Marquer la page')
                        .addClass('btn')
                        .addClass('btn-inverse')
                        .css('margin-top', '-10px')
                        .html($('<i></i>').addClass('icon-ok').addClass('icon-white'));

                        //  définition de l'action du bouton de validation d'ajout d'un nouveau marque page
                        defBookmark.on("click", function() {
                            //  vérification contenu du champs de marque page
                            //  (seuls des chiffres sont attendus)
                            var page = parseInt(defBookmarkValue.prop('value'));
                            if (/^[0-9]+$/.test(defBookmarkValue.prop('value')) && page >= 1 && page <= config.pages_nombre) {
                                //  enregistrement du nouveau marque page
                                bookmarks[page - 1] = 1;
                                appUI.storage.core.set("bookmarks" + appUI.action.uuid, JSON.stringify(bookmarks));
                                //  actualisation du contenu du panneau
                                self.process();
                            }
                        });
                        //  ajout du champs de marque page et de son bouton de validation dans le panneau
                        $('#right-panel .content').append(defBookmarkValueNote);
                        defBookmarkValueNote.append(defBookmarkValue);
                        defBookmarkValueNote.append(defBookmark);

                        //  création des boutons marque page
                        for (var i = 0; i < config.pages_nombre; i++) {
                            if (bookmarks[i] === 1) {
                                noBookmarks = false;
                                var j = i + 1;
                                var titrePage = $('<h4></h4>');
                                var textePage = $('<span></span>').data('page', j).append('<a href="#"> Page : ' + parseInt(j) + '</a>');
                                textePage.on("click", function() {
                                    //  le timeout de 0 est utilisé pour résoudre un bug avec IE
                                    var self = $(this);
                                    setTimeout(function() {
                                        //  lors du click sur le marque page on change la page du document
                                        //  (sans animation)
                                        Annotation.afficher_page(self.data('page'));
                                        var page = Annotation.getConfig().page_en_cours;
                                        $('#indicPage').prop('value', page);
                                        $('#bookmarkValue').prop('value', page);
                                    }
                                               , 0);
                                });
                                //  pour chaque marque page, création du bouton de suppression du marque page
                                var boutonSuppr = $('<span></span>').prop('title', 'Supprimer le marque page')
                                .addClass('btn')
                                .addClass('btn-inverse')
                                .html($('<i></i>').addClass('icon-remove').addClass('icon-white'))
                                .css('float', 'right')
                                .css('margin-top', '-5px');

                                boutonSuppr.data('page', j);
                                //  définition de l'action du bouton de suppression du marque page
                                boutonSuppr.on("click", function() {
                                    //  suppression du marque page en mémoire
                                    bookmarks[$(this).data('page') - 1] = 0;
                                    appUI.storage.core.set("bookmarks" + appUI.action.uuid, JSON.stringify(bookmarks));
                                    //  actualisation du contenu du panneau
                                    self.process();
                                });
                                //  ajout du marque page au panneau
                                titrePage.append(textePage);
                                titrePage.append(boutonSuppr);
                                $('#right-panel .content').append(titrePage);
                            }
                        }
                    }
                }
                if (noBookmarks) {
                    //  si aucun marque page n'est enregistré affichage d'un message générique
                    $('#right-panel .content').append($('<h4></h4>').html('Aucun marque-page.'));
                }
            }
        }
    },

    /**
     *  @function refreshData
     *  @description Rafraîchit le contexte courant
     **/
    refreshData: function(type, account, data) {
    }
});
