/**
 *
 * webroot/idelibre_client/assets/js/appUI/data/data_account.js
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * JavaScript (HTML 5)
 *
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 * @encoding UTF-8
 *
 * SVN Informations
 * $Date: 2014-07-29 14:45:51 +0200 (mar. 29 juil. 2014) $
 * $Revision: 537 $
 * $Author: genot $
 * $HeadURL: svn+ssh://rdubourget@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/webroot/idelibre_client/assets/js/appUI/data/data_account.js $
 * $Id: data_account.js 537 2014-07-29 12:45:51Z genot $
 */


/**
 *  @function data_account
 *  @description Génération d'un compte
 *  @returns {Void}
 **/

var data_account = function() {
    this.id = appUI.genUUID();
};
data_account.prototype = {
    //	id: "",
    label: "",
    login: "",
    password: "",
    //    use_cert: false,
    //    private_key: "",
    serverUrl: "",
    default: false,
    lock: false,
    firstTime: false,
    firstSync: true,
    demo: false,
    userId: "",
    userFirstname: "",
    userLastname: ""
    
};

/**
 *  @function data_accounts_collection
 *  @param {Integer} accountMaxLength
 *  @description Limitation du nombre maximum de mandats par compte
 *  @returns {Void}
 */
var data_accounts_collection = function(accountMaxLength) {
    if (typeof accountMaxLength === 'undefined' || accountMaxLength < 1) {
        accountMaxLength = 10;
    }
    this.accountMaxLength = accountMaxLength;
};
/**
 *
 * @type type
 */
data_accounts_collection.prototype = {
    login: {
        isEmpty: function() {
            return Object.keys(appUI.accounts.login.item).length === 0;
        },
        save: function(login) {
            appUI.accounts.login.item = login;
            this.store();
        },
        load: function() {
            this.restore();
            return appUI.accounts.login.item;
        },
        store: function() {
            localStorage.setItem('appUILogin', JSON.stringify(appUI.accounts.login.item));
        },
        restore: function() {
            appUI.accounts.login.item = JSON.parse(localStorage.getItem('appUILogin'));
        },
        item: {}

    },
    items: [],
    current_item_id: null,
    /**
     *  @function add
     *  @description Ajout d'un nouveau compte
     **/
    add: function() {
        var newId = null;
        if (this.items.length < this.accountMaxLength) {

            //génération du nom du compte
            var newLabel = 'Nouveau mandat';
            var tmp = [0];
            for (var i = 0, ln = this.items.length; i < ln; i++) {
                var reg = /Nouveau mandat ([0-9]+)/g;
                var num = reg.exec(this.items[i].label);
                if (typeof num !== 'undefined' && num !== null && num.length === 2) {
                    tmp.push(parseInt(num[1]));
                }
            }
            newLabel += ' ' + (tmp.reverse()[0] + 1);
            newId = appUI.genUUID();
            var data = {
                id: newId,
                label: newLabel,
                login: '',
                password: '',
                //                use_cert: false,
                //                private_key: '',
                default: this.items.length === 0,
                lock: false,
                firstTime: true,
                firstSync: true,
                demo: false,
                userId: "",
                userFirstname: "",
                userLastname: ""
            };
            var account = $.extend(new data_account(), data);
            this.items.push(account);
        }
        this.store();
        var todo = [];
        appUI.storage.core.set('todo', todo, newId);
        return newId;
    },
    /**
     *  @function addDemo
     *  @description Ajout d'un nouveau compte de démonstration
     **/
    addDemo: function() {
        var demoAccountExists = false;
        for (var i = 0, ln = this.items.length; i < ln; i++) {
            if (this.items[i].demo) {
                demoAccountExists = true;
            }
        }

        var demoId = null;
        if (!demoAccountExists) {
            demoId = appUI.genUUID(),
                demoLabel = 'Elu (démonstration)',
                demoLogin = 'elu@adullact',
                demoPassword = 'idelibre';
            var data = {
                id: demoId,
                label: demoLabel,
                login: demoLogin,
                password: demoPassword,
                //                use_cert: false,
                //                private_key: '',
                default: true,
                lock: false,
                firstTime: true,
                firstSync: true,
                demo: true,
                server_url: 'https://idelibre.demonstrations.adullact.org',
                userId: "",
                userFirstname: "",
                userLastname: ""
            };
            var account = $.extend(new data_account(), data);
            this.items.push(account);
            this.store();
            this.save(demoId, data);
            appUI.storage.core.set('demoFirstTime', "false");
        }
        return demoId;
    },
    /**
     *  @function addDemoWeb
     *  @description Ajout d'un nouveau compte de démonstration pour version Web
     **/
    addDemoWeb: function() {
        var demoAccountExists = false;
        for (var i = 0, ln = this.items.length; i < ln; i++) {
            if (this.items[i].demo) {
                demoAccountExists = true;
            }
        }

        var demoId = null;
        if (!demoAccountExists) {
            demoId = appUI.genUUID(),
                demoLabel = 'Elu (démonstration)',
                demoLogin = 'elu@adullact',
                demoPassword = 'idelibre';
            var data = {
                id: demoId,
                label: demoLabel,
                login: demoLogin,
                password: demoPassword,
                //                use_cert: false,
                //                private_key: '',
                default: true,
                lock: false,
                firstTime: true,
                firstSync: true,
                demo: true,
                server_url: 'https://idelibre.demonstrations.adullact.org',
                userId: "",
                userFirstname: "",
                userLastname: ""
            };
            demoId2 = appUI.genUUID(),
                demoLabel2 = 'Elu2 (démonstration)',
                demoLogin2 = 'elu2@adullact',
                demoPassword2 = 'idelibre';
            var data2 = {
                id: demoId2,
                label: demoLabel2,
                login: demoLogin2,
                password: demoPassword2,
                //                use_cert: false,
                //                private_key: '',
                default: false,
                lock: false,
                firstTime: true,
                firstSync: true,
                demo: true,
                server_url: 'https://idelibre.demonstrations.adullact.org',
                userId: "",
                userFirstname: "",
                userLastname: ""
            };
            var account = $.extend(new data_account(), data);
            this.items.push(account);
            this.store();
            this.save(demoId, data);
            var account2 = $.extend(new data_account(), data2);
            this.items.push(account2);
            this.store();
            this.save(demoId2, data2);
            appUI.storage.core.set('demoFirstTime', "false");
        }
        return demoId;
    },
    /**
     *  @function deleteAll
     *  @description Suppression de tous les comptes actuels
     **/
    deleteAll: function() {

        for (var i = 0, ln = this.items.length; i < ln; i++) {
            appUI.storage.core.delete("seances", this.items[i].id);
            appUI.storage.core.delete("annotations", this.items[i].id);
        }

        this.items = [];
        this.store();
    },
    /**
     *  @function delete
     *  @param {String} id
     *  @description Suppression d'un compte via son id
     *  @returns {Boolean}
     **/
    delete: function(id) {
        var retval = false;
        var toDeleteItem = 0;
        for (var i = 0, ln = this.items.length; i < ln; i++) {
            if (this.items[i].id === id) {
                toDeleteItem = i;
            }
        }
        if (typeof toDeleteItem !== 'undefined') {
            this.items.splice(toDeleteItem, 1);
            retval = true;
        }

        //si le compte par default est supprimé, on defini le premier compte comme étant par défaut
        var noDefaultAccount = true;
        for (var i = 0, ln = this.items.length; i < ln; i++) {
            if (this.items[i].default) {
                noDefaultAccount = false;
            }
        }
        if (noDefaultAccount && typeof this.items[0] !== "undefined") {
            this.items[0].default = true;
        }

        this.store();
        appUI.storage.core.delete("seances", id);
        appUI.storage.core.delete("annotations", id);
        return retval;
    },
        
    /**
     *  @function save
     *  @param {String} id
     *  @param {JSObject} data
     *  @description Sauvegarde des informations d'un compte
     *  @returns {Void}
     **/
    save: function(id, data) {
       // console.log(data);
        var accountKey = -1;
        for (var i = 0, ln = this.items.length; i < ln; i++) {
            if (this.items[i].id === id) {
                accountKey = i;
                this.items[i] = $.extend(this.items[i], data, {firstTime: false, lock: true});
            }
        }
        this.store();
        //  A la sauvegarde du compte, on le connecte au serveur.
       // console.log('connect save');
       // console.log('account key ' + accountKey);
        if (accountKey !== -1) {
            this.connect(accountKey);
        }
    },
        
    /**
     *  @function connect
     *  @param {String} accountKey
     *  @returns {Void}
     **/
    connect: function(accountKey) {
        var url = 'https://' + appUI.accounts.items[accountKey].server_url;
        
        if (appUI.cordova && typeof device !== 'undefined' && (device.platform === "Android" || device.platform === "iOS")) {
            //android et iOS
            appUI.accountsCert.put(appUI.accounts.items[accountKey].id, 'certificat non valide');
            
            window.plugins.sslCertificateChecker.check(function(message) {
                var socket = io.connect(url, {'force new connection': true});
                appUI.engine.initSocket(socket, accountKey);
                
                appUI.cordovaCertSuccessCallback(message);
                
                appUI.accountsCert.put(appUI.accounts.items[accountKey].id, message);
                
                if (appUI.context === "home") {
                    $('#' + appUI.accounts.items[accountKey].id + "Btn .certCheck i").removeClass('icon-unlock').addClass('icon-lock');
                    $('#' + appUI.accounts.items[accountKey].id + "Btn .certCheck .info").html(message);
                }
            }, appUI.cordovaCertErrorCallback, url);
        } else if (!appUI.cordova) {
            //web app
            var socket = io.connect(url, {'force new connection': true});
            console.log('connect');
            appUI.engine.initSocket(socket, accountKey);
        }
    },
        
    /**
     *  @function get
     *  @param {String} id
     *  @description Restitution des informations d un compte parmis la liste des comptes
     *  @returns {JSObject}
     **/
    get: function(id) {
        var account;
        for (var i in this.items) {
            if (this.items[i].id === id) {
                account = this.items[i];
            }
        }
        return account;
    },
        
    /**
     *  @function getDefault
     *  @description Restitution des informations du compte par defaut (actif)
     *  @todo renommer la notion de compte par défaut en compte actif
     *  @returns {JSObject}
     **/
    getDefault: function() {
        var account;
        for (var i in this.items) {
            if (this.items[i].default) {
                account = this.items[i];
            }
        }
        return account;
    },
        
    /**
     *  @function setDefault
     *  @param {String} id
     *  @description Definition du compte par défaut (actif)
     *  @todo renommer la notion de compte par défaut en compte actif
     *  @returns {Void}
     **/
    setDefault: function(id, only) {
        for (var i in this.items) {
            this.items[i].default = (this.items[i].id === id);
        }

        if (typeof only === "undefined" || !only) {
            this.store();
        }
    },
        
    /**
     *  @function switch
     *  @param {String} id
     *  @param {undefined} force
     *  @description Changement des informations du compte courant avec le compte correspondant à id
     *  @returns {Void}
     **/    
    switch : function(id, force) {
        if (appUI.accounts.setCurrentUserInfos(id)) {
            appUI.set('seances');
        }
    },
        
    /**
     *  @function setCurrentUserInfos
     *  @param {String} id
     *  @description Définition du user en cours avec les infos du compte
     *  @returns {Boolean}
     **/
    setCurrentUserInfos: function(id) {
        var retval = false;
        var account = this.get(id);
        if (typeof account !== "undefined") {
            appUI.user.account_id = account.id;
            $('#app-ui-user .default').hide();
            var labelContent = account.label;
            if (account.userFirstname !== "" && account.userLastname !== "") {
                labelContent += " (" +  connName(account.login) + ")";
            }
            $('#app-ui-user .username').html(labelContent);
            appUI.accounts.menu.draw();
            retval = true;
        } else {
            console.log('Le chargement des informations du compte n\'est pas terminé.');
        }
        return retval;
    },
        
    /**
     *  @function getCurrentUserInfos
     *  @description Récupération des informations du user en cours
     *  @returns {JSObject}
     **/
    getCurrentUserInfos: function() {
        return this.get(appUI.user.account_id);
    },
        
    /**
     *  @function store
     *  @param {String} only
     *  @description 
     *  @returns {Boolean}
     **/
    store: function(only) {
        var retval = localStorage.setItem('appUIAccounts', JSON.stringify(this.items));
/*        if (typeof only === "undefined" || !only) {
            appUI.storage.core.store();
        }*/
        return retval;
    },
        
    /**
     *  @function restore
     *  @description Lecture des comptes
     *  @returns {undefined}
     **/
    restore: function() {
        var data = JSON.parse(localStorage.getItem('appUIAccounts'));
        if (data === null) {
            data = [];
        }
        //TODO: retore current account data ?
        this.items = data;
    },
        
    /**
     *  Gestion du menu de changement de compte
     *
     *  @type type
     */
    menu: {
        /**
         *  @function init
         *  @description initialisation du menu
         *  @returns {Void}
         **/
        init: function() {
            $('html').bind('click', function(e) {
                if ($(e.target).attr('id') !== $("#app-ui-user").attr('id')) {
                    $("#app-ui-user").parent().removeClass('open');
                }
            });
            $("#app-ui-user").bind('click', function(e) {
                e.stopPropagation();
                if ($(this).parent().hasClass('open')) {
                    $(this).parent().removeClass('open');
                } else {
                    $(this).parent().addClass('open');
                }
            });
        },
            
        /**
         *  @function draw
         *  @description Affichage du menu
         *  @returns {Void}
         */
        draw: function() {
            $('#account-list').empty();
            var accounts = appUI.accounts.items;
            var ln = accounts.length;
            var noConfiguredAccount = true;
            for (var i = 0; i < ln; i++) {
                if (!accounts[i].firstTime) {
                    noConfiguredAccount = false;
                }
            }
            
            if (ln > 0) {
                if (!noConfiguredAccount) {
                    for (var i = 0; i < ln; i++) {
                        if (!accounts[i].firstTime) {
                            var item = $('<a></a>')
                            .prop('tabindex', '1')
                            .attr('href', '#')
                            .append($('<span></span>').addClass('menu-icon').append($('<i></i>').addClass('icon-user')));
                            var labelContent = accounts[i].label;
                            if (accounts[i].userFirstname !== "" && accounts[i].userLastname !== "") {
                                labelContent += " (" + connName(accounts[i].login) + ")";
                            }
                            item.append(' ' + labelContent);
                            if (accounts[i].id === appUI.user.account_id) {
                                item.css({
                                    color: '#006DCC',
                                    'font-style': 'normal',
                                    'font-weight': 'bold'
                                });
                            }
                            $('#account-list').append($('<li></li>').attr('data-id', accounts[i].id).append(item)   .bind('click', function() {
                                appUI.accounts.switch($(this).attr('data-id'));
                            }));
                        }
                    }
                } else {
                    $('#account-list').append($('<li></li>').append($('<a></a>').prop('tabindex', '1').attr('href', '#').html('Aucun compte utilisateur paramétré')));
                }
            } else {
                $('#account-list').append($('<li></li>').append($('<a></a>').prop('tabindex', '1').attr('href', '#').html('Aucun comtpe utilisateur défini')));
            }
            
        }
    }
}   
