/**
 *
 * webroot/idelibre_client/assets/js/appUI/data/data_imgloaded.js
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * JavaScript (HTML 5)
 *
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 * @encoding UTF-8
 *
 * SVN Informations
 * $Date: 2014-07-29 14:45:51 +0200 (mar. 29 juil. 2014) $
 * $Revision: 537 $
 * $Author: genot $
 * $HeadURL: svn+ssh://rdubourget@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/webroot/idelibre_client/assets/js/appUI/data/data_imgloaded.js $
 * $Id: data_imgloaded.js 537 2014-07-29 12:45:51Z genot $
 */


/**
 *
 * @returns {data_imgloaded}
 */

var data_imgloaded = function() {
};
data_imgloaded.prototype = {
    /**
     * Tableau global
     *
     * @type HashTable
     */
    data: {},
    /**
     *  @function clearStorage
     *  @description Suppression des images en base (PouchDB)
     *  @returns {Void}
     **/
    clearStorage: function() {
        PouchDB.destroy('idelibreDB', function(err, info) {
            if (err) {
                if (appUI.debugMode) {
                    console.log("Base de données non supprimée: " + err);
                }
            } else {
                if (appUI.debugMode) {
                    console.log("Base de données supprimée: " + info);
                }
                appUI.isDeleted = true;
            }
        });

        data = {};
    },
    /**
     *  @function init
     *  @description Initialisation PouchDB
     *  @returns {Void}
     **/
    init: function() {
        //  Initialisation de la base de données locale (PouchDB)
        if (!appUI.cordova) {
            appUI.idb = new PouchDB('idelibreDB');
            
        } else {
            appUI.idb = new PouchDB('idelibreDB', {adapter: 'websql'});
           
            //on insert un premier champs bidon car le plugin phonegap sqlite génère mal la premiere entrée
            appUI.idb.put({
                _id: '00',
               test: 5
            });
        }

        this.restore();
        this.initAllData();
        this.checkAllImagesStatus();

        //TODO: verifier l'interet de la ligne ci-dessous
        var remoteCouch = false;
        appUI.isDeleted = false;
    },
    /**
     *  @function initAccount
     *  @param {JSObject} account
     *  @description Récupère le compte par defaut si account n'est pas précisé
     *  @returns {JSObject}
     **/
    initAccount: function(account) {
        if (typeof account === 'undefined') {
            account = appUI.accounts.getDefault();
        }
        return account;
    },
    /**
     *  @function getStatus
     *  @param {JSObject} account
     *  @description Récupère le chargement global des images de l'ensemble des séances
     *  @returns {JSObject}
     **/
    getStatus: function(account) {
        account = this.initAccount(account);

        var total = 0, loaded = 0;

        for (var i in appUI.imgLoaded.data[account.id]) {
            var data = this.getSeanceStatus(i, account.id);
            total += data.total;
            loaded += data.loaded;
        }

        return {
            total: total,
            loaded: loaded,
            percent: total === 0 ? 0 : parseInt(Math.round(loaded * 100 / total))
        };

    },
    /**
     *  @function getStatus2
     *  @description Récupère le chargement global des images de l'ensemble des projets
     *  @param {JSObject} account
     *  @returns {JSObject}
     **/
    getStatus2: function(account) {
        account = this.initAccount(account);

        var total = 0, loaded = 0, countDownloadedConvocations = 0;

        for (var i in appUI.imgLoaded.data[account.id]) {
            var data = this.getProjetsStatus(i, account.id);
            total += data.total;
            loaded += data.loaded;

            if (this.getConvocationStatus(i, account.id).percent === 100) {
                countDownloadedConvocations++;
            }
        }

        return {
            total: total,
            loaded: loaded,
            percent: total === 0 ? 0 : parseInt(Math.round(loaded * 100 / total)),
            convocations: countDownloadedConvocations
        };

    },
    /**
     *  @function getSeanceStatus
     *  @param {String} seanceId
     *  @param {String} accountId
     *  @description Récupère le chargement des fichiers d'une séance
     *  @returns {JSObject}
     **/
    getSeanceStatus: function(seanceId, accountId) {
        var total = 0, loaded = 0;

        var dataConvocation = this.getConvocationStatus(seanceId, accountId);
        total += dataConvocation.total;
        loaded += dataConvocation.loaded;

        var dataProjets = this.getProjetsStatus(seanceId, accountId);
        total += dataProjets.total;
        loaded += dataProjets.loaded;

        return {
            total: total,
            loaded: loaded,
            percent: total === 0 ? 0 : parseInt(Math.round(loaded * 100 / total))
        };
    },
    /**
     *  @function getSeanceStatus
     *  @param {String} seanceId
     *  @param {String} accountId
     *  @description Récupère le chargement des fichiers d'une convocation
     *  @returns {JSObject}
     **/
    getConvocationStatus: function(seanceId, accountId) {
        var total = 0, loaded = 0;

        var convocation = appUI.imgLoaded.data[accountId][seanceId].convocation;
        for (var i in convocation.doc.imgpages) {
            total++;
            if (convocation.doc.imgpages[i].loaded) {
                loaded++;
            }
        }

        return {
            total: total,
            loaded: loaded,
            percent: total === 0 ? 0 : parseInt(Math.round(loaded * 100 / total))
        };
    },
    /**
     *  @function getSeanceStatus
     *  @param {String} seanceId
     *  @param {String} accountId
     *  @description Récupère le chargement des fichiers des projets
     *  @returns {JSObject}
     **/
    getProjetsStatus: function(seanceId, accountId) {
        var total = 0, loaded = 0;

        var seance = appUI.imgLoaded.data[accountId][seanceId];

        var data = {total: 0, loaded: 0, percent: 0};
        for (var i in seance.projets) {
            data = this.getProjetStatus(seanceId, i, accountId);
            total += data.total;
            loaded += data.loaded;
        }

        return {
            total: total,
            loaded: loaded,
            percent: total === 0 ? 0 : parseInt(Math.round(loaded * 100 / total))
        };
    },
    /**
     *  @function getSeanceStatus
     *  @param {String} seanceId
     *  @param {String} projetId
     *  @param {String} accountId
     *  @description Récupère le chargement des fichiers d'un projet
     *  @returns {JSObject}
     **/
    getProjetStatus: function(seanceId, projetId, accountId) {
        var total = 0, loaded = 0;
        var projets = appUI.imgLoaded.data[accountId][seanceId].projets[projetId];

        for (var i in projets.doc.imgpages) {
            total++;
            if (projets.doc.imgpages[i].loaded) {
                loaded++;
            }
        }

        return {
            total: total,
            loaded: loaded,
            percent: total === 0 ? 0 : parseInt(Math.round(loaded * 100 / total))
        };
    },
    /**
     *  @function initAllData
     *  @description Génère la structure pour chaque mandat
     *  @returns {Void}
     **/
    initAllData: function() {
        if (appUI.debugMode) {
            appUI.notify('checking image loaded status');
        }
        for (var i = 0, ln = appUI.accounts.items.length; i < ln; i++) {
            this.data[appUI.accounts.items[i].id] = this.initData(appUI.accounts.items[i]);
        }

        this.store();
    },
    /**
     *  @function store
     *  @description
     *  @returns {Boolean}
     **/
    store: function() {
        if (appUI.imgLoaded.data === null) {
            appUI.imgLoaded.data = {};
        }
        localStorage.setItem('imgLoaded', JSON.stringify(appUI.imgLoaded.data));

        var retval = false;
        if (localStorage.getItem('imgLoaded') === JSON.stringify(appUI.imgLoaded.data)) {
            retval = true;
        }

        return retval;
    },
    /**
     *  @function restore
     *  @description
     *  @returns {Boolean}
     **/
    restore: function() {
        var data = null;
        var retval = false;

        try {
            data = JSON.parse(localStorage.getItem('imgLoaded'));
            if (data !== null) {
                retval = true;
            }
        } catch (err) {
            console.log(err);
        }

        if (retval) {
            appUI.imgLoaded.data = data;
        } else {
            appUI.imgLoaded.data = {};
        }

        return retval;
    },
    /**
     *  @function initData
     *  @param {JSObject} account
     *  @returns {Boolean}
     **/
    initData: function(account) {
        var retval = {};
        var dataSeances = appUI.storage.core.get('seances', account.id);

        if (typeof dataSeances !== 'undefined' && dataSeances !== null) {

            //detection de toutes les images a charger
            for (var itS = 0, lnS = dataSeances.length; itS < lnS; itS++) {
                var seanceImgLoaded = {
                    seance: {id: dataSeances[itS].Seance.id},
                    convocation: {},
                    projets: {}
                };
                var convocations = dataSeances[itS].Convocation;
                //Convocation
                for (var itC = 0, lnC = convocations.length; itC < lnC; itC++) {
                    if (convocations[itC].user_id === account.userId) { //on choisi la séance dont la convocation est liée avec l utilisateur du compte courant
                        seanceImgLoaded.convocation = {
                            id: dataSeances[itS].Convocation.id,
                            doc: {
                                id: dataSeances[itS].Document.id,
                                imgpages: {}
                            }
                        };
                        //fichier de la convocation
                        var convocationPages = dataSeances[itS].Document.Imgpage;
                        for (var itCP = 0, lnCP = convocationPages.length; itCP < lnCP; itCP++) {
                            seanceImgLoaded.convocation.doc.imgpages[convocationPages[itCP].id] = {
                                id: convocationPages[itCP].id,
                                loaded: false
                            };
                        }

                        //fichiers des rapports
                        var projets = dataSeances[itS].Projet;
                        for (var itP = 0, lnP = projets.length; itP < lnP; itP++) {
                            var projetImgs = {
                                id: projets[itP].id,
                                doc:  {
                                    id: projets[itP].Document.id,
                                    imgpages: {}
                                }
                            };
                            var rapportPages = projets[itP].Document.Imgpage;
                            for (var itRP = 0, lnRP = rapportPages.length; itRP < lnRP; itRP++) {
                                projetImgs.doc.imgpages[rapportPages[itRP].id] = {
                                    id: rapportPages[itRP].id,
                                    loaded: false
                                };
                            }
                            seanceImgLoaded.projets[projets[itP].id] = projetImgs;
                        }
                    }
                }
                retval[dataSeances[itS].Seance.id] = seanceImgLoaded;
            }
        }
        return retval;
    },
    /**
     *  @function checkAllImagesStatus
     *  @description Vérifie si toutes les images sont disponibles dans la base de données locale et rafraîchit les contextes
     *  @returns {Void}
     **/
    checkAllImagesStatus: function() {
        if (appUI.debugMode) {
            appUI.notify('check all images status');
        }
        for (var i = 0, ln = appUI.accounts.items.length; i < ln; i++) {
            this.checkImagesStatus(appUI.accounts.items[i]);
        }
    },
    /**
     *  @function checkImagesStatus
     *  @param {JSObject} account
     *  @description Vérifie si toutes les images d'un compte sont disponibles dans la base de données locale et rafraîchit les contextes associés
     *  @returns {Void}
     **/
    checkImagesStatus: function(account) {
        var dataSeances = appUI.storage.core.get('seances', account.id);
        if (typeof dataSeances !== 'undefined' && dataSeances !== null) {
            //detection de toutes les images a charger
            for (var itS = 0, lnS = dataSeances.length; itS < lnS; itS++) {
                var convocations = dataSeances[itS].Convocation;
                for (var itC = 0, lnC = convocations.length; itC < lnC; itC++) {
                    if (convocations[itC].user_id === account.userId) { //on choisi la séance dont la convocation est liée avec l utilisateur du compte courant
                        //fichier de la convocation
                        var convocationPages = dataSeances[itS].Document.Imgpage;
                        for (var itCP = 0, lnCP = convocationPages.length; itCP < lnCP; itCP++) {

                            //verification asynchrone du chargement des images
                            (function(account, itS, itCP, dataSeances, convocationPages) {
                                appUI.idb.get(convocationPages[itCP].id, function(err, doc) {
                                    if (err) {
                                        if (err.status === 404) {
                                            /*console.log("Image introuvable");*/
                                        } else {
                                            console.log(err);
                                        }
                                    } else {
                                        var accountId = account.id;
                                        var seanceId = dataSeances[itS].Seance.id;
                                        var imgpageId = convocationPages[itCP].id;
                                        var page = appUI.imgLoaded.data[accountId][seanceId].convocation.doc.imgpages[imgpageId];

                                        if (typeof page !== 'undefined') {
                                            page.loaded = true;
                                        }

                                        appUI.contexts[appUI.context].refreshData('loadProgress', account);
                                    }
                                });
                            })(account, itS, itCP, dataSeances, convocationPages)
                        }

                        //fichiers des rapports
                        var projets = dataSeances[itS].Projet;
                        for (var itP = 0, lnP = projets.length; itP < lnP; itP++) {
                            var rapportPages = projets[itP].Document.Imgpage;
                            for (var itRP = 0, lnRP = rapportPages.length; itRP < lnRP; itRP++) {

                                (function(account, itS, itP, itRP, dataSeances, projets, rapportPages) {
                                    appUI.idb.get(rapportPages[itRP].id, function(err, doc) {
                                        if (err) {
                                            if (err.status === 404) {
                                                /*console.log("Image introuvable");*/
                                            } else {
                                                console.log(err);
                                            }
                                        } else {
                                            var accountId = account.id;
                                            var seanceId = dataSeances[itS].Seance.id;
                                            var projetId = projets[itP].id;
                                            var imgpageId = rapportPages[itRP].id;

                                            try {
                                                appUI.imgLoaded.data[accountId][seanceId].projets[projetId].doc.imgpages[imgpageId].loaded = true;
                                            } catch (err) {
                                                if (appUI.debugMode) {
                                                    appUI.notify('check image load error');
                                                }
                                            }

                                            appUI.contexts[appUI.context].refreshData('loadProgress', account);
                                        }
                                    });
                                })(account, itS, itP, itRP, dataSeances, projets, rapportPages)
                            }
                        }
                    }
                }
            }
        }
    }
}
