/**
 *
 * webroot/idelibre_client/assets/js/appUI/data/data_storage.js
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * JavaScript (HTML 5)
 *
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 * @encoding UTF-8
 *
 * SVN Informations
 * $Date: 2013-10-28 11:48:23 +0100 (lun., 28 oct. 2013) $
 * $Revision: 307 $
 * $Author: ssampaio $
 * $HeadURL: svn+ssh://ssampaio@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/webroot/idelibre_client/assets/js/appUI/data/data_state.js $
 * $Id: data_state.js 307 2013-10-28 10:48:23Z ssampaio $
 */

var data_storage = function() {
};
data_storage.prototype = {
    /**
     *
     *  Attention : un seul mode disponible
     *  @todo Refactoriser tout le code!
     *
     **/
    mode: {
        current: 'localStorage',
        localStorage: {
            /**
             *  @function init
             *  @todo
             *  @return {Boolean}
             **/
            init: function() {
                var data = false;
                //CODE
                return data;
            },
            /**
             *  @function get
             *  @param {String} key
             *  @param {String} accountId
             *  @description Récupère les données d'un compte identifié par "accountId" dans le localStorage
             *  @return {JSObject}
             **/
            get: function(key, accountId) {

                var retval = false;
                try {
                    /**
                     *  @see http://www.w3.org/TR/webstorage/#the-localstorage-attribute
                     *
                     **/
                    retval = JSON.parse(localStorage.getItem(accountId + "::" + key));
                } catch (err) {
                    console.log(err);
                }
                return retval;
            },
            /**
             *  @function set
             *  @param {String} key
             *  @data {JSObject} data
             *  @param {String} accountId
             *  @description Stocke les données d'un compte identifié par "accountId" dans le localStorage
             *  @return {Boolean}
             **/
            set: function(key, data, accountId) {
                localStorage.setItem(accountId + "::" + key, JSON.stringify(data));
                return JSON.stringify(data) === localStorage.getItem(accountId + "::" + key);
            },
            /**
             *  @function delete
             *  @param {String} key
             *  @param {String} accountId
             *  @description Supprime les données d'un compte identifié par "accountId" du localStorage
             *  @return {Boolean}
             **/
            delete: function(key, accountId) {
                var retval = false;
                try {
                    retval = localStorage.removeItem(accountId + "::" + key);
                                                 
                } catch (err) {
                console.log(err);
                }
                return retval;
            },
            /**
             *  @function clear
             *  @param {String} key
             *  @param {String} accountId
             *  @description Remise à zéro des données d'un compte identifié par "accountId" dans le localStorage
             *  @return {Boolean}
             **/
            clear: function(key, accountId) {
                data = [];
                localStorage.setItem(accountId + "::" + key, JSON.stringify(data));
                return JSON.stringify(data) === localStorage.getItem(accountId + "::" + key);
            }
        }
    },
    core: {
        /**
         *  @function init
         *  @description Appelle la fonction d'initialisation pour le compte courant du mode de stockage prédéfini
         *  @return {Boolean}
         **/
        init: function() {
            var retval = false;
            if (typeof accountId === 'undefined') {
                if (typeof appUI.user.account_id !== 'undefined') {
                    accountId = appUI.user.account_id;
                } else {
                    accountId = null;
                }
            }

            if (accountId !== null) {
                try {
                    retval = appUI.storage.mode[appUI.storage.mode.current].init();
                } catch (err) {
                    console.log(err);
                }
            }
            return retval;
        },
        /**
         *  @function get
         *  @param {String} key
         *  @param {String} accountId
         *  @description Appelle la fonction de récupération des données pour le compte "accountId" du mode de stockage prédéfini
         *  @return {JSObject}
         **/
        get: function(key, accountId) {
            var retval = false;
            if (typeof accountId === 'undefined') {
                if (typeof appUI.user.account_id !== 'undefined') {
                    accountId = appUI.user.account_id;
                } else {
                    accountId = null;
                }
            }

            if (accountId !== null) {
                try {
                    retval = appUI.storage.mode[appUI.storage.mode.current].get(key, accountId);
                } catch (err) {
                    console.log(err);
                }
            }
            return retval;
        },
        /**
         *  @function set
         *  @param {String} key
         *  @param {JSObject} data
         *  @param {String} accountId
         *  @description Appelle la fonction de stockage des données pour le compte "accountId" du mode de stockage prédéfini
         *  @return {Boolean}
         **/
        set: function(key, data, accountId) {
            var retval = false;
            if (typeof accountId === 'undefined') {
                if (typeof appUI.user.account_id !== 'undefined') {
                    accountId = appUI.user.account_id;
                } else {
                    accountId = null;
                }
            }
            
            if (accountId !== null) {
                try {
                    retval = appUI.storage.mode[appUI.storage.mode.current].set(key, data, accountId);
                } catch (err) {
                    console.log(err);
                }
            }
            return retval;
        },
        /**
         *  @function delete
         *  @param {String} key
         *  @param {String} accountId
         *  @description Appelle la fonction de suppression des données pour le compte "accountId" du mode de stockage prédéfini
         *  @return {Boolean}
         **/
        delete: function(key, accountId) {
            var retval = false;
            if (typeof accountId === 'undefined') {
                if (typeof appUI.user.account_id !== 'undefined') {
                    accountId = appUI.user.account_id;
                } else {
                    accountId = null;
                }
            }
            
            if (accountId !== null) {
                try {
                    retval = appUI.storage.mode[appUI.storage.mode.current].delete(key, accountId);
                } catch (err) {
                    console.log(err);
                }
            }
            return retval;
        },
        /**
         *  @function clear
         *  @param {String} key
         *  @param {String} accountId
         *  @description Appelle la fonction de remise à zéro des données pour le compte "accountId" du mode de stockage prédéfini
         *  @return {Boolean}
         **/            
        clear: function(key, accountId) {
            var retval = false;
            if (typeof accountId === 'undefined') {
                if (typeof appUI.user.account_id !== 'undefined') {
                    accountId = appUI.user.account_id;
                } else {
                    accountId = null;
                }
            }
            
            if (accountId !== null) {
                try {
                    retval = appUI.storage.mode[appUI.storage.mode.current].clear(key, accountId);
                } catch (err) {
                    console.log(err);
                }
            }
            return retval;
        }
    }
}