/**
 *
 * webroot/idelibre_client/assets/js/appUI/engines/engine_debugTools.js
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * JavaScript (HTML 5)
 *
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 * @encoding UTF-8
 *
 * SVN Informations
 * $Date: 2013-10-22 10:49:55 +0200 (mar. 22 oct. 2013) $
 * $Revision: 304 $
 * $Author: ssampaio $
 * $HeadURL: svn+ssh://ssampaio@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/webroot/idelibre_client/assets/js/appUI/engines/engine_sync.js $
 * $Id: engine_sync.js 304 2013-10-22 08:49:55Z ssampaio $
 */

/**
 *  @function engine_debugTools
 *  @description Barre de déboguage déclenchée par le Konami Code
 *  @returns {Void}
 **/
var engine_debugTools = function() {
    //  Création de l'écouteur du Konami Code 
    var easter_egg = new Konami();
    //  Création de la barre de déboguage lorsque le Konami Code est saisi
    easter_egg.code = function() {
        //  Le Mode Debug doit être enclenché par générer la barre de déboguage
        if (!appUI.debugMode) {
            //  Affichage du widget pour la barre de déboguage
            $('<div />').addClass('debugTools').addClass('debugBtn').addClass('btn').addClass('btn-inverse').css({
                position: "fixed",
                top: "0",
                left: "0",
                height: "24px",
                padding: "8px 5px",
                color: 'lightgray'
            })
                    .append($('<i />').addClass('icon').addClass('icon-wrench'))
                    .appendTo('body');
            
            //  Apparition du widget pendant 10 sec
            var debugToolsTimeout = setTimeout(function() {
                appUI.debugTools.unload();
            }, 10000);
            
            //  Suppression du timeout et apparition de la barre de deboguage
            $('.debugTools.debugBtn').click(function() {
                appUI.debugTools.load();
                clearTimeout(debugToolsTimeout);
            })

        } else {
            //  Disparition du widget
            appUI.debugTools.unload();
        }
    }
    easter_egg.load();
};
/**
 *
 * @type type
 */
engine_debugTools.prototype = {
    /**
     * Debug
     *
     * Permet d'afficher les erreurs dans la zone de notitifcation( + affichage dans la console js)
     *
     * le mode debug doit être activé
     * ex :
     *	var appUI = new UI();
     *	appUI.debugMode = true;
     */
    
    /**
     *  @function getLogDateTime
     *  @description Affiche la date actuelle
     *  @return {Void}
     **/
    getLogDateTime: function() {
        return $('<div></div>').css({
            color: "white",
            backgroundColor: "#999",
            borderRadius: "3px",
            padding: "1px 3px",
            textShadow: 'none'
        }).html(':' + new Date().getTime() + '<br />').prepend($('<i></i>').addClass('icon').addClass('icon-time').addClass('icon-white'));
    },
    
    /**
     *  @function debug
     *  @param {JSObject} variable
     *  @returns {Void}
     **/
    debug: function(variable) {
        if (appUI.debugMode) {
            console.log(variable);
            var nd = new Date();
            this.notify(nd.toString() + ' [' + this.context + '] ' + lastClick + ': ' + variable.toString(), 1);
        }
    },
    /**
     *  @function unload
     *  @description Désactivation du mode Debug et de la barre de déboguage (suppression)
     *  @returns {Void}
     **/
    unload: function() {
        appUI.debugMode = false;
        $('.debugTools').remove();
    },
    /**
     *  @function load
     *  @description charge dans le DOM la barre d'outils de débug.
     *  @return {Void}
     **/
    load: function() {
        //après vérification du mode débug, la barre est chargée si nécessaire
        appUI.debugMode = true;
        var that = this;
        //création du div gauche
        var debugToolsDivLeft = $('<div></div>').addClass('debugTools').addClass('left');
        //création du bouton de suppression du localStorage
        var clearLocalStorageDiv = $('<div></div>').attr('id', 'clearLocalStorageDiv').addClass('btn').addClass('btn-danger').attr('title', 'clear localStorage').html($('<i></i>').addClass('icon-trash')).click(function() {
            try {
                localStorage.clear();
                appUI.notify('localStorage cleared');
            } catch (err) {
                appUI.error('Impossible de vider localStorage', err);
            }
        });
        //création du bouton de suppression du sessionStorage
        var clearSessionStorageDiv = $('<div></div>').attr('id', 'clearSessionStorageDiv').addClass('btn').addClass('btn-danger').attr('title', 'clear sessionStorage').html($('<i></i>').addClass('icon-trash')).click(function() {
            try {
                sessionStorage.clear();
                appUI.notify('sessionStorage cleared');
            } catch (err) {
                appUI.error('Impossible de vider sessionStorage', err);
            }
        });
        //création du bouton d'affichage du localStorage
        var showLocalStorageDiv = $('<div></div>').attr('id', 'showLocalStorageDiv').addClass('btn').addClass('btn-inverse').attr('title', 'show localStorage').html($('<i></i>').addClass('icon-tablet').addClass('icon-white')).click(function(e) {
            e.stopPropagation();
            $('.popover').prev().popover('hide');
            element.empty();
            element.append(JSON.stringify(localStorage, null, '\t'));
            showLocalStorageDiv.popover({
                html: true,
                title: "Popover Title",
                content: element
            });
        });
        //création du bouton d'affichage du sessionStorage
        var showSessionStorageDiv = $('<div></div>').attr('id', 'showSessionStorageDiv').addClass('btn').addClass('btn-inverse').attr('title', 'show sessionStorage').html($('<i></i>').addClass('icon-hdd').addClass('icon-white')).click(function(e) {
            e.stopPropagation();
            $('.popover').prev().popover('hide');
            element.empty();
            element.append(JSON.stringify(sessionStorage, null, '\t'));
            showSessionStorageDiv.popover({
                html: true,
                title: "Popover Title",
                content: element
            });
        });
        //création du bouton d'affichage des données enregistrées
        var showPouchDB = $('<div></div>').attr('id', 'showPouchDB').addClass('btn').addClass('btn').addClass('btn-inverse').attr('title', 'show PouchDB').html($('<i></i>').addClass('icon-archive').addClass('icon-white')).click(function(e) {
            /*btnActionContent.empty().show();*/
            e.stopPropagation();
            $('.popover').prev().popover('hide');
            element.empty();
            appUI.idb.allDocs(function(err, response) {
                if (err) {
                    element.append('<hr>');
                    element.append(err);
                } else {
                    element.append('<hr>');
                    element.append(JSON.stringify(response, null, '\t'));
                }
            });

            appUI.idb.info(function(err, info) {
                if (err) {
                    element.append('<hr>');
                    element.append(err);
                } else {
                    element.append('<hr>');
                    element.append(JSON.stringify(info, null, '\t'));
                }
            });

            showPouchDB.popover({
                html: true,
                title: "Popover Title",
                content: element
            });
        });
        //création du bouton d'affichage des données enregistrées
        var clearPouchDB = $('<div></div>').attr('id', 'clearPouchDB').addClass('btn').addClass('btn-danger').attr('title', 'clear PouchDB').html($('<i></i>').addClass('icon-trash').addClass('icon-white')).click(function() {
            appUI.imgLoaded.clearStorage();
            appUI.imgLoaded.checkAllImagesStatus();
        });

        var showAnnotations = $('<div></div>').attr('id', 'showAnnotations').addClass('btn').addClass('btn').addClass('btn-inverse').attr('title', 'show Annotations').html($('<i></i>').addClass('icon-comment').addClass('icon-white')).click(function(e) {
            /*btnActionContent.empty().show();*/
            e.stopPropagation();
            $('.popover').prev().popover('hide');
            element.empty();
            element.append(JSON.stringify(appUI.storage.core.get('annotations'), null, '\t'));
            showAnnotations.popover({
                html: true,
                title: "Popover Title",
                content: element
            });
        });
        //création du bouton de suprression des annotations enregistrées de l'utilisateur courant
        var clearAnnotations = $('<div></div>').attr('id', 'clearAnnotations').addClass('btn').addClass('btn-danger').attr('title', 'clear stored annotations').html($('<i></i>').addClass('icon-trash').addClass('icon-white')).click(function() {
            appUI.storage.core.clear('annotations');
        });
        var showSeances = $('<div></div>').attr('id', 'showSeances').addClass('btn').addClass('btn').addClass('btn-inverse').attr('title', 'show Seances').html($('<i></i>').addClass('icon-file').addClass('icon-white')).click(function(e) {
            /*btnActionContent.empty().show();*/
            e.stopPropagation();
            $('.popover').prev().popover('hide');
            element.empty();
            element.append(JSON.stringify(appUI.storage.core.get('seances'), null, '\t'));
            showSeances.popover({
                html: true,
                title: "Popover Title",
                content: element
            });
        });
        //création du bouton de suprression des seances enregistrées de l'utilisateur courant
        var clearSeances = $('<div></div>').attr('id', 'clearSeances').addClass('btn').addClass('btn-danger').attr('title', 'clear stored seances').html($('<i></i>').addClass('icon-trash').addClass('icon-white')).click(function() {
            appUI.storage.core.clear('seances');
        });
        var purgeSockets = $('<div></div>').attr('id', 'purgeSockets').addClass('btn').addClass('btn-primary').attr('title', 'purge sockets').html($('<i></i>').addClass('icon-trash').addClass('icon-white')).click(function() {
            appUI.engine.purgeSockets();
        });
        var clearAll = $('<div></div>').attr('id', 'clearAll').addClass('btn').addClass('btn-warning').attr('title', 'clear All').html($('<i></i>').addClass('icon-trash').addClass('icon-white')).click(function() {
            $(clearLocalStorageDiv).trigger('click');
            $(clearSessionStorageDiv).trigger('click');
            $(clearPouchDB).trigger('click');
            $(purgeSockets).trigger('click');
        });
        //création du bouton de rechargement
        var reloadDiv = $('<div></div>').attr('id', 'reloadDiv').addClass('btn').addClass('btn-inverse').attr('title', 'reload page').html($('<i></i>').addClass('icon-refresh').addClass('icon-white')).click(function() {
            location.reload();
        });

        //création du bouton de rechargement et suppression des données
        var fullReloadDiv = $('<div></div>').attr('id', 'fullReloadDiv').addClass('btn').addClass('btn-danger').attr('title', 'purge storages and reload page').html($('<i></i>').addClass('icon-refresh')).click(function() {
            location.reload(true);
        });


        //création du bouton d'affichage des FPS
        var showFPS = $('<div></div>').attr('id', 'showFPS').addClass('btn').addClass('btn-inverse').attr('title', 'show FPS').html($('<i></i>').addClass('icon-dashboard').addClass('icon-white')).click(function() {
            var script = document.createElement('script');
            script.src = 'assets/annotations/stats.min.js';
            document.body.appendChild(script);
            script = document.createElement('script');
            script.innerHTML = 'var interval=setInterval(function(){if(typeof Stats==\'function\'){clearInterval(interval);var stats=new Stats();stats.domElement.style.position=\'fixed\';stats.domElement.style.left=\'0px\';stats.domElement.style.bottom=\'40px\';stats.domElement.style.zIndex=\'10000\';document.body.appendChild(stats.domElement);setInterval(function(){stats.update();},1000/60);}},100);';
            document.body.appendChild(script);

        });
        //création du bouton de masquage des barres de menu
        var hideFixedBars = $('<div></div>').attr('id', 'hideFixedBars').addClass('btn').addClass('btn-inverse').attr('title', 'show / hide fixedbars').html($('<i></i>').addClass('icon').addClass('icon-white').addClass('icon-fullscreen')).click(function() {
            $('#footer, .app-ui-header').slideToggle('fast');
        });
        //création du bouton d'affichage de la console javascript
        var showConsole = $('<a></a>').attr('id', 'showConsole').prop('href', "javascript:(function(F,i,r,e,b,u,g,L,I,T,E){if(F.getElementById(b))return;E=F[i+'NS']&&F.documentElement.namespaceURI;E=E?F[i+'NS'](E,'script'):F[i]('script');E[r]('id',b);E[r]('src',I+g+T);E[r](b,u);(F[e]('head')[0]||F[e]('body')[0]).appendChild(E);E=new%20Image;E[r]('src',I+L);})(document,'createElement','setAttribute','getElementsByTagName','FirebugLite','4','firebug-lite.js','releases/lite/latest/skin/xp/sprite.png','https://getfirebug.com/','#startOpened');").addClass('btn').addClass('btn-inverse').attr('title', 'show Console').html($('<i></i>').addClass('icon-terminal').addClass('icon-white'));


        //création du groupe de boutons contenu dans la barre de débug
        var debugToolsDivLeftContent = $('<div></div>').addClass('content').addClass('btn-group').addClass('btn-group-vertical').append($('<br />'));
        //ajout de tout les boutons dans le groupe de boutons
        //et ajout du groupe de bouton dans la barre de débug
        debugToolsDivLeftContent
                .append(showLocalStorageDiv)
                .append(clearLocalStorageDiv)
                .append(showSessionStorageDiv)
                .append(clearSessionStorageDiv)
                .append(showPouchDB)
                .append(clearPouchDB)
                .append(showSeances)
                .append(clearSeances)
                .append(showAnnotations)
                .append(clearAnnotations)
                .append(purgeSockets)
                .append(clearAll)
                .append(showFPS)
                .append(showConsole)
                .append(hideFixedBars)
                .append(reloadDiv)
                .append(fullReloadDiv)
                .append($('<br />'));
        debugToolsDivLeft.append(debugToolsDivLeftContent)
                .appendTo($('body'));

        var element = $('<pre></pre>').attr('id', 'popoverActionContent').css({
            width: '800px',
            height: '300px',
            overflow: 'auto'
        });
    }
};

