/**
 *
 * webroot/idelibre_client/assets/js/appUI/engines/engine_ws.js
 *
 * i-delibRE : le porte-document nomade des élus pour le suivi des séances délibérantes (https://adullact.net/projects/idelibre)
 *
 * JavaScript (HTML 5)
 *
 * @author Stéphane Sampaio
 * @copyright Adullact Projet
 * @link http://adullact.org/
 * @license http://www.cecill.info/licences/Licence_CeCILL_V2-fr.txt CeCiLL v2
 * @encoding UTF-8
 *
 * SVN Informations
 * $Date: 2014-03-18 16:30:20 +0100 (mar. 18 mars 2014) $
 * $Revision: 363 $
 * $Author: ssampaio $
 * $HeadURL: svn+ssh://ssampaio@scm.adullact.net/scmrepos/svn/idelibre/trunk/app/webroot/idelibre_client/assets/js/appUI/engines/engine_sync.js $
 * $Id: engine_sync.js 363 2014-03-18 15:30:20Z ssampaio $
 */

/**
 * appElement object
 */



// compare 2 tableaux
function arraysIdentical(a, b) {
    var i = a.length;
    if (i != b.length)
        return false;
    while (i--) {
        if (a[i] !== b[i])
            return false;
    }
    return true;
}
;



/**
 * donne la position des élément different entre les 2 tableaux de meme longeur
 * retourne vrai si ils sont de longeur differentes pour ne pas avoir le probleme de la nouvelle convocation
 * @param {type} tab1
 * @param {type} tab2
 * @returns {positionDif.tabReturn|Array|Boolean}
 */

var positionDif = function(tab1, tab2) {
    var len = tab1.length;
    if (len !== tab2.length) {
        return true;
    }
    var tabReturn = [];
    for (var i = 0; i < len; i++) {
        if (tab1[i] != tab2[i]) {
            tabReturn.push(i);
        }
    }
    return tabReturn;

};




var engine_ws = function() {
};

engine_ws.prototype = {
    /**
     * [
     *      {
     *          account: id,
     *          type: type de l envoi
     *          data: données a envoyer
     *      },
     *      {
     *          account: id,
     *          type: type de l envoi
     *          data: données a envoyer
     *      },
     *  ]
     *
     *
     * @type Array
     */

    /**
     *  @function getSocket
     *  @param {String} accountId
     *  @description Sert à récupérer la socket associée au compte entré en paramètre.
     *  @return {JSObject}
     **/
    getSocket: function(accountId) {
        var socket = null;
        for (var iS = 0, lnS = appUI.engine.sockets.length; iS < lnS; iS++) {
            if (appUI.engine.sockets[iS].accountId === accountId) {
                socket = appUI.engine.sockets[iS].socket;
            }
        }
        return socket;
    },
    /**
     *  @function getCurrentSocket
     *  @description Sert à récupérer la socket associée au compte courant.
     *  @return {JSObject}
     **/
    getCurrentSocket: function() {
        var socket = null;
        for (var iS = 0, lnS = appUI.engine.sockets.length; iS < lnS; iS++) {
            if (appUI.engine.sockets[iS].accountId === appUI.user.account_id) {
                socket = appUI.engine.sockets[iS].socket;
            }
        }
        return socket;
    },
    /**
     *  @function purgeSockets
     *  @description Déconnexion des sockets en cours et remise à zéro de la collection de sockets
     *  @returns {Void}
     **/
    purgeSockets: function() {
        //stop all sockets
        for (var i in appUI.engine.sockets) {
            appUI.engine.sockets[i].socket.disconnect();
        }
        appUI.engine.sockets = [];
    },
    //  Compteur d'images chargées.
    imgpagesloaded: 0,
    //  Tableau de sockets pour stocker chacune des sockets associée à chacun des comptes.
    sockets: [],
    //  Gestion des icônes de débit ascendant ou descendant.
    ico: {
        timeout: {
            emit: null,
            on: null
        },
        //  Signalisation débit ascendant
        emit: function() {
            clearTimeout(this.timeout.emit);
            $('#socketEmit').addClass('active');
            this.timeout.emit = setTimeout(function() {
                $('#socketEmit').removeClass('active');
            }, 500);
        },
        //  Signalisation débit descendant
        on: function() {
            clearTimeout(this.timeout.on);
            $('#socketOn').addClass('active');
            this.timeout.on = setTimeout(function() {
                $('#socketOn').removeClass('active');
            }, 500);
        }
    },
    /**
     *  @function initSocket
     *  @param {JSObject} socket
     *  @param {Integer} i
     *  @description Initialisation d'une socket par compte, et définition des écouteurs d'événement.
     *  @returns {Void}
     **/
    initSocket: function(socket, i) {
        //  Stockage des identifiants du compte dans credentials.
        var credentials = {username: appUI.accounts.items[i].login, password: appUI.accounts.items[i].password};

        /**
         *  @event initSocket#connecting
         *  @description Tentative de connexion
         **/
        socket.on('connecting', function() {
            appUI.engine.ico.on();
            if (appUI.context === "home") {
                //  On modifie l'icône sur le contexte home pour signifier que le compte est en cours de connexion (gears).
                $('#' + appUI.accounts.items[i].id + "Btn .socketConnected i").removeClass('icon-unlink').removeClass('icon-link').addClass('icon-gears');
            }
        });

        /**
         *  @event initSocket#connect
         *  @description Connexion
         **/
        socket.on('connect', function() {
            var todo = appUI.storage.core.get('todo', appUI.accounts.items[i].id);
            if (todo === null || typeof todo === "undefined")
                appUI.engine.ico.on();
            if (!appUI.accountsSockets.containsKey(appUI.accounts.items[i].id)) {
                //  On associe un compte à une socket dans un dictionnaire à l'aide de leurs identifiants.
                appUI.accountsSockets.put(appUI.accounts.items[i].id, socket.socket.sessionid);
            }
            //  On modifie l'icône sur le contexte home pour signifier que le compte est connecté (link).
            if (appUI.context === "home") {
                $('#' + appUI.accounts.items[i].id + "Btn .socketConnected i").removeClass('icon-unlink').removeClass('icon-gears').addClass('icon-link');
                $('#' + appUI.accounts.items[i].id + "Btn .socketConnected .info").html('connect&eacute;');
            }
            appUI.engine.ico.emit();
            //  A la connexion, on s'authentifie auprès du serveur avec les credentials.
            socket.emit('auth', credentials);
        });

        /**
         *  @event initSocket#disconnect
         *  @description Déconnexion
         **/
        socket.on('disconnect', function() {
            appUI.engine.ico.on();
            if (appUI.accountsSockets.containsKey(appUI.accounts.items[i].id)) {
                //  On dissocie le compte de la socket qui lui était associée.
                appUI.accountsSockets.remove(appUI.accounts.items[i].id);
            }

            if (appUI.context === "home") {
                //  On modifie l'icône sur le contexte home pour signifier que le compte est déconnecté (unlink).
                $('#' + appUI.accounts.items[i].id + "Btn .socketConnected i").removeClass('icon-gears').removeClass('icon-link').addClass('icon-unlink');
                $('#' + appUI.accounts.items[i].id + "Btn .socketConnected .info").html('d&eacute;connect&eacute;');
            }
        });

        /**
         *  @event initSocket#connect_failed
         *  @description Erreur de connexion
         **/
        socket.on('connect_failed', function() {
            appUI.engine.ico.on();
            appUI.notify("Impossible de se connecter au serveur" + (appUI.accounts.items[i].server_url !== "" ? " : " + appUI.accounts.items[i].server_url : ""));
        });

        /**
         *  @event initSocket#reconnecting
         *  @description Tentative de reconnexion
         **/
        socket.on('reconnecting', function() {
            appUI.engine.ico.on();
            if (appUI.context === "home") {
                //  On modifie l'icône sur le contexte home pour signifier que le compte est en cours de reconnexion (gears).
                $('#' + appUI.accounts.items[i].id + "Btn .socketConnected i").removeClass('icon-unlink').removeClass('icon-link').addClass('icon-gears');
                $('#' + appUI.accounts.items[i].id + "Btn .socketConnected .info").html('connexion ... ');
            }
        });

        /**
         *  @event initSocket#reconnect
         *  @description Reconnexion
         **/
        socket.on('reconnect', function() {
            appUI.engine.ico.on();
            //            appUI.notify('reconnected');
            //  On associe le compte à la socket ayant permis la reconnexion.
            if (!appUI.accountsSockets.containsKey(appUI.accounts.items[i].id)) {
                appUI.accountsSockets.put(appUI.accounts.items[i].id, socket.socket.sessionid);
            }

            if (appUI.context === "home") {
                //  On modifie l'icône sur le contexte home pour signifier que le compte est reconnecté (link).
                $('#' + appUI.accounts.items[i].id + "Btn .socketConnected i").removeClass('icon-gears').removeClass('icon-unlink').addClass('icon-link');
            }
            appUI.engine.ico.emit();
            //  A la reconnexion, on s'authentifie de nouveau auprès du serveur.
            socket.emit('auth', credentials);
        });

        /**
         *  @event initSocket#reconnect_failed
         *  @description Erreur de reconnexion
         **/
        socket.on('reconnect_failed', function() {
            appUI.engine.ico.on();
            appUI.notify("Impossible de se reconnecter au serveur" + (appUI.accounts.items[i].server_url !== "" ? " : " + appUI.accounts.items[i].server_url : ""));
        });


        /**
         *  @event initSocket#error
         *  @description Signal d'erreur
         **/
        socket.on('error', function(data) {
            appUI.notify("Impossible de se connecter au serveur, veuillez vérifier l'URL du serveur.");
        });

        /**
         *  @event initSocket#auth_feedback
         *  @description Ecoute et gère le retour d'authentification du compte
         **/
        socket.on('auth feedback', function(data) {
            console.log(data);
            appUI.engine.ico.on();
            // Sur le retour d'authentification (réponse du serveur), on remplit les champs du compte concerné.
            appUI.accounts.items[i].userId = data.userId;
            appUI.accounts.items[i].userFirstname = data.firstname;
            appUI.accounts.items[i].userLastname = data.lastname;
            appUI.accounts.store();
            appUI.accounts.menu.draw();

            appUI.accountsSockets.each(function(accountId, socketId) {
                if (socketId === socket.socket.sessionid) {
                    //  On signale à l'utilisateur qu'il est connecté.
                    var account = appUI.accounts.get(accountId);
                    appUI.notify('[Mandat : ' + account.label + '] Vous êtes identifié(e) en tant que : ' + data.firstname + ' ' + data.lastname);

                    var tmpSeances = appUI.storage.core.get("seances", accountId);
                    var existingSeances = [];

                    //  Remplissage du tableau de séances existantes afin d'opérer un diff sur les données du serveur
                    if (tmpSeances !== null && tmpSeances.length > 0) {
                        for (var iS = 0, lnS = tmpSeances.length; iS < lnS; iS++) {
                            existingSeances.push(tmpSeances[iS].Seance.id);
                        }
                    }
                    appUI.engine.ico.emit();

                    //  On execute toutes les actions que l'utilisateur a fait en mode hors-ligne (todo propre à chaque compte).
                    var todo = appUI.storage.core.get('todo', accountId);
                    if (todo !== null) {

                        var j = 0;
                        while (todo.length !== 0) {
                            switch (todo[j].function) {
                                case "annot":
                                    socket.emit("annot", todo[j].args);
                                    break;
                                case "ar":
                                    socket.emit("ar", todo[j].args);
                                    break;
                                default:
                                    break;
                            }
                            todo.splice(j, 1);
                        }
                    } else {
                        todo = [];
                    }
                    appUI.storage.core.set('todo', todo, accountId);


                    console.log(accountId);

                    //  Une fois authentifié, on récupère toutes les nouvelles séances, pour cela, on opère un diff avec les séances qu'on a déjà.
                    socket.emit('sync seances', {
                        username: credentials.username,
                        password: credentials.password,
                        seances: existingSeances
                    });


                    //  Remplissage du tableau d'annotations existantes afin d'opérer un diff sur les données du serveur
                    var tmpAnnotations = appUI.storage.core.get("annotations", accountId);
                    var existingAnnotations = [];

                    if (tmpAnnotations !== null && tmpAnnotations.length > 0) {
                        for (var iA = 0, lnA = tmpAnnotations.length; iA < lnA; iA++) {
                            existingAnnotations.push(tmpAnnotations[iA].id);
                        }
                    }
                    appUI.engine.ico.emit();
                    //  Une fois authentifié, on récupère toutes les nouvelles annotations, pour cela, on opère un diff avec les séances qu'on a déjà.
                    socket.emit('sync annots', {
                        username: credentials.username,
                        password: credentials.password,
                        annotations: existingAnnotations
                    });

///////////////                    
                    console.log('ok');
                    console.log(credentials.username);
                    // On demande de resynchroniser les seances modifiés
                    socket.emit('reSync seances', {
                        username: credentials.username,
                        password: credentials.password,
                        seances: existingSeances
                    });

                }
            });
        });

        /**
         *  @event initSocket#logout
         *  @description Deconnexion : le serveur peut demander la déconnexion de l'utilisateur si ses identifiants sont erronés.
         *
         **/
        socket.on('logout', function() {
            appUI.accountsSockets.each(function(accountId, socketId) {
                if (socketId === socket.socket.sessionid) {
                    var account = appUI.accounts.get(accountId);
                    //  L'utilisateur est notifié de sa déconnexion
                    appUI.notify('[Mandat : ' + account.label + '] Vous avez été déconnecté(e) ou bien vos paramètres d\'identification sont erronés.');
                    appUI.engine.ico.on();
                    //            appUI.accounts.items[i].userId = '';
                    //            appUI.accounts.items[i].userFirstname = '';
                    //            appUI.accounts.items[i].userLastname = '';
                    socket.disconnect();
                }
            });
        });

        /**
         * Gets
         *
         */

        /**
         *  @event initSocket#message
         *  @description Ecoute et notifie le message reçu
         **/
        socket.on('message', function(data) {
            appUI.engine.ico.on();
            appUI.notify('message : ' + data, 'message');
        });

        /**
         *  @event initSocket#annot_feedback
         *  @description Ecoute et gère le retour d'une annotation précédemment créée, éditée, supprimée par le client
         **/
        socket.on('annot feedback', function(data) {
            var jsonData = data;
            var annot = jsonData.data;

            appUI.engine.ico.on();
            if (jsonData.action === 'create' && jsonData.success) {
                appUI.accountsSockets.each(function(accountId, socketId) {
                    if (socketId === socket.socket.sessionid) {
                        //  On récupère la liste des annotations du compte concerné
                        var account = appUI.accounts.get(accountId);
                        var allAnnots = appUI.storage.core.get('annotations', accountId);

                        var annotPos = -1;
                        for (var iAA = 0, lnAA = allAnnots.length; iAA < lnAA; iAA++) {
                            if (allAnnots[iAA].id === annot.annot_client_id) {
                                annotPos = iAA;
                            }
                        }
                        //  On remplace les annotations qui ont été éditées.
                        if (annotPos !== -1) {
                            allAnnots.splice(annotPos, 1);
                            allAnnots.push(annot);
                            // set allAnnots in localStorage, create a live object.
                            appUI.storage.core.set('annotations', allAnnots, accountId);
                            // refresh context
                            appUI.contexts[appUI.context].refreshData('annotations', account);
                        }

                        //  On reproduit le comportement au sein de l'objet live (celui que manipule annotations.js)
                        var liveAnnots = Annotation.getSauvegarde();
                        var annotPos = -1;

                        var iLAP = Annotation.getConfig().page_en_cours - 1;
                        for (var iLA = 0, lnLA = liveAnnots[iLAP].length; iLA < lnLA; iLA++) {
                            if (liveAnnots[iLAP][iLA].id === annot.annot_client_id) {
                                annotPos = iLA;
                            }
                        }
                        if (annotPos !== -1) {
                            var annotUpdated = liveAnnots[iLAP][annotPos];
                            annotUpdated.id = annot.id;

                            liveAnnots[iLAP].splice(annotPos, 1, annotUpdated);
                            //  On recrée la sauvegarde de l'objet live d'annotations.js et on actualise.
                            Annotation.setSauvegarde(liveAnnots);
                            Annotation.actualiser();
                        }
                    }
                });
            }
        });

        /**
         *  @event initSocket#seance_push
         *  @description Ecoute et gère la réception d'une seance envoyée par le serveur
         **/
        socket.on('seance push', function(data) {
            appUI.accountsSockets.each(function(accountId, socketId) {
                if (socketId === socket.socket.sessionid) {

                    var account = appUI.accounts.get(accountId);
                    //  On signale à l'utilisateur qu'il a reçu de nouvelles séances.
                    //      appUI.notify('[Mandat : ' + account.label + '] Vous êtes convoqué(e) à une ou plusieurs nouvelles séances.');
                    appUI.notify('[Mandat : ' + account.label + '] Vous êtes convoqué(e) à une ou plusieurs nouvelles séances.');
                    //  On crée le tableau pour faire la diff entre les séances enregistrées et les séances côté serveur.
                    var tmpSeances = appUI.storage.core.get("seances", accountId);
                    var existingSeances = [];

                    if (tmpSeances !== null && tmpSeances.length > 0) {
                        for (var iS = 0, ln = tmpSeances.length; iS < ln; iS++) {
                            existingSeances.push(tmpSeances[iS].Seance.id);
                        }
                    }
                    appUI.engine.ico.emit();

                    console.log(credentials.username);

                    //  On demande à recevoir les séances disponibles pour le client et non enregistrées.
                    socket.emit('sync seances', {
                        username: credentials.username,
                        password: credentials.password,
                        seances: existingSeances
                    });



                }
            });
        });




        /**
         *  @event initSocket#seance_repush
         *  @description Ecoute et gère la réception d'une modification de seance demandée par le serveur
         **/
        socket.on('seance repush', function(data) {
            appUI.accountsSockets.each(function(accountId, socketId) {
                if (socketId === socket.socket.sessionid) {

                    var account = appUI.accounts.get(accountId);
                    //  On signale à l'utilisateur qu'il a reçu de nouvelles séances.
                    //appUI.notify('[Mandat : ' + account.label + '] Vous êtes convoqué(e) à une ou plusieurs nouvelles séances.');

                    //  On crée le tableau pour faire la diff entre les séances enregistrées et les séances côté serveur.
                    var tmpSeances = appUI.storage.core.get("seances", accountId);
                    var existingSeances = [];

                    if (tmpSeances !== null && tmpSeances.length > 0) {
                        for (var iS = 0, ln = tmpSeances.length; iS < ln; iS++) {
                            existingSeances.push(tmpSeances[iS].Seance.id);
                        }
                    }
                    appUI.engine.ico.emit();


                    console.log(credentials.username);

                    console.log('push');
                    socket.emit('reSync seances', {
                        username: credentials.username,
                        password: credentials.password,
                        seances: existingSeances
                    });


                }
            });
        });








        /**
         *  @function getAllImages
         *  @param {JSObject} account
         *  @param {String} seanceId
         *  @param {Boolean} inverse
         *  @description Retourne l'ensemble des images à charger
         *  @returns {Array|engine_ws.prototype.initSocket.getAllImages.retval}
         **/
        function getAllImages(account, seanceId, inverse) {
            var retval = [];
            var dataSeances = appUI.storage.core.get('seances', account.id);

            //detection de toutes les images a charger
            for (var itS = 0, lnS = dataSeances.length; itS < lnS; itS++) {

                var action = false;
                if (typeof seanceId !== "undefined") {
                    if (typeof inverse !== "undefined" && inverse) {
                        if (dataSeances[itS].Seance.id !== seanceId) {
                            action = true;
                        }
                    } else {
                        if (dataSeances[itS].Seance.id === seanceId) {
                            action = true;
                        }
                    }
                } else {
                    action = true;
                }

                if (action) {
                    var convocations = dataSeances[itS].Convocation;
                    //Convocation
                    for (var itC = 0, lnC = convocations.length; itC < lnC; itC++) {
                        if (convocations[itC].user_id === account.userId) { //on choisit la séance dont la convocation est liée avec l utilisateur du compte courant

                            //fichier de la convocation
                            var convocationPages = dataSeances[itS].Document.Imgpage;
                            for (var itCP = 0, lnCP = convocationPages.length; itCP < lnCP; itCP++) {

                                if (retval.indexOf(convocationPages[itCP].id) === -1) {
                                    retval.push(convocationPages[itCP].id);
                                }
                            }

                            //fichiers des rapports
                            var projets = dataSeances[itS].Projet;
                            for (var itP = 0, lnP = projets.length; itP < lnP; itP++) {
                                var rapportPages = projets[itP].Document.Imgpage;
                                for (var itRP = 0, lnRP = rapportPages.length; itRP < lnRP; itRP++) {

                                    if (retval.indexOf(rapportPages[itRP].id) === -1) {
                                        retval.push(rapportPages[itRP].id);
                                    }
                                }
                            }
                        }
                    }
                }
            }
            return retval;
        }
        /**
         *  @function deleteSeance
         *  @param {JSObject} account
         *  @param {String} seanceId
         *  @description Supprime la séance désignée par "seanceId" ainsi que toutes ses dépendances (images et annotations)
         *  @returns {Void}
         **/
        function deleteSeance(account, seanceId) {

            var imgs = {
                allImages: getAllImages(account),
                allSeanceImages: getAllImages(account, seanceId),
                allOtherSeancesImages: getAllImages(account, seanceId, true)
            };

            var imgpagesToDelete = [];
            for (var z in imgs.allSeanceImages) {


                var present = false;
                for (var zz in imgs.allOtherSeancesImages) {
                    if (imgs.allSeanceImages[z] === imgs.allOtherSeancesImages[zz]) {
                        present = true;
                    }
                }

                if (!present) {
                    imgpagesToDelete.push(imgs.allSeanceImages[z]);
                }
            }

            //suppression des images indexeddb;
            for (var itID = 0, lnID = imgpagesToDelete.length; itID < lnID; itID++) {
                appUI.idb.get(imgpagesToDelete[itID], function(err, doc) {
                    if (err) {
                        if (appUI.debugMode) {
                            console.log(err);
                        }
                    } else {
                        appUI.idb.remove(doc, function(err, response) {
                            if (err) {
                                console.log("Erreur lors de la suppression de l'image");
                            }
                            else {
                                console.log("Image supprimé");
                            }
                        });
                    }
                });
            }


            //suppression des annotations
            var newDataAnnotations = [];
            var dataAnnotations = appUI.storage.core.get('annotations', account.id);
            for (var itA = 0, lnA = dataAnnotations.length; itA < lnA; itA++) {
                if (imgpagesToDelete.indexOf(dataAnnotations[itA].imgpage_id) === -1) {
                    newDataAnnotations.push(dataAnnotations[itA]);
                }
            }


            //suppression de la seance
            var newDataSeances = [];
            var dataSeances = appUI.storage.core.get('seances', account.id);
            //detection de toutes les images a charger
            for (var itS = 0, lnS = dataSeances.length; itS < lnS; itS++) {
                if (dataSeances[itS].Seance.id !== seanceId) {
                    newDataSeances.push(dataSeances[itS]);
                }
            }

            if (appUI.storage.core.set('annotations', newDataAnnotations, account.id) && appUI.storage.core.set('seances', newDataSeances, account.id)) {
                appUI.notify('Une séance a été supprimée.');
            }


            // supression de la liste des seances modifiées
            var modifiedSeances = appUI.storage.core.get('modifiedSeances', account.id);

            if (in_array(seanceId, modifiedSeances)) {
                console.log("present dans les modifs");
                unset(seanceId, modifiedSeances);
                appUI.storage.core.set('modifiedSeances', modifiedSeances, account.id);
            }




        }

        /**
         *  @event initSocket#seance_push_delete
         *  @param {JSObject} data
         *  @description Ecoute et gère la demande de suppression d'une seance de la part du serveur
         **/
        socket.on('seance push delete', function(data) {
            appUI.engine.ico.on();
            appUI.accountsSockets.each(function(accountId, socketId) {
                if (socketId === socket.socket.sessionid) {
                    var account = appUI.accounts.get(accountId);

                    var jsonData = data;
                    var seanceId = jsonData.data.seanceId;

                    deleteSeance(account, seanceId);

                    //appUI.contexts[appUI.context].refreshData('seances', account);
                    appUI.contexts[appUI.context].refreshData('seances', account);
                    appUI.contexts[appUI.context].refreshData('annotations', account);
                    appUI.contexts[appUI.context].refreshData('remove', account);

                }
            });
        });

        /**
         *  @event initSocket#annot_push
         *  @param {JSObject} data
         *  @description Ecoute et gère la réception d'une annotation envoyée par le serveur (intermédiaire d'un autre client)
         **/
        socket.on('annot push', function(data) {
            appUI.engine.ico.on();
            appUI.accountsSockets.each(function(accountId, socketId) {
                if (socketId === socket.socket.sessionid) {

                    var account = appUI.accounts.get(accountId);

                    //  On récupère les annotations du compte concerné
                    var allAnnots = appUI.storage.core.get('annotations', accountId);
                    if (allAnnots === null) {
                        allAnnots = {};
                    }

                    var jsonData = data;
                    var annot = jsonData.data;

                    var annotPos = -1;
                    for (var iAA = 0, lnAA = allAnnots.length; iAA < lnAA; iAA++) {
                        if (allAnnots[iAA].id === annot.id) {
                            annotPos = iAA;
                        }
                    }
                    /**
                     * FIXME : ici on force la valeur lu/non lu(booleen) à non lu(false) parceque le serveur envoi toujours lu(true). Faire en sorte que le serveur envoi la bonne information !
                     */
                    annot.lue = false;

                    //  On signale à l'utilisateur qu'il a reçu une annotation et s'il s'agit d'une édition, on remplace l'annotation existante en mémoire.
                    if (annotPos === -1) {
                        allAnnots.push(annot);
                        appUI.notify('[Mandat : ' + account.label + '] Une annotation vous a été partagée.');
                    } else {
                        allAnnots.splice(annotPos, 1, annot);
                    }

                    // set allAnnots in localStorage, create a live object.
                    appUI.storage.core.set('annotations', allAnnots, accountId);

                    // refresh context
                    appUI.contexts[appUI.context].refreshData('annotations', account);
                }
            });

        });

        /**
         *  @event initSocket#annot_push_delete
         *  @param {JSObject} data
         *  @description Ecoute et gère la demande de suppression d'une annotation de la part du serveur
         **/
        socket.on('annot push delete', function(data) {
            appUI.engine.ico.on();
            appUI.accountsSockets.each(function(accountId, socketId) {
                if (socketId === socket.socket.sessionid) {
                    var account = appUI.accounts.get(accountId);
                    var allAnnots = appUI.storage.core.get('annotations', accountId);
                    var jsonData = data;
                    var annotId = jsonData.data;

                    var annotPos = -1;
                    for (var iAA = 0, lnAA = allAnnots.length; iAA < lnAA; iAA++) {
                        if (allAnnots[iAA].id === annotId) {
                            annotPos = iAA;
                        }
                    }

                    //  On supprime l'annotation concerné par l'évènement de suppression.
                    if (annotPos !== -1) {
                        allAnnots.splice(annotPos, 1);
                        // set allAnnots in localStorage, create a live object.
                        appUI.storage.core.set('annotations', allAnnots, accountId);
                        // refresh context
                        appUI.contexts[appUI.context].refreshData('annotations', account);
                    }
                }
            });
        });

        /**
         *  @event initSocket#ar_feedback
         *  @param {JSObject} data
         *  @description Ecoute et gère le retour d'un accusé réception validé par le client
         **/
        socket.on('ar feedback', function(data) {
            appUI.engine.ico.on();
            appUI.accountsSockets.each(function(accountId, socketId) {
                if (socketId === socket.socket.sessionid) {
                    //  Le serveur confirme que la séance a été lue et on notifie le client que c'est bien le cas.
                    var account = appUI.accounts.get(accountId);
                    appUI.notify('[Mandat : ' + account.label + '] La convocation a bien été notée comme lue.');

                }

            });

        });


        /**
         * Syncs
         */

        /**
         *  @event initSocket#sync_seances_feedback
         *  @param {JSObject} data
         *  @description Ecoute et gère le retour d'une demande de synchronisation de seances de la part du client
         **/
        socket.on('sync seances feedback', function(data) {
            appUI.engine.ico.on();
            var jsonData = data;

            console.log('dans le sync seance feedback');

            appUI.nbPagesConvocTotales = 0;
            appUI.nbPagesProjetTotales = 0;

            appUI.accountsSockets.each(function(accountId, socketId) {
                if (socketId === socket.socket.sessionid) {
                    var account = appUI.accounts.get(accountId);

                    //suppression des seances supprimées (archivées)
                    for (var iSRem = 0, lnSRem = jsonData.data.toRemove.length; iSRem < lnSRem; iSRem++) {
                        deleteSeance(account, jsonData.data.toRemove[iSRem]);
                    }

                    console.log(accountId);

                    /**
                     * Enregistrement des seances envoyées par le serveur
                     */
                    var dataSeances = appUI.storage.core.get("seances", accountId);
                    if (dataSeances !== null) {
                        for (var i = 0, ln = jsonData.data.new.length; i < ln; i++) {
                            //  On vérifie si les séances reçus diffèrent des séances déjà enregistrées, si c'est le cas, on les enregistre aussi.
                            var exists = false;
                            for (var itDS = 0, lnDS = dataSeances.length; itDS < lnDS; itDS++) {
                                if (dataSeances[itDS].Seance.id === jsonData.data.new[i].Seance.id) {
                                    exists = true;
                                }
                            }

                            if (!exists) {
                                console.log(jsonData.data.new[i]);
                                dataSeances.push(jsonData.data.new[i]);
                            }
                        }
                    } else {
                        dataSeances = jsonData.data.new;
                    }

                    //enregistrement des seances
                    if (appUI.storage.core.set("seances", dataSeances, accountId)) {
                        console.log(dataSeances);
                        //Décompte du nombre de séances non lues.
                        var unreadSeances = 0;
                        for (var itUS = 0, lnUS = dataSeances.length; itUS < lnUS; itUS++) {
                            for (var itUSC = 0, lnUSC = dataSeances[itUS].Convocation.length; itUSC < lnUSC; itUSC++) {
                                if (dataSeances[itUS].Convocation[itUSC].user_id === appUI.accounts.get(accountId).userId) {
                                    if (!dataSeances[itUS].Convocation[itUSC].read) {
                                        unreadSeances++;
                                    }
                                }
                            }
                        }
                        if (unreadSeances > 0) {
                            appUI.notify('[Mandat : ' + appUI.accounts.get(accountId).label + '] Vous avez ' + unreadSeances + ' convocation' + (unreadSeances > 1 ? 's' : '') + ' non lue' + (unreadSeances > 1 ? 's' : ''));
                        }

                        //demande de rafraichissement du contexte courant
                        appUI.contexts[appUI.context].refreshData('seances', account);

                        /**
                         * Stockage des images
                         */
                        //  Detection de toutes les images a charger
                        for (var itS = 0, lnS = dataSeances.length; itS < lnS; itS++) {
                            var convocations = dataSeances[itS].Convocation;
                            //  Convocation
                            for (var itC = 0, lnC = convocations.length; itC < lnC; itC++) {
                                if (convocations[itC].user_id === account.userId) { //  On choisit la séance dont la convocation est liée avec l utilisateur du compte courant
                                    //fichier de la convocation
                                    var convocationPages = dataSeances[itS].Document.Imgpage;
                                    for (var itCP = 0, lnCP = convocationPages.length; itCP < lnCP; itCP++) {
                                        appUI.nbPagesConvocTotales++;

                                        //chargement des convocations auto meme en autoLoad = false;
                                        (function(account, convocationPages, itC, itCP, socket) {
                                            appUI.engine.loadImage(account, convocationPages[itCP], convocations[itC], socket);
                                        })(account, convocationPages, itC, itCP, socket);
                                    }

                                    //fichiers des rapports
                                    var projets = dataSeances[itS].Projet;
                                    for (var itP = 0, lnP = projets.length; itP < lnP; itP++) {
                                        var rapportPages = projets[itP].Document.Imgpage;
                                        for (var itRP = 0, lnRP = rapportPages.length; itRP < lnRP; itRP++) {
                                            appUI.nbPagesProjetTotales++;

                                            //chargement auto
                                            if (appUI.autoLoad) {
                                                (function(account, rapportPages, itP, itRP, socket) {
                                                    appUI.engine.loadImage(account, rapportPages[itRP], projets[itP], socket);
                                                })(account, rapportPages, itP, itRP, socket);
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            });
            //rechargement des informations concernant les images
            appUI.imgLoaded.initAllData();
            appUI.imgLoaded.checkAllImagesStatus();
        });








        socket.on('reSync seances feedback', function(data) {


            appUI.engine.ico.on();
            var jsonData = data;

            console.log('dans le resync feedback');

            appUI.nbPagesConvocTotales = 0;
            appUI.nbPagesProjetTotales = 0;

            appUI.accountsSockets.each(function(accountId, socketId) {
                if (socketId === socket.socket.sessionid) {
                    var account = appUI.accounts.get(accountId);


                    console.log(accountId);


                    // on recupère les seances presentes dans le localStorage
                    var dataSeances = appUI.storage.core.get("seances", accountId);

                    if (typeof dataSeances == "undefined" || dataSeances == null) {
                        dataSeances = [];
                    }

                    // on recupere les seances du server
                    var seancesServer = jsonData.data.new;


                    console.log(seancesServer);

                    // on liste les seances deja enregistrée et leurs revs

                    var listeLocalSeanceId = [];
                    var listeSeanceRevs = {};  // tableau associatif seanceId : rev
                    for (var i = 0, ln = dataSeances.length; i < ln; i++) {
                        listeLocalSeanceId.push(dataSeances[i].Seance.id);
                        listeSeanceRevs[dataSeances[i].Seance.id] = dataSeances[i].Seance.rev;
                    }



                    // on stocke les seances modifiée
                    var nbModif = 0;
                    var seanceChanged = [];

                    for (var i = 0, ln = seancesServer.length; i < ln; i++) {
                        if (in_array(seancesServer[i].Seance.id, listeLocalSeanceId)) {
                            // si l'id correspondant à la rev est diffferent
                            if (listeSeanceRevs[seancesServer[i].Seance.id] !== seancesServer[i].Seance.rev) {
                                seanceChanged.push(seancesServer[i].Seance.id);
                                nbModif += 1;
                            }
                        }
                    }

                    console.log(nbModif);

                    // si il existe des seances modifiées
                    if (listeLocalSeanceId.length > 0) {

                        //on recupere les seances modifiée deja stockée dans le localStorage


                        var modifiedSeance = appUI.storage.core.get("modifiedSeances", accountId);

                        if (typeof modifiedSeance == "undefined" || modifiedSeance == null) {
                            modifiedSeance = [];
                            console.log('undefined');
                        }

                        for (i = 0, ln = seanceChanged.length; i < ln; i++) {
                            if (!in_array(seanceChanged[i], modifiedSeance)) {
                                modifiedSeance.push(seanceChanged[i]);
                            }
                        }




                    


                        //on ajoute les les id des seances modifiés dans modifiedSeance dans le localStorage
                        appUI.storage.core.set("modifiedSeances", modifiedSeance, accountId);

                        // on acffiche la notification de modification de seance
                        appUI.notify('[Mandat : ' + account.label + '] ' + nbModif + ' séance(s) à(ont) été modifiée(s)');

                        // on suprime les seances correspondant à ce compte
                        localStorage.removeItem(accountId + "::seances");

                        if (socketId === socket.socket.sessionid) {
                            var account = appUI.accounts.get(accountId);


                            dataSeances = jsonData.data.new;


                            if (appUI.storage.core.set("seances", dataSeances, accountId)) {
                                console.log(dataSeances);

                                //demande de rafraichissement du contexte courant 
                                appUI.contexts[appUI.context].refreshData('seances', account);
                                appUI.contexts[appUI.context].refreshData('modification', account);
                                //Stockage des images

                                //  Detection de toutes les images a charger
                                for (var itS = 0, lnS = dataSeances.length; itS < lnS; itS++) {
                                    var convocations = dataSeances[itS].Convocation;
                                    //  Convocation
                                    for (var itC = 0, lnC = convocations.length; itC < lnC; itC++) {
                                        if (convocations[itC].user_id === account.userId) { //  On choisit la séance dont la convocation est liée avec l utilisateur du compte courant
                                            //fichier de la convocation
                                            var convocationPages = dataSeances[itS].Document.Imgpage;

                                            //fichiers des rapports
                                            var projets = dataSeances[itS].Projet;
                                            for (var itP = 0, lnP = projets.length; itP < lnP; itP++) {
                                                var rapportPages = projets[itP].Document.Imgpage;
                                                for (var itRP = 0, lnRP = rapportPages.length; itRP < lnRP; itRP++) {
                                                    appUI.nbPagesProjetTotales++;


                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            });
            //rechargement des informations concernant les images
            appUI.imgLoaded.initAllData();
            appUI.imgLoaded.checkAllImagesStatus();
        });










        /**
         *  @event initSocket#imgpage_feedback
         *  @param {JSObject} data
         *  @description Ecoute et gère le retour d'une demande d'image de la part du client
         **/
        socket.on('imgpage feedback', function(data) {
            appUI.engine.ico.on();
            var jsonData = data;
            appUI.idb.get(jsonData.data.id, function(err, doc) {
                if (err) {
                    if (err.status === 404) {
                        /*console.log("Image introuvable");*/
                        appUI.idb.put(
                                {
                                    data: jsonData.data.content,
                                    id: jsonData.data.id,
                                    name: jsonData.data.name
                                }, jsonData.data.id, function(err, response) {
                            if (err) {
                                console.log(err);
                            } else {
                                var kitkat = appUI.data_imgpages_id.get(jsonData.data.id);

                                var setLoad = appUI.imgLoaded.data[kitkat.accountId][kitkat.seanceId].convocation.doc.imgpages[kitkat.imgpageId];
                                if (typeof setLoad !== "undefined") {
                                    setLoad.loaded = true;
                                } else {
                                    setLoad = appUI.imgLoaded.data[kitkat.accountId][kitkat.seanceId].projets[kitkat.projetId].doc.imgpages[kitkat.imgpageId];
                                    if (typeof setLoad !== "undefined") {
                                        setLoad.loaded = true;
                                    } else if (appUI.debugMode) {
                                        appUI.notify('check image load error');
                                    }
                                }

                                appUI.contexts[appUI.context].refreshData('loadProgress', appUI.accounts.get(kitkat.accountId));
                            }
                        });
                    } else {
                        console.log(err);
                    }
                }
            });
        });

        /**
         *  @event initSocket#sync_annots_feedback
         *  @param {JSObject} data
         *  @description Ecoute et gère le retour d'une demande de synchronisation des annotations de la part du client
         **/
        socket.on('sync annots feedback', function(data) {
            appUI.engine.ico.on();

            jsonData = data;

            /**
             *
             *  Algorithme pour supprimer les duplicats par 25 - problème à corriger côté serveur
             */
            //            var i = 0;
            //            var j = 0;
            //            while (i < jsonData.data.length) {
            //                j = i + 1;
            //                while (j < jsonData.data.length) {
            //                    if (jsonData.data[i].id === jsonData.data[j].id) {
            //                        jsonData.data.splice(j, 25);
            //                    } else {
            //                        j++;
            //                    }
            //                }
            //                i++;
            //            }
            /**
             *  Fin Algo.
             */

            appUI.accountsSockets.each(function(accountId, socketId) {
                if (socketId === socket.socket.sessionid) {

                    var dataAnnotations = appUI.storage.core.get("annotations", accountId);
                    if (dataAnnotations !== null) {
                        for (var iA = 0, lnA = jsonData.data.length; iA < lnA; iA++) {
                            for (var jA = 0, lndA = dataAnnotations.length; jA < lndA; jA++) {
                                if (jsonData.data[iA].id === dataAnnotations[jA].id) {
                                    dataAnnotations.splice(jA, 1);
                                }
                            }
                            dataAnnotations.push(jsonData.data[iA]);
                        }
                    } else {
                        dataAnnotations = jsonData.data;
                    }

                    if (appUI.storage.core.set("annotations", dataAnnotations, accountId)) {
                        //notification interne
                        var account;
                        for (var i in appUI.accounts.items) {
                            if (appUI.accounts.items[i].id === accountId) {
                                account = appUI.accounts.items[i];
                            }
                        }
                        //  Demande de rafraichissement du contexte courant
                        appUI.contexts[appUI.context].refreshData('annotations', account);
                    }
                }
            });
        });

        /**
         *  @event initSocket#broadcast_notify
         *  @param {JSObject} data
         *  @description Ecoute et notifie le client
         **/
        socket.on('broadcast notify', function(data) {
            appUI.engine.ico.on();
            appUI.notify('broadcast notify : ' + data, 'broadcast notify');
        });

        var socketObject = {
            socket: socket,
            accountId: appUI.accounts.items[i].id
        };
        appUI.engine.sockets.push(socketObject);
    },
    /**
     *  @function init
     *  @description Initialisation d'engine_ws et mise en place des différents mécanismes de certification selon la plate-forme
     *  @returns {Void}
     **/
    init: function() {
        for (var i = 0, ln = appUI.accounts.items.length; i < ln; i++) {

            var url = 'https://' + appUI.accounts.items[i].server_url;
            if (appUI.cordova && typeof device !== 'undefined' && (device.platform === "Android" || device.platform === "iOS")) {

                appUI.accountsCert.put(appUI.accounts.items[i].id, 'certificat non valide');

                (function(i, url) {
                    //android
                    window.plugins.sslCertificateChecker.check(function(message) {
                        var socket = io.connect(url, {'force new connection': true});
                        appUI.engine.initSocket(socket, i);
                        appUI.cordovaCertSuccessCallback(message);

                        appUI.accountsCert.put(appUI.accounts.items[i].id, message);

                        if (appUI.context === "home") {
                            $('#' + appUI.accounts.items[i].id + "Btn .certCheck i").removeClass('icon-unlock').addClass('icon-lock');
                            $('#' + appUI.accounts.items[i].id + "Btn .certCheck .info").html(message);
                        }

                    }, appUI.cordovaCertErrorCallback, url);
                })(i, url);
            } else if (!appUI.cordova) {
                //web app
                var socket = io.connect(url, {'force new connection': true});
                (function(socket, i) {
                    appUI.engine.initSocket(socket, i);
                })(socket, i);
            }
        }
    },
    /**
     *  @function loadImage
     *  @param {JSObject} account
     *  @param {JSObject} imgpage
     *  @param {JSObject} projet
     *  @param {JSObject} socket
     *  @description Demande le téléchargement d'une image si elle n'est pas disponible dans la base de données locale
     *  @returns {Void}
     */
    loadImage: function(account, imgpage, projet, socket) {
        appUI.idb.get(imgpage.id, function(err, doc) {
            if (err) {
                if (err.status === 404) {
                    appUI.engine.ico.emit();
                    socket.emit('imgpage', {
                        username: account.login,
                        password: account.password,
                        imgpageId: imgpage.id
                    });
                    appUI.data_imgpages_id.put(imgpage.id, {accountId: account.id, seanceId: projet.seance_id, projetId: projet.id, documentId: imgpage.document_id, imgpageId: imgpage.id});
                } else {
                    console.log(err);
                }
            }
        });
    },
    /**
     *  @function loadSeanceImages
     *  @param {JSObject} account
     *  @param {String} seanceId
     *  @param {JSObject} socket
     *  @description Lance le téléchargement de toutes les images appartenant à une séance définie par "seanceId"
     *  @returns {Void}
     */
    loadSeanceImages: function(account, seanceId, socket) {
        var dataSeances = appUI.storage.core.get("seances", account.id);
        if (dataSeances !== null) {
            for (var itS = 0, lnS = dataSeances.length; itS < lnS; itS++) {
                if (dataSeances[itS].Seance.id === seanceId) {
                    //fichiers des rapports
                    var projets = dataSeances[itS].Projet;
                    for (var itP = 0, lnP = projets.length; itP < lnP; itP++) {
                        var rapportPages = projets[itP].Document.Imgpage;
                        for (var itRP = 0, lnRP = rapportPages.length; itRP < lnRP; itRP++) {
                            //chargement auto
                            (function(account, rapportPages, itP, itRP, socket) {
                                appUI.engine.loadImage(account, rapportPages[itRP], projets[itP], socket);
                            })(account, rapportPages, itP, itRP, socket);
                        }
                    }
                }
            }
        }
    },
    /**
     *  @function loadProjetImages
     *  @param {JSObject} account
     *  @param {String} seanceId
     *  @param {JSObject} socket
     *  @description Lance le téléchargement de toutes les images appartenant à un projet "projetId" au sein d'une séance "seanceId"
     *  @returns {Void}
     */
    loadProjetImages: function(account, seanceId, projetId, socket) {
        var dataSeances = appUI.storage.core.get("seances", account.id);
        if (dataSeances !== null) {
            for (var itS = 0, lnS = dataSeances.length; itS < lnS; itS++) {
                if (dataSeances[itS].Seance.id === seanceId) {
                    //fichiers des rapports
                    var projets = dataSeances[itS].Projet;
                    for (var itP = 0, lnP = projets.length; itP < lnP; itP++) {
                        if (projets[itP].id === projetId) {
                            var rapportPages = projets[itP].Document.Imgpage;
                            for (var itRP = 0, lnRP = rapportPages.length; itRP < lnRP; itRP++) {
                                //chargement auto
                                (function(account, rapportPages, itP, itRP, socket) {
                                    appUI.engine.loadImage(account, rapportPages[itRP], projets[itP], socket);
                                })(account, rapportPages, itP, itRP, socket);
                            }
                        }
                    }
                }
            }
        }
    }
};
